(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: MATYKA.PAS                                                     *)
(*  Obsah: ruzne matematicke funkce, veci pro praci s vektory, komplexnimi *)
(*         cisly a maticemi (od scitani az po inverzi)                     *)
(*  Autor: Mircosoft (http://mircosoft.ic.cz)                              *)
(*         M. Milda (prevody ciselnych soustav, obecna mocnina)            *)
(*         Laaca (Arcsin, Arccos)                                          *)
(*  Posledni uprava: 15.4.2008                                             *)
(*  Pro kompilaci: nic                                                     *)
(*  Pro spusteni: nic                                                      *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
unit matyka;
{$R-} {vypnuti kontroly rozsahu, nutne pro dynamicka pole}
{$I-} {vypnuti kontroly operaci se soubory (kontroluji se rucne)}
interface

(**************************** obecne funkce: ********************************)

function Preved(cislo:longint; zaklad,sirka:byte):string;
{Prevod cisla z desitkove soustavy na jinou o zakladu Zaklad. Sirka je pocet
cifer; kdyby vysel vysledek kratsi, doplni se na zacatek nuly.}
function NaDes(ret:string; zaklad:byte):longint;
{prevod cisla zadaneho v retezci Ret na desitkove cislo}
function pow(a,n:real):real;
{Obecna mocnina A na n-tou. Pozor, ze pri umocnovani zaporneho cisla
necelociselnym exponentem vraci nulu (coz je chyba).}
function tg(x:real):real;
{vraci hodnotu tg(x), s pojistkou proti deleni nulou (v takovem pripade vraci
hodnotu 0)}
function cotg(x:real):real;
{vraci hodnotu cotg(x), s pojistkou proti deleni nulou}
function sgn(x:real):shortint;
{Funkce signum x (znamenko x). Obcas se hodi.
 x>0 => sgn=1
 x=0 => sgn=0
 x<0 => sgn=-1}
function VzdalBodu(x1,y1,x2,y2:longint):word;
{Vraci vzdalenost dvou bodu v rovine.}
function VzdalBoduOdPrimky(x,y,a,b,c:real):real;
{Vraci vzdalenost bodu o souradnicich x,y od primky dane rovnici a*x+b*y+c=0.}
procedure VektorovySoucin(u1,u2,u3,v1,v2,v3:real; var r1,r2,r3:real);
{vektorovy soucin r := u x v}
procedure KvadRovnice(a,b,c:real; var x1,x2:real; var CoVyslo:byte);
{Spocita kvadratickou rovnici. a,b a c jsou koeficienty (rovnice je ve tvaru
a*x*x+b*x+c=0), x1 a x2 jsou vysledne koreny. V promenne CoVyslo se vraci typ
reseni:}
const krNemaReseni=0;     {a=0, b=0, c<>0 - nesplnitelna pro jakekoli x}
      krNekonecnoReseni=1;{a=0, b=0, c=0 - splnena automaticky pro jakekoli x}
      krLinearni=2;       {a=0 - neni tam x na druhou, takze je to obycejna
                      linearni rovnice s jednim resenim (vraceno v x1 i v x2)}
      krDvaRealne=3;      {D>0 - dva ruzne realne koreny x1 a x2}
      krDvojnasobny=4;    {D=0 - jeden dvojnasobny realny koren x1 = x2}
      krKomplexni=5;      {D<0 - dva komplexne sdruzene koreny
                                 x1+x2*i a x1-x2*i (i je komplexni jednotka)}
function orez(A,min,max:integer):integer;
{Pokud je A mimo interval min..max, zvetsi se nebo zmensi tak, aby se do
nej veslo, funkce vraci vyslednou hodnotu.}
function rychlySin(x:integer):integer;
function rychlyCos(x:integer):integer;
{Vraci sin a cos podle tabulky (const). X je ve stupnich, vysledek je potreba
jeste vydelit 10000. O dost rychlejsi nez standardni sin a cos.}
function VPoli(x,y,x1,y1,x2,y2:integer):boolean;
{true, pokud bod x,y lezi uvnitr daneho obdelniku
 x1,y1 - levy horni roh toho obdelniku
 x2,y2 - pravy dolni roh}
function ArcSin(x:real):real;
function ArcCos(x:real):real;
{inverze k sin a cos, vysledek je v radianech}
function VetsiZ(a,b:integer):integer;
function MensiZ(a,b:integer):integer;
{Vraci vetsi nebo mensi cislo z a,b. Primitivni vec, ale obcas se hodi.}
function UhelVektoru(u1,u2,v1,v2:integer):real;
{vraci uhel mezi dvema rovinnymi vektory, v radianech (nevyzkouseno, ale mela
by fungovat)}
function UhelPruvodice(x,y:integer):real;
{Vraci uhel (v radianech), ktery svira pruvodic bodu x,y s kladnou poloosou x.
Uhel je bezpecne rozlisen pro vsechny 4 kvadranty. V prvnich trech je kladny
(0..1.5*pi), ve ctvrtem zaporny (0..-pi/2). Pokud jsou x i y nulove, vraci se
uhel pi/2 (deleni nulou nevznika).
   ^
   |           /
  y+. . . . ./
   |       / .
   |     /   .
   |   / \   .
   | /uhel|  .
  -|---------+---->
  O          x               }
function OtocByte(cislo:byte):byte;
{zrcadlove otoci dane cislo - nejnizsi bity prohodi s nejvyssimi:
10110000b -> 00001101b}
function OtocWord(cislo:word):word;
{totez, ale pro word}
procedure RozvinPlast(md,vd,h:real; var mr,vr,alfa:real);
{Rozvine zadany komoly kuzel do roviny. Kdyz vysledek narysujete na papir,
vystrihnete, vytvarujete a slepite, vyjde presne to teleso, ktere jste zadali.
Vstupy:
 md - prumer mensi podstavy pozadovaneho kuzele; pokud date 0, nebude to
      komoly, ale obycejny kuzel
 vd - prumer vetsi podstavy; pokud bude stejne velky jako md, nebude to kuzel,
      ale valec - pak se ale zmeni vyznam vystupu, viz dale
 h - pozadovana vyska kuzele
Vystupy (rozmery vysece mezikruzi):
 mr - maly (vnitrni) polomer rozvinuteho plaste, pri md=0 bude 0 a nebude to
      vysec mezikruzi, ale kruhu
 vr - velky polomer rozvinuteho plaste
 alfa - uhel rozvinuteho plaste v radianech
Pokud jste zadali md=vd, bude mit rozvinuty plast tvar obdelniku a vystupni
hodnoty znamenaji toto:
 mr - sirka plaste (obvod podstav)
 vr - vyska plaste
 alfa - bude 0 }
function Interpol(x1,y1,x2,y2,x:real):real;
{Linearni interpolace. Pouziti (napr.): mame tabulku s hodnotami x a y. Pak
mame nejake x a chceme k nemu v te tabulce najit y. Jenze tohle x v tabulce
jako obvykle neni, trefili jsme se nekam mezi nejake dve hodnoty. Takze:
 x1 - x vlevo od te nasi hodnoty
 y1 - y pro toto x1
 x2 - x vpravo od nasi hodnoty
 y2 - y pro toto x2
 x - nase pozadovana xova hodnota
Funkce vrati hledanou hodnotu y odpovidajici zadanemu x.
Vypocet probiha tak, ze se mezi dva sousedni body prolozi primka (je to jen
priblizne, ale obvykle dostacujici):
            |       |
   |       y|  __..-+y2
   |   __..-+~~     |
 y1+-~~     |       |
   |        |       |
  -+--------+-------+--
   x1       x       x2     }
function DivUp(co,cim:longint):longint;
{celociselne deleni se zaokrouhlenim nahoru}


(*************************** komplexni cisla: *******************************)

type KomplexniCislo = record
                      _R,{absolutni hodnota}
                      _fi:real;{uhel cili faze}
                      end;
{Komplexni cislo je ulozeno v goniometrickem tvaru:
cislo=R*(cos(fi)+i*sin(fi))  neboli podle Eulera:  cislo=R*exp(i*fi), a to
proto, ze je jednodussi tento tvar prevest na algebraicky (cislo=a+b*i) nez
z algebraickeho cucat goniometricky}

procedure ZadejKCAB(var KC:KomplexniCislo; a,b:real);
{slouzi pro zadani komplexniho cisla v algebraickem tvaru}
procedure ZadejKCRFi(var KC:KomplexniCislo; R,fi:real);
{pro zadani komp. cisla v goniometrickem tvaru}
function KCRe(KC:KomplexniCislo):real;
{vraci realnou cast daneho kompl. cisla}
function KCIm(KC:KomplexniCislo):real;
{vraci imaginarni cast daneho kompl. cisla}
procedure SectiKC(co,sCim:KomplexniCislo; var vysledek:KomplexniCislo);
{secte dve kompl. cisla}
procedure OdectiKC(OdCeho,Co:KomplexniCislo; var vysledek:KomplexniCislo);
{odecte komp. cisla}
procedure NasobKC(Co,Cim:KomplexniCislo; var vysledek:KomplexniCislo);
{vynasobi dve komp. cisla}
procedure VydelKC(Co,Cim:KomplexniCislo; var vysledek:KomplexniCislo);
{vydeli dve k. cisla}
procedure KCxRC(var Co:KomplexniCislo; cim:real);
{vynasobi komplexni cislo realnym}


(***************** remake MATLABu :-) (maticove operace): *******************)

type TypPrvkuMatice=real; {je mozne zmenit na libovolny realny typ dle potreby}
     PolePrvkuMatice=array[0..10] of TypPrvkuMatice; {pomocny typ - sablona pro ukazatel.
      Na velikosti nezalezi, pole bude dynamicke. Prvky v tomto jednorozmernem
      poli jsou usporadany za sebou, radek po radku, a spolu s udaji o sirce
      a vysce to cele muze fungovat jako dvojrozmerna matice.}
     UkNaPPM=^PolePrvkuMatice;

     Matice = record {uzivatelsky typ}
              _vyska,_sirka:word; {rozmery matice}
              _data:uknappm;      {ukazatel na vlastni matici - pole cisel}
              end;

var MatResult:byte;{nastavuje se automaticky po kazde maticove operaci:}
const matOK=0;              {0 - zadna chyba, vse je v poradku               }
      matMaloPameti=1;      {1 - nedostatek pameti pro vytvoreni matice
                                 nebo je matice vetsi nez 64 KB              }
      matNesediRozmery=2;   {2 - nesouhlasi rozmery matic nebo vkladame prvek
                                 na misto mimo matici                        }
      matChybaVZadani=3;    {3 - chyba v ciselnem retezci pri zadavani matice}
      matZdrojNeexistuje=4; {4 - matice, se kterou se snazime neco provest,
                                 neexistuje                                  }
      matKonecZadani=5;     {5 - pri nacitani matice ze zadaneho retezce byl
                                 dosazen konec retezce, ale jeste nebyly
                                 nacteny vsechny prvky matice}
      matSingularni=6;      {6 - matice je singularni, a proto s ni nejde ta
                                 operace, kterou s ni prave provadime        }
      matIOchyba=7;         {7 - chyba pri operaci se souborem}

 { Vsechny matice pojmenovane Vysledek vytvari prislusna procedura
 automaticky, neni proto treba se o jejich hodnotu pred zavolanim te procedury
 starat. V pripade chyby je obvykle Vysledek neexistujici matice,
 tj. _data=nil.
  Pozor! Nedavejte zdrojovou matici stejnou jako cilovou, protoze dynamicka
 data nefunguji jako lokalni promenna volana hodnotou, ale mohla by byt jeste
 pred dokoncenim procedury prepsana a vysel by chybny vysledek!
  Pokud neni neco jasne z teoretickeho hlediska, podivejte se na implementaci
 procedur; definice a vysvetleni maticovych operaci jsou v komentarich na
 konci procedur.
  Veskere indexy v parametrech procedur jsou pocitany od 1 (tj. levy horni roh
 matice ma index 1,1, pravy dolni pak _vyska,_sirka). Prvni index znamena
 cislo radku, druhy cislo sloupce.
  Vsechny procedury krome Initm nastavuji MatResult.}

procedure InitM(var M:matice);
{Vynuluje _sirku, _vysku a ukazatel _data. Je nutne s ni osetrit vsechny
maticove promenne pred prvnim pouzitim, hlavne ty lokalne definovane.
Nepouzivat na jiz alokovane matice!}
function VytvorM(var M:matice; vyska,sirka:word):boolean;
{Vytvori matici danych rozmeru (getmem), hodnoty prvku necha nedefinovane.
Pokud jiz ta matice existuje, zkontroluji se jeji rozmery a pokud jsou stejne
jako pozadovane, nic se uz nedela. Pokud jsou jine, matice se zrusi a vytvori
se nova o spravnych rozmerech. Maximalni velikost matice je obvyklych 64 KB.
Pokud se zada vyska nebo sirka 0, s matici se nic delat nebude (hodi se v
nekterych dalsich procedurach).}
procedure ZrusM(var M:matice);
{pokud matice existuje, vymaze ji, nastavi sirku a vysku na 0 a _data na nil}
procedure ZadejPrvekM(var M:matice; i,j:word; hodnota:TypPrvkuMatice);
{Umoznuje primo zadat prvek na i-tem radku a j-tem sloupci matice M.
Matice M musi byt jiz vytvorena.}
procedure ZadejM(var M:matice; Data:string);
{Umoznuje zadat celou matici. Jednotliva cisla v retezci Data oddelujte
mezerami (je jedno kolika), carka i tecka se berou jako desetinna carka.
Jednotlive radky oddelujte strednikem. Matice M bude podle zadaneho retezce
vytvorena.
Priklad: data='3 -87,7 98.77  7E-2;1 0 -5 5,5'
  vyjde M = 3  -87.7  98.77  0.07
            1   0     -5     5.5
Hexadecimalni zapis ($...) nefunguje.}
procedure ZadejRadekM(var M:matice; i:word; data:string);
{Umozni zadat i-ty radek matice. M musi existovat a mit spravne rozmery.
Tak trochu low-level, pouziti hlavne kdyby byla matice tak velka, ze by se
jeji zapis nevesel do stringu a nestacila by procedura ZadejM.}
procedure VyplnM(var M:matice; vyska,sirka:word; hodnota:TypPrvkuMatice);
{Vyplni celou matici danou hodnotou. Pokud matice neexistuje, bude vytvorena.
Pokud jiz existuje a chcete jeji rozmery zachovat, je dobre zadat sirku a
vysku 0 (viz proceduru VytvorM).}
procedure JednotkovaM(var M:matice; vyska,sirka:word);
{Z matice M udela jednotkovou matici, tj. na hlavni diagonale bude mit
jednicky a vsude jinde nuly. O zadavani rozmeru plati totez jako u VyplnM.
Pozn. pro ty, kteri to jeste ve skole nebrali: hlavni diagonala jsou ty prvky,
ktere maji stejne cislo radku a sloupce. Zacina v levem hornim rohu matice a
pokracuje doprava dolu.}
procedure TranspM(M:matice; var Vysledek:matice);
{Do matice Vysledek ulozi transponovanou matici M, tj. matici zrcadlove
otocenou kolem hlavni diagonaly.}
procedure SectiM(M1,M2:matice; var vysledek:matice);
{secte matice M1 a M2 (musi byt stejne velke)}
procedure OdectiM(M1,M2:matice; var vysledek:matice);
{od M1 odecte M2 (musi byt stejne velke)}
procedure MinusM(var M:matice);
{otoci znamenko u kazdeho prvku matice M}
procedure NasobCislemM(var M:matice; Cislo:typprvkumatice);
{kazdy prvek matice M vynasobi danym Cislem}
procedure NasobM(M1,M2:matice; var vysledek:matice);
{maticove vynasobi M1*M2 (sirka M1 se musi rovnat vysce M2)}
function PrvekM(var M:matice; i,j:word):typprvkumatice;
{vrati hodnotu prvku matice v i-tem radku a j-tem sloupci}
procedure RadekM(M:matice; i:word; var vysledek:matice);
{ve Vysledku vrati i-ty radek matice M}
procedure SloupecM(M:matice; j:word; var vysledek:matice);
{ve Vysledku vrati j-ty sloupec matice M}
procedure SubM(M:matice; odi,odj,doi,doj:word; var vysledek:matice);
{ve Vysledku vrati submatici z matice M s levym hornim rohem na pozici odi,odj
a s pravym dolnim rohem na pozici doi,doj}
procedure SpojM(M1,M2:matice; pod_sebe:boolean; var vysledek:matice);
{Ve Vysledku vraci spojene matice M1 a M2:
 pod_sebe - true: M1 a pod ni pripojena M2 (obe musi byt stejne siroke)
            false: M1 a zprava k ni pripojena M2 (obe musi byt stejne vysoke)}
const mPodSebe=true;
      mVedleSebe=false;    {pro prehlednejsi zapis}
procedure ProhodRadkyM(var M:matice; Ktery,sKterym:word);
{v matici M prohodi radky s danymi indexy}
procedure KopirujM(Odkud:matice; var Kam:matice);
{kopiruje matici Odkud do matice Kam (Kam se automaticky vytvori)}
procedure TrojuhM(M:matice; var vysledek:matice; var determinant:typprvkumatice);
{Ve Vysledku vrati matici M upravenou na trojuhelnikovou, a to specialne tak,
ze na hlavni diagonale jsou same jednicky (a pod ni nuly). Determinant vypadne
jako vedlejsi produkt, delejte si s nim co chcete. Procedura funguje i pro
obdelnikovou matici, ale determinant je potom nesmyslny.}
function DetM(M:matice):typprvkumatice;
{vraci determinant matice M (M musi byt ctvercova)}
procedure InvM(M:matice; var vysledek:matice);
{Ve Vysledku vrati matici inverzni k matici M (inverzni matice k M je takova
matice, kterou kdyz vynasobite M, vyjde vam jednotkova matice). M musi byt
ctvercova.}
procedure VypisM(M:matice; jmeno:string);
{Vypise matici M s prvky zaokrouhlenymi na 2 desetinna mista, zadne osetreni
prilis dlouhych radku apod., jen pro textovy rezim (writeln). Jmeno se zobrazi
pri vypisu. Priklad:
 napiseme:  vypism(A,'matice A');
 na obrazovce se objevi (napr.):  matice A =
                                      5.00   -4.00
                                      0.00   12.05  }
procedure NactiM(var M:matice; soubor:string);
{Nacte matici z textoveho souboru. Format souboru:
 vyska
 sirka
 jednotlive prvky matice razene po radcich: prvni je 1,1, druhy je 1,2 atd.
Nezalezi na poctu mezer nebo odradkovani mezi cisly (cte se procedurou Read).
Komentare a podobne neciselne udaje se smi psat az na konci souboru za matici.}
procedure UlozM(M:matice; soubor:string);
{ulozi matici do textoveho souboru}

implementation

(******************************* matice: ************************************)

procedure InitM(var M:matice);
Begin
with M do begin _sirka:=0; _vyska:=0; _data:=nil; end;
End;{initm}

procedure ZrusM(var M:matice);
Begin
with M do if _data=nil then matresult:=matzdrojneexistuje {uz je zrusena nebo jeste nebyla vytvorena -> neni co resit}
                       else begin {existuje, zrusime ji}
                            freemem(_data,_vyska*_sirka*sizeof(TypPrvkuMatice));
                            _sirka:=0; _vyska:=0; _data:=nil;
                            matresult:=0;
                            end;
End;{zrusm}

function VytvorM(var M:matice; vyska,sirka:word):boolean;
var velikost:longint;
Begin
if (vyska=0)or(sirka=0)
  then if M._data=nil then begin{bylo zadano "nemenit rozmery" a matice neexistovala, takze neexistuje i nadale}
                           matresult:=matzdrojneexistuje;
                           vytvorm:=false;
                           end
                      else begin{bylo zadano "nemenit rozmery" a matice existuje, takze neni co resit}
                           matresult:=0;
                           vytvorm:=true;
                           end
  else with M do begin {byly zadany nejake rozmery}
                 if _data<>nil then if (_sirka=sirka)and(_vyska=vyska) then begin{matice uz existuje a je spravne velka}
                                                                            matresult:=0;
                                                                            vytvorm:=true;
                                                                            exit;
                                                                            end
                                                                       else {matice existuje, ale neni spravne velka:}
                                                                           freemem(_data,_vyska*_sirka*sizeof(TypPrvkuMatice));
                 velikost:=vyska*sirka*sizeof(TypPrvkuMatice);{kolik pameti nova matice zabere}
                 if (velikost>maxavail)or(velikost>$FFF0{jestli mam tu max. velikost spatne, tak ji prosim opravte})
                   then begin{je moc velka}
                        _vyska:=0; _sirka:=0; _data:=nil;
                        vytvorm:=false;
                        matresult:=matmalopameti;
                        end
                   else begin{dobry, vytvorime ji}
                        getmem(_data,velikost);
                        _vyska:=vyska; _sirka:=sirka;
                        vytvorm:=true;
                        matresult:=0;
                        end;
                 end;{else, with}
End;{vytvorm}

procedure ZadejPrvekM(var M:matice; i,j:word; hodnota:TypPrvkuMatice);
Begin
with M do if _data=nil then matresult:=matzdrojneexistuje{neni do ceho vkladat}
 else if (j>_sirka)or(i>_vyska)or(i<1)or(j<1) then matresult:=matnesedirozmery{indexy vychazeji mimo matici}
  else begin{OK}
       _data^[(i-1)*_sirka+(j-1)]:=hodnota;{Jak se dostat k prvku 2D matice
       ulozene v 1D poli: cislo radku (i) pocitane od nuly vynasobime sirkou
       matice, dostaneme index prvniho prvku na i-tem radku. Pak jeste
       pricteme cislo sloupce (j, opet pocitane od nuly) a je to.}
       matresult:=0;
       end;
End;{zadejprvekm}

procedure VyplnM(var M:matice; vyska,sirka:word; hodnota:TypPrvkuMatice);
var i:word;
Begin
if vytvorm(M,vyska,sirka) then
 with M do for i:=0 to _vyska*_sirka-1 do _data^[i]:=hodnota;
 {staci jednoduchy cyklus, protoze _data^ je jednorozmerne pole}
End;{vyplnm}

procedure JednotkovaM(var M:matice; vyska,sirka:word);
var i,j:word;
Begin
if vytvorm(M,vyska,sirka) then
 with M do for i:=0 to _vyska-1 do
            for j:=0 to _sirka-1 do if i=j then _data^[i*_sirka+j]:=1
                                           else _data^[i*_sirka+j]:=0;
{jednotkova matice vypada napr. takhle: 1 0 0 0
                                        0 1 0 0
                                        0 0 1 0
                                        0 0 0 1
Jakakoli matice maticove vynasobena jednotkovou matici se nezmeni.}
End;{jednotkovam}

procedure TranspM(M:matice; var vysledek:matice);
var i,j:word;
Begin
if vytvorm(vysledek,M._sirka,M._vyska) then
 for i:=0 to M._vyska-1 do
  for j:=0 to M._sirka-1 do vysledek._data^[j*vysledek._sirka+i]:=M._data^[i*M._sirka+j];
   {transpozice = prohozeni radku a sloupcu (prvek z i,j jde na j,i)}
End;{transpm}

procedure SectiM(M1,M2:matice;var vysledek:matice);
var i:word;
Begin
if (M1._sirka<>M2._sirka)or(M1._vyska<>M2._vyska)
 then begin{matice nejsou stejne velke, takze je nelze scitat}
      zrusm(vysledek);
      matresult:=matnesedirozmery;
      end
 else if vytvorm(vysledek,M1._vyska,M1._sirka) then
        for i:=0 to M1._sirka*M1._vyska-1 do vysledek._data^[i]:=M1._data^[i]+M2._data^[i];
 {soucet matic = matice s prislusnymi prvky sectenymi:
  vysledek[1,1]=M1[1,1]+M2[1,1]; vysledek[1,2]=M1[1,2]+M2[1,2] atd.}
End;{sectim}

procedure OdectiM(M1,M2:matice;var vysledek:matice);
var i:word;
Begin
{odcitani je prakticky to same, jenom je tam - misto +}
if (M1._sirka<>M2._sirka)or(M1._vyska<>M2._vyska)
 then begin
      zrusm(vysledek);
      matresult:=matnesedirozmery;
      end
 else if vytvorm(vysledek,M1._vyska,M1._sirka) then
        for i:=0 to M1._sirka*M1._vyska-1 do vysledek._data^[i]:=M1._data^[i]-M2._data^[i];
End;{odectim}

procedure NasobCislemM(var M:matice; Cislo:typprvkumatice);
var i:word;
Begin
if M._data=nil then matresult:=matzdrojneexistuje
               else begin
                    with M do for i:=0 to _vyska*_sirka-1 do _data^[i]:=_data^[i]*cislo;
                    matresult:=matOK;
                    end;
 {nasobit matici cislem znamena nasobit tim cislem kazdy jeji prvek}
End;{nasobcislemm}

procedure MinusM(var M:matice);
var i:word;
Begin
if M._data=nil then matresult:=matzdrojneexistuje
               else begin
                    with M do for i:=0 to _vyska*_sirka-1 do _data^[i]:=-_data^[i];
                    matresult:=matOK;
                    end;
 {v podstate nasobeni cislem -1}
End;{minusm}

procedure NasobM(M1,M2:matice; var vysledek:matice);
var i,j,k:word;
    pom:typprvkumatice;
Begin
if M1._sirka<>M2._vyska then begin
                             zrusm(vysledek);
                             matresult:=matnesedirozmery;
                             exit;
                             end;
if vytvorm(vysledek,M1._vyska,M2._sirka) then
  for i:=0 to vysledek._vyska-1 do
   for j:=0 to vysledek._sirka-1 do
     begin
     pom:=0;
     for k:=0 to M1._sirka-1 do pom:=pom+M1._data^[i*M1._sirka+k]*M2._data^[k*M2._sirka+j];
     with vysledek do _data^[i*_sirka+j]:=pom;
     end;
 {vysledek maticoveho soucinu je matice stejne vysoka jako ta prvni, stejne
 siroka jako druha a kde kazdy prvek na pozici i,j je skalarnim soucinem
 iteho radku prvni matice a jteho sloupce druhe matice.
 Skalarni soucin vektoru vypada napr. takhle: (x,y,z)*(X,Y,Z)=x*X+y*Y+z*Z.
 Maticovy soucin neni obecne komutativni.}
End;{nasobm}

function PrvekM(var M:matice; i,j:word):typprvkumatice;
Begin
with M do if _data=nil then matresult:=matzdrojneexistuje
 else if (i<1)or(j<1)or(i>_vyska)or(j>_sirka) then matresult:=matnesedirozmery
  else prvekm:=_data^[(i-1)*_sirka+j-1];
End;{prvekm}

procedure RadekM(M:matice;i:word; var vysledek:matice);
Begin
if M._data=nil then begin
                    matresult:=matzdrojneexistuje;
                    zrusm(vysledek);
                    end
               else if vytvorm(vysledek,1,M._sirka) then move(M._data^[(i-1)*M._sirka],{odkud}
                                                              vysledek._data^[0],{kam}
                                                              vysledek._sirka*sizeof(typprvkumatice));{kolik bytu}
                                                         {move je rychlejsi nez presouvat prvek po prvku}
End;{radekm}

procedure SloupecM(M:matice;j:word; var vysledek:matice);
var i:word;
Begin
if M._data=nil then begin
                    matresult:=matzdrojneexistuje;
                    zrusm(vysledek);
                    end
               else if not vytvorm(vysledek,M._vyska,1) then begin
                                                             dec(j);
                                                          for i:=0 to M._vyska-1 do vysledek._data^[i]:=M._data^[i*M._sirka+j];
                                                             {tady uz move nejde}
                                                             end;
End;{sloupecm}

procedure SubM(M:matice;odi,odj,doi,doj:word; var vysledek:matice);
var sirka,vyska,i,j:word;
Begin
if M._data=nil
 then begin
      matresult:=matzdrojneexistuje;
      zrusm(vysledek);
      end
 else begin
      vyska:=doi-odi+1; sirka:=doj-odj+1;
      if vytvorm(vysledek,vyska,sirka) then
        begin
        dec(odi); dec(odj);
        for i:=0 to vyska-1 do
         for j:=0 to sirka-1 do vysledek._data^[i*vysledek._sirka+j]:=M._data^[(odi+i)*M._sirka+odj+j];
        end;
      end;
End;{subm}

procedure SpojM(M1,M2:matice; Pod_Sebe:boolean; var vysledek:matice);
var i,pocet:word;
Begin
if (M1._data=nil)and(M2._data=nil) then begin{neni co spojovat}
                                        matresult:=matzdrojneexistuje;
                                        zrusm(vysledek);
                                        exit;
                                        end;
if M1._data=nil then begin   {matice, ke ktere pripojime neexistujici matici, se nezmeni}
                     kopirujm(M2,vysledek);
                     exit;
                     end;
if M2._data=nil then begin
                     kopirujm(M1,vysledek);
                     exit;
                     end;
if pod_sebe then
 begin
 if M1._sirka<>M2._sirka then matresult:=matnesedirozmery;
 if (matresult<>0) or not vytvorm(vysledek,M1._vyska+M2._vyska,M1._sirka) then zrusm(vysledek)
  else begin
       with M1 do pocet:=_sirka*_vyska;
       {prekopirovani cele prvni matice:}
       move(M1._data^[0],vysledek._data^[0],pocet*sizeof(typprvkumatice));
       {prekopirovani druhe matice pod prvni:}
       move(M2._data^[0],vysledek._data^[pocet],M2._sirka*M2._vyska*sizeof(typprvkumatice));
       end;
 end
else{vedle sebe:}
 if M1._vyska<>M2._vyska then matresult:=matnesedirozmery;
 if (matresult<>0) or not vytvorm(vysledek,M1._vyska,M1._sirka+M2._sirka) then zrusm(vysledek)
  else for i:=0 to M1._vyska-1 do
   begin {pro kazdy radek vysledku:}
   {radek z M1:}
   move(M1._data^[i*M1._sirka],vysledek._data^[i*vysledek._sirka],M1._sirka*sizeof(typprvkumatice));
   {za nej radek z M2:}
   move(M2._data^[i*M2._sirka],vysledek._data^[i*vysledek._sirka+M1._sirka],M2._sirka*sizeof(typprvkumatice));
   end;
End;{spojm}

const cifry=['0'..'9','-','.',',','e','E'];{znaky, ze kterych se skladaji cisla}

procedure ZadejRadekM(var M:matice; i:word; data:string);
var j:byte;
    cislo:typprvkumatice;
    pozice:byte;
    slovo:string;
    kod:integer;
Begin
if data='' then matresult:=matkoneczadani{nic nebylo zadano}
 else if(i>M._vyska)or(i<1) then matresult:=matnesedirozmery{nesmyslny index radku}
  else if M._data=nil then matresult:=matzdrojneexistuje{matice neexistuje, neni kam ukladat}
   else begin
        dec(i);{kvuli pocitani od nuly}
        pozice:=1;{zacatek od prvniho znaku retezce}
        matresult:=0;{jestli vsechno dobre dopadne, ta nula tu zustane}
        with M do for j:=0 to _sirka-1 do {pro kazdy prvek radku}
         begin
         slovo:='';
         while not(data[pozice] in cifry) do inc(pozice);{preskoceni mezer pred cislem}
         if pozice>byte(data[0]) then begin{jsme za koncem retezce, coz je spatne}
                                      matresult:=matkoneczadani;
                                      exit;
                                      end;
         while (pozice<=byte(data[0]))and(data[pozice] in cifry) do
           begin {nacteni cisla}
           slovo:=slovo+data[pozice];
           inc(pozice);
           end;
         val(slovo,cislo,kod); {prevod retezce Slovo na cislo}
         if kod=0 then _data^[i*_sirka+j]:=cislo {OK}
                  else matresult:=matchybavzadani; {prevod se nezdaril}
         end;{with}
        end;{else}
End;{zadejradekm}

procedure ZadejM(var M:matice; data:string);
var i:word;
    radku,sloupcu:word;
    cislo:typprvkumatice;
    pozice:byte;
    slovo:string;
    kod:integer;
Begin
while not (data[1] in cifry) do delete(data,1,1);{oriznuti pripadnych mezer ze zacatku}
while not (data[byte(data[0])] in cifry) do dec(byte(data[0]));{oriznuti pripadnych mezer z konce}
if data='' then begin
                matresult:=matkoneczadani;
                zrusm(M);
                exit;
                end;
pozice:=1;
{spocitani radku (pocet radku = pocet stredniku + 1):}
radku:=1;
for i:=1 to byte(data[0]) do if data[i]=';' then inc(radku);
{spocitani sloupcu ( = pocet cisel v libovolnem radku, treba prvnim):}
i:=1; sloupcu:=0;
 repeat
 while data[i] in cifry do inc(i); {dojdeme za cislo}
 inc(sloupcu); {cislo znamena sloupec}
 while (data[i]<>';')and not(data[i] in cifry) do inc(i); {dojdeme bud na dalsi cislo nebo na strednik - konec radku}
 until data[i]=';'; {a to cele dokud nejsme na konci radku}
{vytvoreni matice:}
if not vytvorm(M,radku,sloupcu) then matresult:=matmalopameti
 else {nacteni dat do matice (diky usporadani matice po radcich staci jeden cyklus):}
      for i:=0 to radku*sloupcu-1 do begin
                                     slovo:='';
                                     while not (data[pozice] in cifry) do inc(pozice);{na zacatek cisla}
                                     if pozice>byte(data[0]) then begin{jsme za koncem retezce, coz je chyba}
                                                                  matresult:=matkoneczadani;
                                                                  exit;
                                                                  end;
                                     while (pozice<=byte(data[0]))and(data[pozice] in cifry) do
                                       begin{nacteni cisla}
                                       slovo:=slovo+data[pozice];
                                       inc(pozice);
                                       end;
                                     kod:=pos(',',slovo);{nalezeni pripadne desetinne carky...}
                                     if kod<>0 then slovo[kod]:='.';{...a jeji prepsani na tecku}
                                     val(slovo,cislo,kod);{prevod slova na cislo}
                                     if kod=0 then M._data^[i]:=cislo{OK}
                                              else begin{chyba}
                                                   matresult:=matchybavzadani;
                                                   exit;
                                                   end;
                                     end;{for i}
End;{zadejm}

procedure ProhodRadkyM(var M:matice; Ktery,sKterym:word);
var bafr:uknappm;
    velikost:word;
Begin
with M do
 begin
 dec(ktery); dec(skterym); {pro pocitani od nuly}
 if (ktery>_vyska-1)or(skterym>_vyska-1) then matresult:=matnesedirozmery {indexy jsou mimo matici}
  else if ktery=skterym then matresult:=0 {jeden radek sam se sebou neni potreba prohazovat -> OK, hotovo}
   else if _data=nil then matresult:=matzdrojneexistuje
    else begin
         velikost:=_sirka*sizeof(typprvkumatice); {velikost radku v bytech}
         if velikost>maxavail then matresult:=matmalopameti {je pamet?}
          else begin {a konecne vlastni presun:}
               getmem(bafr,velikost);
               move(_data^[ktery*_sirka],bafr^,velikost);
               move(_data^[skterym*_sirka],_data^[ktery*_sirka],velikost);
               move(bafr^,_data^[skterym*_sirka],velikost);
               freemem(bafr,velikost);
               matresult:=0;
               end;
         end;
 end;
End;{prohodradkym}

procedure KopirujM(odkud:matice; var kam:matice);
Begin
if odkud._data=nil
 then begin
      zrusm(kam);
      matresult:=matzdrojneexistuje; {v tomto pripade je to vlastne OK}
      end
 else if vytvorm(kam,odkud._vyska,odkud._sirka)then begin
                                                    move(odkud._data^,kam._data^,kam._vyska*kam._sirka*sizeof(typprvkumatice));
                                                    matresult:=0;
                                                    end
                                               else matresult:=matmalopameti;
End;{kopirujm}

procedure TrojuhM(M:matice; var vysledek:matice; var determinant:typprvkumatice);
var i,j,k:word;{indexy}
    pom:typprvkumatice;{pomocne cislo}
Begin
if M._data=nil then begin
                    zrusm(vysledek);
                    matresult:=matzdrojneexistuje;
                    end
 else begin
      kopirujm(M,vysledek);
      determinant:=1;{budeme ho pak ruzne nasobit}
      with vysledek do
       begin
       for i:=0 to mensiz(_sirka,_vyska)-1 do
        begin
        {test, kde v tom sloupci neni nula:}
        k:=i;{zacne se od diagonaly a postupuje se dolu}
        while (vysledek._data^[k*vysledek._sirka+i]=0)and(k<vysledek._vyska) do inc(k);
        {jestli jsou nuly vsude, matice je singularni a koncime:}
        if k>=_vyska then begin
                          zrusm(vysledek);
                          matresult:=matsingularni;
                          determinant:=0;
                          exit;
                          end;
        {na diagonale potrebujeme nenulove cislo:}
        if k<>i then begin
                     prohodradkym(vysledek,k+1,i+1);
                     determinant:=-determinant;{prohozenim radku se zmeni znamenko determinantu}
                     end;
        {vydelit i-ty radek prvkem na diagonale, aby na diagonale byla 1:}
        pom:=_data^[i*_sirka+i];{nacteni prvku}
        for j:=0 to _sirka-1 do _data^[i*_sirka+j]:=_data^[i*_sirka+j]/pom;{vydeleni vsech prvku radku tim na diagonale}
        determinant:=determinant*pom;
         {vydelenim radku konstantou se vydelil i determinant, takze ho musim zase vynasobit, aby zustal spravne}
        {nulovani sloupce pod prvkem na diagonale:}
        for k:=i+1 to _vyska-1 do{pro kazdy radek pod diagonalou}
         begin
         pom:=_data^[k*_sirka+i];{prvni prvek toho radku}
         for j:=i to _sirka-1 do{pro prvky toho radku (pred i-tym jsou bud nuly nebo oblast mimo matici)}
          _data^[k*_sirka+j]:=_data^[k*_sirka+j]-pom*_data^[i*_sirka+j];{odecteme pom-nasobek i-teho radku}
          {determinant se nemeni}
         end;{for k}
        end;{for i}
       end;{with}
      end;{else}
{Pro vypocet determinantu byla pouzita tato pravidla:
 - pokud prohodime dva radky matice, zmeni se znamenko determinantu matice
 - pokud nejaky radek vynasobime nejakym cislem, vynasobi se jim i hodnota
   determinantu
 - pokud k jednomu radku pricteme linearni kombinaci ostatnich (tj. treba
   nejaky nasobek jineho radku), hodnota determinantu se nezmeni
 - determinant trojuhelnikove matice se rovna soucinu prvku na hlavni
   diagonale}
End;{trojuhm}

function DetM(M:matice):typprvkumatice;
var pomm:matice;
    det:typprvkumatice;
Begin
initm(pomm);
with M do
if _data=nil then matresult:=matzdrojneexistuje
 else if _sirka<>_vyska then matresult:=matnesedirozmery{determinant jde jen ze ctvercove matice}
  else if _sirka=1 then detm:=_data^[0]{pro matici 1x1 je determinant rovny tomu jedinemu prvku}
   else if _sirka=2 then detm:=_data^[0]*_data^[3]-_data^[1]*_data^[2]
                         {2x2: det.= soucin prvku z hlavni diagonaly minus soucin prvku z te druhe diagonaly}
    else if _sirka=3 then detm:=_data^[0]*_data^[4]*_data^[8]
                               +_data^[3]*_data^[7]*_data^[2]     {3x3: Sarussovo pravidlo,}
                               +_data^[6]*_data^[1]*_data^[5]     {trochu komplikovanejsi,}
                               -_data^[2]*_data^[4]*_data^[6]     {ale porad jeste v rozumnych mezich}
                               -_data^[5]*_data^[7]*_data^[0]
                               -_data^[8]*_data^[1]*_data^[3]
     else begin
          {pro matice vetsi nez 3x3 uz zadna zjednodusujici pravidla neplati
          a obecny rozvoj podle nektereho radku nebo sloupce by byl rekurzivni
          a pomaly, takze pojedeme pres trojuhelnikovou matici:}
          trojuhm(M,pomm,det);
          zrusm(pomm);
          detm:=det;
          end;
End;{detm}

procedure InvM(M:matice; var vysledek:matice);
var i,j,k:word;
    pom,pom2:matice;
    x:typprvkumatice;
Begin
if M._sirka<>M._vyska
 then begin {inverze jde jen ze ctvercove matice}
      matresult:=matnesedirozmery;
      zrusm(vysledek);
      end
 else begin
      initm(pom); initm(pom2);
      {postup: k dane matici pripojim zprava stejne velkou jednotkovou a to
      cele pomoci radkovych uprav upravuji tak dlouho, az z te puvodni vlevo
      mam jednotkovou. Ta vpravo (puvodne jednotkova) bude hledana inverze.
      Je na to i nejaky vzorec s determinanty a subdeterminanty, ale proc
      delat veci slozite, kdyz to jde jednoduse. A navic to takhle bude mozna
      i rychlejsi.}
      jednotkovam(pom,M._vyska,M._sirka);
      SpojM(M,pom,false,pom2);      {pom2 = M|pom}
      trojuhm(pom2,pom,x);{pom = trojuh.}
      zrusm(pom2);
      {ted jsou na hlavni diagonale jednicky a pod nimi nuly,
      zbyva vynulovat prvky nad hl. diagonalou:}
      with pom do
       for i:=0 to _vyska-2 do {pro i od nulteho do predposledniho radku (posledni neni potreba upravovat, tam je jenom ta 1)}
        for j:=i+1 to M._sirka-1 do {pro j pro kazdy prvek i-teho radku vpravo od diagonaly}
         begin
         x:=_data^[i*_sirka+j];
         for k:=j to _sirka-1 do
          _data^[i*_sirka+k]:=_data^[i*_sirka+k]-x*_data^[(j)*_sirka+k];{odecitani x-nasobku radku}
         end;
      {ted je vlevo jednotkova matice. Zbyva uz jenom tu pravou ukrojit a
      vratit jako vysledek:}
      subm(pom,1,M._sirka+1,pom._vyska,pom._sirka,vysledek);
      zrusm(pom);
      end;
End;{invm}

procedure VypisM(M:matice; jmeno:string);
var i,j:word;
Begin
write(jmeno);
if M._data=nil then writeln(' neexistuje.')
 else begin
      writeln(' =');
      for i:=1 to M._vyska do
       begin
       for j:=1 to M._sirka do write(prvekm(M,i,j):8:4);
       writeln;
       end;
      end;
writeln;
End;{vypism}

procedure NactiM(var M:matice; soubor:string);
var f:text;
    i:word;
    s,v:word;
    pom:typprvkumatice;
Begin
assign(f,soubor); reset(f);
if ioresult=0 then begin
                   read(f,v); read(f,s);
                   if (ioresult=0)and vytvorm(M,v,s)
                     then for i:=1 to s*v do begin
                                             read(f,pom);
                                             M._data^[i-1]:=pom;
                                             end;
                   if ioresult<>0 then matresult:=matiochyba;
                   close(f);
                   if ioresult<>0 then {to uz se snad nestane};
                   end
              else matresult:=matiochyba;
End;{nactim}

procedure UlozM(M:matice; soubor:string);
var f:text;
    i,j:word;
Begin
if M._data=nil then begin
                    matresult:=matzdrojneexistuje;
                    exit;
                    end;
assign(f,soubor); rewrite(f);
if ioresult=0 then with M do begin
                             writeln(f,_vyska);
                             writeln(f,_sirka);
                             for i:=0 to _vyska-1 do
                              begin
                              for j:=0 to _sirka-1 do write(f,_data^[i*_sirka+j]:9:6);
                              writeln;
                              end;
                             close(f);
                             if ioresult=0 then matresult:=0
                                           else matresult:=matiochyba;
                             end
              else matresult:=matiochyba;
End;{ulozm}

(************************ komplexni cisla: **********************************)

procedure ZadejKCAB(var KC:KomplexniCislo;a,b:real);
Begin
with KC do begin
           _R:=sqrt(a*a+b*b);{abs. hodnota je vzdalenost od pocatku -> Pythagorova veta}
           {s uhlem je to slozitejsi:}
           if a>0 then _fi:=arctan(b/a){prvni nebo ctvrty kvadrant}
                  else if a<0 then _fi:=pi+arctan(b/a){druhy nebo treti kvadrant}
                              else if b>=0 then _fi:=pi/2{a=0, b je kladne (nebo 0)}
                                           else _fi:=3*pi/2;{a=0, b je zaporne}
           end;
End;{zadejkcab}

procedure ZadejKCRFi(var KC:KomplexniCislo;R,fi:real);
Begin
with KC do begin
           _R:=R;
           _fi:=fi;
           end;
End;{zadejkcrfi}

function KCRe(KC:KomplexniCislo):real;
Begin
with KC do kcre:=_R*cos(_fi);
End;{kcinfo}

function KCIm(KC:KomplexniCislo):real;
Begin
with KC do kcim:=_R*sin(_fi);
End;{kcinfo}

procedure SectiKC(co,sCim:KomplexniCislo; var vysledek:KomplexniCislo);
Begin
{pro scitani je lepsi algebraicky tvar}
zadejKCab(vysledek,co._R*cos(co._fi)+scim._R*cos(scim._fi),
                   co._R*sin(co._fi)+scim._R*sin(scim._fi));
End;{sectikc}

procedure OdectiKC(OdCeho,Co:KomplexniCislo; var vysledek:KomplexniCislo);
Begin
zadejKCab(vysledek,odceho._R*cos(odceho._fi)-co._R*cos(co._fi),
                   odceho._R*sin(odceho._fi)-co._R*sin(co._fi));
End;{odectikc}

procedure NasobKC(Co,Cim:KomplexniCislo; var vysledek:KomplexniCislo);
Begin
{nasobit komplexni cisla znamena vynasobit jejich absolutni hodnoty a secist
jejich uhly (Moivreova veta):}
vysledek._R:=co._R*cim._R;
vysledek._fi:=co._fi+cim._fi;
End;{nasobkc}

procedure VydelKC(Co,Cim:KomplexniCislo; var vysledek:KomplexniCislo);
Begin
vysledek._R:=co._R/cim._R;
vysledek._fi:=co._fi-cim._fi;
End;{vydelkc}

procedure KCxRC(var Co:KomplexniCislo; cim:real);
Begin
with co do _R:=_R*cim;
{je to totez jako nasobit to komplexnim cislem s nulovym uhlem
(coz je realne cislo)}
End;{kcxrc}

(**************************** ostatni veci: *********************************)

function pow(a,n:real):real;
var i:longint;
    vysledek:real;
Begin
if a=0 then pow:=0 {nula na cokoli je porad nula}
 else if n=0 then pow:=1 {cokoli na nultou je 1}
  else if frac(n)=0 then begin {obecna mocnina, ale s celym exponentem}
                         vysledek:=1;
                         for i:=1 to abs(trunc(n)) do vysledek:=vysledek*a;
                         if n<0 then vysledek:=1/vysledek; {protoze a^(-n) = 1/(a^n)}
                         pow:=vysledek;
                         end
   else {uplne obecna mocnina s realnym exponentem}
        if a<0 then pow:=0 {radsi 0 nez Invalid floating point operation pri zapornem zakladu logaritmu}
               else pow:=exp(n*(ln(a))); {protoze ln(a^n)=n*ln(a) a exp(ln(x))=x}
End;{pow}

function tg(x:real):real;
var cosinus:real;
Begin
cosinus:=cos(x);
if cosinus=0 then tg:=0 {lepsi nula ve vysledku nez ve jmenovateli (Division by zero)}
             else tg:=sin(x)/cosinus;
End;{tg}

function cotg(x:real):real;
Begin
if x=0 then cotg:=0
       else cotg:=cos(x)/sin(x);
End;{cotg}

function sgn(x:real):shortint;
Begin
if x<0 then sgn:=-1
       else if x>0 then sgn:=1
                   else sgn:=0;
End;{sgn}

function vzdalbodu(x1,y1,x2,y2:longint):word;
Begin
vzdalbodu:=round(sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)));{Pythagorova veta}
End;{vzdalbodu}

function preved(cislo:longint;zaklad,sirka:byte):string;
var vysl:string;
    i:word;
    minus:boolean;
Begin
minus:=cislo<0; cislo:=abs(cislo);
vysl:='';
 repeat {prevod}
 i:=cislo mod zaklad;
 cislo:=cislo div zaklad;
 if i<10 then vysl:=chr(i+ord('0'))+vysl
         else vysl:=chr(i+ord('A')-10)+vysl;
 until cislo=0;
while byte(vysl[0])<sirka do vysl:='0'+vysl;{prodlouzeni na pozadovanou sirku}
if minus then preved:='-'+vysl else preved:=vysl;
End;{preved}

function naDes(ret:string;zaklad:byte):longint;
var a:word;
    i:byte;
    minus:boolean;
Begin
minus:=ret[1]='-'; if minus then delete(ret,1,1);
for i:=1 to byte(ret[0]) do ret[i]:=upcase(ret[i]);{vsechno na velka pismena}
a:=0;
for i:=1 to byte(ret[0]) do{pro kazdy znak retezce, zleva}
  begin
  a:=a*zaklad;
  if ret[i]>='A'then inc(a,byte(ret[i])-55)  {65=ord('A')}
                else inc(a,byte(ret[i])-48); {48=ord('0')}
  end;
if minus then nades:=-a
         else nades:=a;
End;{nades}

procedure vektorovysoucin(u1,u2,u3,v1,v2,v3:real;var r1,r2,r3:real);
Begin
r1:=u2*v3-u3*v2;
r2:=u3*v1-u1*v3;
r3:=u1*v2-u2*v1;
End;{vektorovysoucin}

procedure kvadrovnice(a,b,c:real; var x1,x2:real; var CoVyslo:byte);
var d:real;{diskriminant}
Begin
if a=0 then if b=0 then if c=0 then covyslo:=krnekonecnoreseni
                               else covyslo:=krnemareseni
                   else begin
                        covyslo:=krlinearni;
                        x1:=-c/b;
                        x2:=x1;
                        end
       else begin
            d:=b*b-4*a*c;
            if d>0 then begin
                        covyslo:=krdvarealne;
                        d:=sqrt(d);
                        x1:=(-b+d)/(2*a);
                        x2:=(-b-d)/(2*a);
                        end
                   else if d=0 then begin
                                    covyslo:=krdvojnasobny;
                                    x1:=(-b)/(2*a);
                                    x2:=x1;
                                    end
                               else begin
                                    covyslo:=krkomplexni;
                                    d:=sqrt(-d);
                                    x1:=(-b)/(2*a);
                                    x2:=d/(2*a);
                                    end;
            end;
End;{kvadrovnice}

function orez(a,min,max:integer):integer;
Begin
if a<min then a:=min;
if a>max then a:=max;
orez:=a;
End;{orez}

function rychlysin(x:integer):integer;
const sinus:array[0..180]of integer={tabulka hodnot}
                 (0,175,349,523,698,872,1045,1219,1392,1564,1736,1908,2079,2250,2419,2588,2756,2924,3090,3256,3420,3584,3746,
                  3907,4067,4226,4384,4540,4695,4848,5000,5150,5299,5446,5592,5736,5878,6018,6157,6293,6428,6561,6691,6820,
                  6947,7071,7193,7314,7431,7547,7660,7771,7880,7986,8090,8192,8290,8387,8480,8572,8660,8746,8829,8910,8988,
                  9063,9135,9205,9272,9336,9397,9455,9511,9563,9613,9659,9703,9744,9781,9816,9848,9877,9903,9925,9945,9962,
                  9976,9986,9994,9998,10000,9998,9994,9986,9976,9962,9945,9925,9903,9877,9848,9816,9781,9744,9703,9659,9613,
                  9563,9511,9455,9397,9336,9272,9205,9135,9063,8988,8910,8829,8746,8660,8572,8480,8387,8290,8192,8090,7986,
                  7880,7771,7660,7547,7431,7314,7193,7071,6947,6820,6691,6561,6428,6293,6157,6018,5878,5736,5592,5446,5299,
                  5150,5000,4848,4695,4540,4384,4226,4067,3907,3746,3584,3420,3256,3090,2924,2756,2588,2419,2250,2079,1908,
                  1736,1564,1392,1219,1045,872,698,523,349,175,0);
var zn:shortint;{znamenko}
Begin
x:=x mod 360;
if x<0 then begin
            zn:=-1; x:=-x;
            end
       else if x>0 then zn:=1
                   else zn:=0;
if x>180 then rychlysin:=zn*(-sinus[x-180])
         else rychlysin:=zn*sinus[x];
End;{rychlysin}

function rychlycos(x:integer):integer;
Begin
rychlycos:=rychlysin(x+90);
End;{rychlycos}

function vpoli(x,y,x1,y1,x2,y2:integer):boolean;
Begin
vpoli:=(x>=x1)and(x<=x2)and(y>=y1)and(y<=y2);
End;{vpoli}

function ArcSin(x:real):real;
Begin
arcsin:=arctan(x/sqrt(1-x*x));
End;{arcsin}

function ArcCos(x:real):real;
var r:real;
Begin
r:=1-x*x;
if r>=0 then arccos:=arctan(sqrt(1-x*x)/x)
        else arccos:=0;
End;{arccos}

function vetsiz(a,b:integer):integer;
Begin
if a>b then vetsiz:=a else vetsiz:=b;
End;{vetsiz}

function mensiz(a,b:integer):integer;
Begin
if a<b then mensiz:=a else mensiz:=b;
End;{mensiz}

function uhelvektoru(u1,u2,v1,v2:integer):real;
var u,v:real;
Begin
u:=sqrt(u1*u1+u2*u2);
v:=sqrt(v1*v1+v2*v2);
uhelvektoru:=arccos((u1*v1+u2*v2)/(u*v));
End;{uhelvektoru}

function Uhelpruvodice(x,y:integer):real;
Begin
if x>0 then uhelpruvodice:=arctan(y/x) {1. a 4. kvadrant}
       else if x<0 then uhelpruvodice:=pi+arctan(y/x) {2. a 3. kvadrant}
                   else if y>=0 then uhelpruvodice:=pi/2 {primo nahoru}
                                else uhelpruvodice:=3*pi/2; {primo dolu}
End;{uhelpruvodice}

function otocbyte(cislo:byte):byte; assembler;
Asm    {v Asm je to kupodivu jednodussi a pohodlnejsi na napsani nez v obycejnem Pascalu :-) }
mov BL,cislo    {BL = zdroj}
xor AL,AL       {AL = cil, zatim 0}
mov CX,8        {bude to 8krat (pocet bitu v bytu)}
 @cyklus:
 shl AL,1       {AL o 1 bit doleva}
 ror BL,1       {BL rotuj o 1 bit doprava}
 adc AL,0       {pokud "pretekla" jednicka, pricti ji k AL (skonci na nejnizsim bitu)}
 loop @cyklus
{navratova hodnota funkce se rovnou preda v AL, kde uz je vysledek}
End;{otocbyte}

function OtocWord(cislo:word):word; assembler;
Asm                {totez, ale pro 16 bitu}
mov BX,cislo
xor AX,AX
mov CX,16
 @cyklus:
 shl AX,1
 ror BX,1
 adc AX,0
 loop @cyklus
{navratova hodnota funkce se preda v AX}
End;{otocword}

procedure rozvinplast(md,vd,h:real; var mr,vr,alfa:real);
var a,b,c:real;
Begin
if md>vd then begin{kdyz je zadany mensi prumer vetsi nez ten vetsi, radsi je prohodim}
              a:=md;
              md:=vd;
              vd:=a;
              end;
if md=vd then begin {je to valec}
              mr:=pi*md; {obvod podstavy = sirka plaste}
              vr:=h; {vyska plaste je rovna te zadane}
              alfa:=0; {uhel dvou rovnobezek je 0}
              end
         else begin {je to kuzel, at uz komoly nebo obycejny}
              md:=md/2; vd:=vd/2; {odtedka to jsou polomery!}
              {pomocne hodnoty:}
              a:=2*pi*vd;
              b:=2*pi*md;
              c:=sqrt(sqr(h)+sqr(vd-md));
              {vlastni vypocet:}
              alfa:=(a-b)/c;
              mr:=b/alfa;
              vr:=a/alfa;
              end;
End;{rozvinplast}

function interpol(x1,y1,x2,y2,x:real):real;
Begin
interpol:=y1+(y2-y1)*(x-x1)/(x2-x1);
End;{interpol}

function DivUp(co,cim:longint):longint;
Begin
divup:=co div cim+ord((co mod cim)<>0);
End;{divup}

function VzdalBoduOdPrimky(x,y,a,b,c:real):real;
Begin
vzdalboduodprimky:=abs(a*x+b*y+c)/sqrt(a*a+b*b);
End;{vzdalboduodprimky}

END.