(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: PCX.PAS                                                        *)
(*  Obsah: procedury pro ukladani a zobrazovani obrazku ve formatu PCX,    *)
(*         pouze pro 256barevnou grafiku (VESA)                            *)
(*  Autor: Mircosoft (http://mircosoft.ic.cz)                              *)
(*         ASP/VR Group                                                    *)
(*  Posledni uprava: 18.6.2008                                             *)
(*  Pro kompilaci: VESA.TPU, PALETA2.TPU                                   *)
(*  Pro spusteni: nic                                                      *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
unit PCX;
interface

function UlozPCX(x1,y1,x2,y2:word; JmenoSouboru,Popis:string; ISPaletou:boolean):boolean;
{Ulozi dany vyrez obrazovky jako obrazek PCX. Vraci true, pokud se to povede.
 x1,y1 - levy horni roh   \ pozor, nesmi vycuhovat z obrazovky (_getimage)
 x2,y2 - pravy dolni roh  /
 JmenoSouboru - jmeno souboru, do ktereho se ma obrazek ulozit, i s koncovkou.
                Pokud uz soubor existuje, bude prepsan bez varovani.
 Popis - v hlavicce PCX souboru je rezervovanych 58 B. Proc je nevyuzit?
         Tento retezec bude do toho rezervovaneho mista ulozen (max. 57 znaku)
 ISPaletou - true => nacte se aktualni paleta z obrazovky a ulozi se spolu
                     s obrazkem. Obvykla volba.
             false => soubor se ulozi bez palety. Pokud chcete obrazek
                      zobrazovat jen v programech, kde bude paleta nastavena
                      a potrebujete usetrit misto, pouzijte tuto volbu.}

procedure ZobrazPCX(x,y:integer; JmenoSouboru:string; ISPaletou:boolean);
{Zobrazi obrazek z PCX souboru. Pokud soubor neni 256barevny PCX nebo se
ho nepodari nacist, nic se nezobrazi.
 x,y - pozadovana poloha leveho horniho rohu obrazku na obrazovce. Pokud date
       nejakou z tech souradnic -1000, pouzije se souradnice nactena z hla-
       vicky souboru. Vycuhovani obrazku z obrazovky je povoleno (orizne se).
 JmenoSouboru - jmeno souboru k nacteni, i s koncovkou
 ISPaletou - true => pokud soubor obsahuje paletu, bude nactena a nastavena.
             false => i kdyby tam paleta byla, nactena nebude.
Text ulozeny v rezervovanem miste v hlavicce procedura ulozi do teto promenne:}
var PCXtext:string[57];

procedure Screenshot;
{Vyfoti obrazovku a ulozi ji ve formatu PCX (i s paletou). Jmeno souboru bude
ve tvaru SCR#.PCX, kde misto # bude nejake cislo. Prvni screenshot bude mit
cislo nahodne (0..9999), kazdy dalsi vzdy o 1 vetsi nez ten predchozi.}


{Neco o standardni RLE (Run Length Encoding) kompresi pouzite v PCX obrazcich.
Data jsou v souboru ulozena po radcich zleva doprava a shora dolu. Rozbaleni
dat pri cteni obrazku ze souboru probiha takto:
- Nacte se 1 byte (nazveme ho treba X).
- Pokud ma horni 2 bity nastaveny na 1 (tj. (X and $C0)=$C0 ), znamena to, ze
  ostatni bity v nem (tj. X and $3F) urcuji, kolikrat se ma na obrazovku
  zapsat byte, ktery po bytu X nasleduje.
- Pokud horni 2 bity nejsou oba 1 (tj. (X and $C0)<>$C0 ), znamena to, ze jde
  o nekomprimovany pixel, ktery staci rovnou vzit a zobrazit.
Z toho vyplyva nasledujici:
- Kdyz chci ulozit pixel s hodnotou, ktera ma shodou okolnosti v hornich dvou
  bitech jednicky, musim ho ulozit tak, ze pred nej dam infobyte X, i kdyby
  tam ten pixel mel byt treba jenom jeden. Coz rozhodne kompresi neprida
  na kvalite.
- Kdyz ukladam sekvenci stejnych pixelu, muze jich byt max. 63 (X and $3F),
  protoze 2 bity z bytu se spotrebuji na informaci, ze jde o infobyte.
  Napr. na sekvenci 252 stejnych pixelu tak budu potrebovat 4 infobyty.}

implementation
uses VESA,paleta2;

type
hlavickaPCX = record
              znacka:byte;{signatura PCX souboru (10)}
              verze:byte;{obvykle 5}
              kodovani:byte;{zde 1 (RLE)}
              BituNaPixel:byte;{zde 8}
              LHX,LHY,{souradnice leveho horniho rohu obrazku}
              PDX,PDY:word;{souradnice praveho dolniho rohu obrazku}
              rozliseniX,rozliseniY:word;{aktualni rozliseni obrazovky (napr. 640,480)}
              prvnich16barev:array[0..15,1..3] of byte;
      {zacatek palety. Slozky jsou v poradi B,G,R a vynasobene 4. Zde celkem k nicemu, vyuzilo by se to u 16barevnych obrazku}
              rezervovano1:byte;{k nicemu}
              PocetBitovychRovin:byte;{zde 1}
              BytuNaRadek:word;{zde rovno sirce obrazovky}
              ObsahujePaletu:word;{1 - na konci souboru je 256barevna paleta, 0 - neni tam}
              rezervovano2:string[57];{58 bytu, ktere nejsou opet nijak vyuzity.
                                       Rozhodl jsem se je tedy vyuzit pro ulozeni ruznych textovych informaci.}
              end;

pole768bytu=array[0..767]of byte; {pomocny typ pro prepocitavani palety}

var indexscreenshotu:word;

function UlozPCX(x1,y1,x2,y2:word;JmenoSouboru,popis:string;ISPaletou:boolean):boolean;
var hlavicka:hlavickaPCX;
    r,g,b:byte;
    sirka,i:word;
    f:file;
    paleta:rgbpal256;
    InPos,OutPos:word;
    Data,Count:byte;
    buffer:array[1..8192] of byte;
Begin
ulozpcx:=false;
sirka:=x2-x1+1;
if sirka>4096 then Exit;
with hlavicka do begin
                 znacka:=10;
                 verze:=5;
                 kodovani:=1;
                 BituNaPixel:=8;
                 LHX:=x1; LHY:=y1;
                 PDX:=x2; PDY:=y2;
                 rozliseniX:=_maxx+1; rozliseniY:=_maxy+1;
                 for i:=0 to 15 do begin
                                   getrgbpal(i,r,g,b);
                                   prvnich16barev[i,1]:=b shl 2;
                                   prvnich16barev[i,2]:=g shl 2;
                                   prvnich16barev[i,3]:=r shl 2;
                                   end;
                 rezervovano1:=0;
                 PocetBitovychRovin:=1;
                 BytuNaRadek:=sirka;
                 if odd(bytunaradek) then inc(bytunaradek);
                 ObsahujePaletu:=ord(ispaletou);
                 rezervovano2:=popis;
                 end;
assign(f,jmenosouboru);
rewrite(f,1);
if ioresult<>0 then exit;
blockwrite(f,hlavicka,sizeof(hlavicka));
OutPos:=1;
{implementace kompresniho algoritmu je od ASP/VR Group, ja jsem jenom pridal
komentare a trosicku optimalizoval nektera mista}
for i:=y1 to y2 do
  begin
  _getimage(x1,i,x2,i,@buffer);
  InPos:=1;
   repeat{komprese a zapis}
   {cteni useku stejnych pixelu:}
   Data:=buffer[InPos];
   Count:=1;
   inc(inpos);
    repeat
    if InPos=hlavicka.bytunaradek+1 then break;
    if buffer[InPos]=Data then begin
                               Inc(Count);
                               Inc(InPos);
                               end
                          else break;
    until Count=63;
   {Inpos je ted za prectenym usekem, v Data je hodnota pixelu a v Count je, kolikrat tam byl}
   if (Count>1)or((Data and $C0)=$C0) then begin{jestli tam pixel byl vic nez jednou,
                                                 nebo jestli tam byl jednou, ale ma hodnotu s hornimi dvema bity 1}
{btw, pokud count=2, komprese se jeste nevyplati, ale to se tady neresi}
                                           Buffer[OutPos+4096]:=($C0 or Count);
                                             {zapiseme infobyte (kolikrat tam ten nasledujici pixel bude)}
                                           Inc(OutPos);{o byte v bufferu dal}
                                           if OutPos=4097 then begin{jestli jsme na konci bufferu, hodime ho do souboru}
                                                               OutPos:=1;
                                                               BlockWrite(f,Buffer[4097],4096);
                                                               end;
                                           end;
   Buffer[OutPos+4096]:=Data;{zapis vlastniho pixelu}
   Inc(OutPos);{o byte dal}
   if OutPos=4097 then begin{kdyz jsme na konci bufferu}
                       OutPos:=1;
                       BlockWrite(f,Buffer[4097],4096);
                       end;
   until InPos=hlavicka.bytunaradek+1;
  end;
if OutPos>1 then BlockWrite(f,Buffer[4097],OutPos-1);
if ispaletou then begin{ulozeni palety}
                  zjistipaletu(paleta);
                  {prepocitani z 0..63 na 0..255:}
                  for i:=0 to 767 do pole768bytu(paleta)[i]:=round(pole768bytu(paleta)[i]*255/63);
                  b:=$0C;
                  blockwrite(f,b,1);{zapis kontrolniho bytu}
                  blockwrite(f,paleta,768);
                  end;
close(f);
if ioresult=0 then ulozpcx:=true;
End;{ulozPCX}

procedure ZobrazPCX(x,y:integer;JmenoSouboru:string;ISPaletou:boolean);
var InCounter,OutCounter:word;
    i,LineCounter:word;
    hlavicka:hlavickaPCX;
    Dummy:word;
    Count,Data:byte;
    f:file;
    Buffer:array[1..8192] of byte;
    paleta:rgbpal256;
Begin
assign(f,jmenosouboru);
reset(f,1);
if ioresult<>0 then exit;
blockread(f,hlavicka,sizeof(hlavicka));
with hlavicka do if (ioresult<>0)or(znacka<>$0A)or(bitunapixel<>8)
                     or(pocetbitovychrovin<>1)or((pdx-lhx+1)>4096)
                   then begin{kdyz to neni 256barevny PCX, koncime}
                        Close(f);
                        Exit;
                        end;
with hlavicka do begin
                 if x=-1000 then x:=lhx;
                 if y=-1000 then y:=lhy;
                 {nacteni pripadneho popisu:}
                 if (byte(rezervovano2[0])<>0)and(byte(rezervovano2[0])<=57)
                   then pcxtext:=hlavicka.rezervovano2
                   else pcxtext:='';
                 end;
InCounter:=1; OutCounter:=1; linecounter:=0;
blockread(f,Buffer,4096,Dummy);
 repeat {postupne cteni, dekomprese a zobrazeni}
 if (Buffer[InCounter] and $C0)=$C0 then begin{horni 2 bity infobytu jsou 1 => zbyvajici bity urcuji,
                                               kolikrat tam bude nasledujici pixel}
                                         Count:=(Buffer[InCounter]) and $3F;{pocet}
                                         Inc(InCounter);{o byte ve zdroji dal, meli bychom pak byt na tom pixelu}
                                         if InCounter=4097 then begin{dojeli jsme na konec => musime nacist dalsi davku dat}
                                                                InCounter:=1;
                                                                BlockRead(f,Buffer,4096,Dummy);
                                                                end;
                                         Data:=Buffer[InCounter];{hodnota pixelu}
                                         Inc(InCounter);{zas o byte dal}
                                         if InCounter=4097 then begin
                                                                InCounter:=1;
                                                                BlockRead(f,Buffer,4096,Dummy);
                                                                end;
                                         end
                                    else begin{horni 2 bity nejsou 1 => je to nekomprimovany pixel}
                                         Count:=1;{bude jeden}
                                         Data:=Buffer[InCounter];
                                         Inc(InCounter);{o byte dal}
                                         if InCounter=4097 then begin{kdybychom prejeli konec}
                                                                InCounter:=1;
                                                                BlockRead(f,Buffer,4096,Dummy);
                                                                end;
                                         end;
 for i:=1 to Count do begin
                      {zapis zjisteneho poctu pixelu do vystupniho bufferu:}
                      Buffer[OutCounter+4096]:=Data;
                      Inc(OutCounter);
                      {pokud jsme na konci bufferu (radku), hodime ho na obrazovku:}
                      if OutCounter=hlavicka.bytunaradek+1 then begin
                                                                OutCounter:=1;
                                                                _putimage(x,y,hlavicka.pdx-hlavicka.lhx+1,1,
                                                                          @Buffer[4097]);
                                                                inc(y); inc(linecounter);
                                                                end;
                      end;
 until LineCounter=hlavicka.pdy-hlavicka.lhy+1;
if ispaletou then begin
                  seek(f,filesize(f)-769);{protoze nacitani obrazovych dat je trochu prasacke a neskonci presne na konci}
                  blockread(f,data,1);{nacteni kontrolniho bytu}
                  if (ioresult=0)and(data=$0C) then begin
                                                    blockread(f,paleta,768,dummy);
                                                    for i:=0 to 767 do
                                                     pole768bytu(paleta)[i]:=round(pole768bytu(paleta)[i]*63/255);
                                                    nastavceloupaletu(paleta);
                                                    end;
                  end;
close(f);
dummy:=ioresult;{vynulovani ioresultu}
End;{zobrazPCX}

procedure Screenshot;
var soubor:string[5];
    stra:boolean;
Begin
stra:=_strankovani;
if stra then _nastavstrankovani; {aby se vzdycky fotilo to, co zrovna vidime}
str(indexscreenshotu,soubor);
if ulozpcx(0,0,_maxx,_maxy,'SCR'+soubor+'.PCX','screenshot',true) then inc(indexscreenshotu);
if stra then _nastavstrankovani; {navrat do puvodniho stavu}
End;{screenshot}

BEGIN
randomize;
indexscreenshotu:=random(10000);
END.