(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: WOKNOWS4.PAS                                                   *)
(*  Obsah: soubor objektu, funkci a procedur pro tvorbu                    *)
(*         grafickeho uzivatelskeho rozhrani v programech                  *)
(*  Autor: Mircosoft (http://mircosoft.ic.cz)                              *)
(*  Posledni uprava: 12.6.2008                                             *)
(*  Pro kompilaci: DISK.TPU, MYS3.TPU, GRAFIKA3.TPU, VESA.TPU, KLAVESY.TPU,*)
(*                 RETEZCE.TPU, ERRMSG.TPU, WOKBARVY.INC, CFG.TPU, DOS.TPU *)
(*  Pro spusteni: (MSDFLT.FNT, STDSIPKA.KUR, W4TXT??.DAT)                  *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
unit Woknows4;
{$I-,B-,R-,X-,E-,F-,S-,Q-} {prvni tri jsou dulezite, ostatni ne}
{...$D-,L-} {usetri trochu pameti prekladace; pro ucely ladeni zruste}
interface
uses disk,mys3,grafika3,vesa,retezce,dos,xms;

{Vse, co zde najdete, volejte v grafickem rezimu.}

(******************** inicializace a par informaci: *************************)

procedure InitWoknows(_font,_kurzor,_texty:boolean);
{Inicializacni procedura, popsana bude o par radku dal. Pokud se vam to nechce
cist, nastavte defaultne vsechny tri parametry na true, tim se nic nezkazi.}
procedure ZrusWoknows;
{Postara se o zaverecny uklid - vymaze z pameti font, kurzor (pri tom ukonci
pripadnou automatickou obsluhu mysi, viz Zrusmys z jednotky MYS3) a texty.
(Jednotka XMS se postara o automaticke uvolneni rozsirene pameti i v pripade,
ze na tuhle proceduru zapomenete, ale bezpecnejsi a jistejsi je ji na konci
programu explicitne pouzit.)}

{Veskere texty v teto jednotce (krome nekolika mezer, otazniku, tecek apod.)
jsou ulozeny v externim textovem souboru:}
 const woktexty:string[12]='W4TXTCZ.DAT';
{Nacitaji se do objektu WT procedurou Initwoknows (obvykle). Pokud texty
nejsou nactene, daji se pouzivat pouze objekty krome Ciselne osy a Menu a
procedury Infokno a Zjistujsouradnice. Vsechno ostatni texty pouziva a kdyby
nebyly nactene, nebylo by co zobrazovat.
V souboru W4TXTCZ.DAT jsou ceske hacky a carky kodovany ve znakove sade
Latin 2. Vychozi font (MSDFLT.FNT) je kodovan stejne a zarucuje, ze se nebudou
zobrazovat paznaky. Pokud ve svem programu nastavite jiny font, jednotka si
vzdy pred zobrazenim "svych" textu prepne na "svuj" font a pak zase zpet.
Texty, ktere se jednotce predavaji z programu (hlasky v informacnich oknech
apod.) se vypisuji bud tim fontem, jaky je v programu aktualne nastaven (takze
si je muzete psat treba v Kamenikach) nebo take standardnim fontem, prepina se
pomoci jedne globalni promenne, viz dale. Nadpisy dialogovych oken jsou vzdy
psany aktualne nastavenym fontem a v pripade, ze zadate nadpis='', jsou
doplneny preddefinovanym textem z vyse uvedeneho souboru. Tady pak muze
dochazet k problemum, pokud pouzivate jine kodovani. V takovem pripade bud
vsechny nadpisy dusledne zadavejte rucne nebo si diakritiku v souboru s texty
prepiste do sve znakove sady.

Indexy textu jsou ulozeny zde:}
{$I WOKTEXTY.INC}

{Procedura Initwoknows krome textu nacita i vychozi font MSDFLT.FNT a mysi
kurzor STDSIPKA.KUR. Pokud potrebujete pouzit jine soubory nez tyto, nastavte
prislusne parametry na false a do globalnich promennych Wokfont a Wokkurzor
si rucne nactete neco jineho (proceduru Initwoknows vubec neni nutne pouzivat,
mam ji tu jen pro zkraceni zapisu).

Jednotka VESA umoznuje pouzivat v zobrazovanych textech ridici kody pro
nastaveni barvy, polohy a velikosti pisma. Jednotka Woknows tyto kody plne
respektuje jen v nadpisech objektu (texty na tlacitkach, nadpisy v modre liste
okna a tak). Vsude jinde (texty uvnitr dialogu, popisky polozek u prepinacu
atd.) sice ridici kody funguji take, ale mohou zpusobit problemy, protoze se
rozmery techto textu pocitaji pouze zjednodusenou formou z delky retezce (bez
vyhodnocovani kodu). Takze tam se zmenami velikosti a polohy velmi opatrne.}


(******************************** objekty: **********************************)
{orientacni seznam:

 Tlacitko    - textove tlacitko (button)
 Tlacitko2   - graficke tlacitko
 Zatrzitko   - zatrhavaci policko (checkbox)
 Prepinac    - vyber z nekolika moznosti (radio buttons)
 Roleta      - podobny vyber, ale jinak vypada (combobox/select)
 Posuvnik    - svisly posuvnik (scrollbar)
 CiselnaOsa  - vodorovny posuvnik urceny pro zadavani cisel
 Hotspot     - klikaci obdelnikova oblast (neviditelne tlacitko)
 TextovePole - jednoradkove pole pro zadavani textu (edit/text)
 Menu        - mnohoradkove menu s volitelnym poctem polozek i textu
 Okno        - proste okno, ktere po zavreni obnovi obsah obrazovky v miste,
               na kterem se otevrelo

 Vsechny funguji skoro stejne:
 Inicializace: Init(...) a spol. (vyjimky existuji, ale na ty budete vcas
upozorneni). Priradi se pocatecni hodnoty a cele zarizeni se vykresli
(o pripadne vypnuti kurzoru se postara, nakonec ho necha zapnuty). Ovladaci
prvek se vzdy musi vejit na obrazovku, na vycuhovani neni staveny - pozor!
Souradnice x,y znamenaji obvykle neco kolem leveho horniho rohu objektu.
 Opetovne zobrazeni: metoda Zobraz, pouzitelna kdykoli. O kurzor mysi se
postara a nechava ho pak zapnuty. U vetsiny objektu se prave tato metoda stara
o vypocty rozmeru podle aktualniho nastaveni fontu, takze pokud font zmenite a
pak znovu pouzijete zobrazeni, velikost objektu se prepocita. Take je mozne
nastavit v objektu rucne polozku Hodnota (ale pozor na povoleny rozsah!)
a pak zavolat zobrazeni, ktere vymaze puvodni zobrazenou hodnotu a vykresli
objekt s novou.
 Prace: v nejakem cyklu volejte metodu Kontrola (ve vetsine pripadu je lepsi
napred pockat na klik mysi nebo stisk klavesy), ktera otestuje mys, nastavi
Hodnotu a pripadne zobrazi prislusnou animaci.
 Cteni stavu: rucne, primo z polozky Hodnota. Pokud potrebujete zjistit,
jestli se Hodnota od minule kontroly zmenila, zeptejte se metody Zmeneno.
 Zruseni: metodou Zrus. Obsahuji ji jen ty prvky, u kterych je zruseni
potreba (neco se dynamicky alokovalo).


Barvy vsech objektu (konstanty) jsou v tomto souboru:}
{...$I wokbarvy.inc}
{$I kswokbar.inc}

type
NadpStr=string[60]; {trochu kratsi retezec pro nadpisy oken
               (cely string by byl vetsinou zbytecne plytvani pameti,
               ale v pripade potreby se tenhle typ da prodlouzit)}

wzakladobj = object
             _zmeneno:boolean; {tuto polozku bude v potomcich nastavovat metoda Kontrola}
             function zmeneno:boolean;{vraci aktualni hodnotu atributu _zmeneno a pote ho nastavi na false}
             end;
{zakladni objekt, ze ktereho budou dedit ostatni}

Tlacitko = object
           x,y:integer;{levy horni roh tlacitka}
           sirka,vyska,{rozmery (jsou o 1 mensi, aby se s nimi lip pocitalo), jsou urceny rozmery textu}
           pridavek:word;{vyska tlacitka je urcena vyskou textu plus Pridavek nad nim a pod nim}
           napis:string[60];{napis na tlacitku}
           popis:string;{zobrazi se v okenku po kliknuti pravym mysitkem}
           hodnota:byte;{cim na nej bylo kliknuto: 0 = nic, 1/2/4 = leve/prave/prostredni mysitko. Uchovava se do nejblizsiho
                         zavolani metody Kontrola (ktera ji nastavi podle aktualni situace).}
           procedure init(ix,iy:integer;inapis,ipopis:string;ipridavek:byte);
           procedure zobraz;{vypocita rozmery tlacitka a zobrazi ho}
           procedure kontrola;{kontroluje mys a pri kliknuti na tlacitko zobrazi animaci,
                               nastavi Hodnotu a pripadne zavola Obsluhu}
           end;
{Klasicke tlacitko s textem. Funkci Kontrola je potreba volat v cyklu po
Cekej, vyhodnoceni stavu pres Hodnotu. Velikost tlacitka je urcena rozmery
napisu v okamziku zobrazeni. Osetreny jsou i ty nejsilenejsi kombinace
ridicich kodu, da se pomoci nich udelat treba i viceradkovy text na jednom
tlacitku. Pokud je Popis='', klik pravym mysitkem se normalne vyhodnoti a do
Hodnoty se ulozi 2. Jestli obsahuje nejaky text, tak se po pravokliknuti tento
text zobrazi v okenku nad tlacitkem a klik bude ignorovan (Hodnota bude 0).}

Tlacitko2 = object
            x,y:word;
            hodnota:byte;
            popis:string;
            obrazky:tobrazky;
            procedure init(ix,iy:integer;ipopis:string);
            procedure zobraz;
            procedure kontrola;
            procedure zrus;{vymaze z pameti sadu obrazku pripojenou k tomuto tlacitku}
            end;
{Graficke tlacitko. NEJDRIVE je nutne rucne do polozky Obrazky odnekud nacist
obrazky (procedura Nactiobrazky nebo Nactiobrazky2 z jednotky Grafika3,
popripade muzete polozku Obrazky obycejnym prirazenim (':=') nasmerovat na jiz
existujici sadu obrazku) a POTOM teprve volat Init! Rozmery obrazku libovolne,
pocet v sade alespon 2 (prvni pro tlacitko v klidu, druhy na stisknute,
pripadne dalsi se nevyuziji, ale nevadi).
Jinak pracuje uplne stejne jako textove tlacitko.}

Zatrzitko = object(wzakladobj)
            x,y:integer;{levy horni roh ctverecku}
            sirka,vyska:word;{rozmery aktivni (kliknutelne) oblasti - ctverecek i text vedle nej}
            textik:string[60];{text vedle zatrzitka}
            popis:string;
            hodnota:boolean;{zatrzeno = true}
            procedure init(ix,iy:integer; itextik,ipopis:string; ihodnota:boolean);
            procedure zobraz;
            procedure kontrola;
            end;
{Bily zatrhavaci ctverecek s jednim radkem textu vedle. Kliknout se da bud do
bileho ctverecku nebo na ten text, rozmery oblasti pro kliknuti jsou urceny
rozmery textu v okamziku zobrazeni (podle aktualniho nastaveni pisma, opet
jsou podporovany veskere ridici kody). Na prave mysitko reaguje bud zobrazenim
popisu nebo, pokud je popis prazdny, vubec.}

Prepinac = object(wzakladobj)
           x,y:integer;{levy horni roh prvniho kolecka (nadpis je nad touto souradnici!)}
           nadpis:string[50];{radek zobrazeny nad polozkami prepinace}
           popis:string;{vyvola ho prave mysitko}
           Hodnota:byte;{0 = nevybrano nic, 1..PocetRadku = cislo prislusneho radku}
           sirka:word;{sirka aktivni oblasti podle nejsirsiho radku [pix.]}
           VyskaRadku:word;{vyska jednoho radku}
           texty:uknaradek;{ukazatel na seznam textu polozek}
           procedure init(ix,iy:integer; inadpis,itexty,ipopis:string; ihodnota:byte);
           procedure zobraz;
           procedure kontrola;
           procedure zrus;
           end;
{Nekolik radku, u kazdeho je bile kolecko. Kliknutim vyberete vzdy jednu
moznost. Cislo vybrane polozky urcite z promenne Hodnota. Polozky jsou
cislovany od 1; pokud je hodnota=0, neni vybrano nic. Parametry pro Init:
itexty je retezec se seznamem polozek (jsou oddeleny znakem '|'), ihodnota je
pocatecni Hodnota (muze byt i 0, pak neni vybrano nic). Ostatni je snad jasne.}

Roleta = object(wzakladobj)
         x,y:integer;{levy horni roh}
         sirka1,sirka2,vyska:word;{sirka leveho pole, sirka komplet vcetne rozbalovaciho tlacitka, vyska pole}
         popis:string;{zobrazi se po kliku pravym mysitkem}
         polozky,{ukazatel na seznam polozek}
         vybrana:uknaradek;{ukazuje na vybranou polozku (stejne jako Hodnota)}
         hodnota:word;{cislo aktualne vybrane polozky, cislovano od 1,
                       0 = nevybrano nic (to jde zadat pri inicializaci).}
         MaxHodnota:word;{celkovy pocet polozek}
         MaxDelka:byte;{pocet znaku nejdelsi polozky}
         procedure nuluj;
          {vynuluje ukazatel na polozky - pred prvnim pouzitim NUTNE!!!}
         procedure PridejPolozky(zdroj:string);
          {Pridava polozky do seznamu. Plati pravidla jako u procedury
          Nactitexty z jednotky Retezce. Pouzivat pred Init.}
         procedure init(ix,iy:integer; ihodnota:word; ipopis:string);
          {ihodnota muze byt i 0, pak neni vybrano nic}
         procedure zobraz;
         procedure kontrola;
         procedure zrus;
         end;
{Policko s nejakou textovou hodnotou a s rozbalovacim tlacitkem vpravo. Po
kliknuti levym mysitkem se ukaze roleta s nabidkou moznosti, ze kterych si
jednu vybereme.
Inicializace je ponekud netradicni - nejdriv je treba vynulovat, potom zadavat
polozky a az nakonec volat init. Zadavani polozek je tak mozne zautomatizovat
(pres nejaky cyklus apod.). Ovladani: mysi vyberete polozku z nabidky,
potvrdite kliknutim nebo enterem, zrusite kliknutim mimo roletu nebo escapem.}

Posuvnik = object(wzakladobj)
           x,y:integer;{levy horni roh horniho tlacitka}
           PocetPoloh:word;{pocet moznych poloh}
           hodnota:word;{aktualne nastavena poloha, pocitano od 0 do Pocetpoloh-1}
           VyskaPolohy:word;{vyska jedne polohy na stredni casti posuvniku}
           sirka:word;{sirka posuvniku, urcuje ji sirka textu pri zobrazeni}
           nahoruT,doluT:tlacitko;
           procedure init(ix,iy:integer; iVyska,iPocetPoloh,iHodnota:word);
              {ivyska je vyska samotne stredni casti posuvniku (bez tlacitek), [px]}
           procedure zobraz;
           procedure kontrola;
           end;{posuvnik}
{Svisly posuvnik.
Pozor, ze vyska, kterou zadate pri inicializaci, je spis orientacni. Posuvnik
si ji prizpusobi tak, aby co nejlepe odpovidala poctu poloh, takze nakonec
muze vyjit nizsi nez jste zadali. Vyssi muze vyjit pouze v pripade, ze zadany
pocet poloh je vetsi nez zadana vyska - pak budou jednotlive polohy vysoke
1 pixel (coz je absolutni minimum), ale ani tak se do zadaneho prostoru
nevejdou.}

CiselnaOsa = object(wzakladobj)
             x,y:integer;{levy horni roh leveho tlacitka}
             x2:integer;{levy okraj stredni casti osy}
             min,max,hodnota:longint;{rozsah hodnot a aktualni hodnota}
             sirka:word;{skutecna sirka osy v pixelech}
             ubert,pridejt:tlacitko;
             psatcisla:boolean;{jestli se ma pod osou vypisovat minimalni, maximalni a aktualni hodnota}
             pozadi:byte;{jakou barvou pozadi se bude mazat aktualni hodnota pri prepisovani}
             nadpis:string[50];{bude zobrazen nad osou}
             textmin,textmax:string[30];{budou zobrazeny nad osou (pod nadpisem)}
             procedure init(ix,iy:integer; isirka:word; imin,imax,ihodnota:longint; inadpis,itextmin,itextmax:string;
                            icisla:boolean; ipozadi:byte);
             procedure zobraz;
             procedure kontrola;
             end;{ciselnaosa}
{Vodorovny posuvnik pro zadavani cisel. Kliknutim do oblasti posuvniku levym
mysitkem presunete "jezdec" na souradnici kurzoru, prave mysitko vyvola
napovedu k tlacitkum: ze kliknutim levym mysitkem zmenite hodnotu o 1, pravym
o 10, Shift+levym o 100, Shift+pravym o 1000.}

hotspot = object(wzakladobj)
          a,b,c,d:integer; {[a,b] je levy horni roh, [c,d] pravy dolni}
          popis:string; {funguje stejne jako popis u tlacitek}
          hodnota:byte; {kterym mysitkem sem bylo od posledni kontroly kliknuto}
          xKliku,yKliku:word; {do ktereho mista bylo kliknuto (pocitano relativne od leveho horniho rohu)}
          resetovat:boolean; {jestli se ma po kliku cekat na uvolneni tlacitek}
          procedure init(ia,ib,ic,id:integer; ires:boolean; ipopis:string);
          procedure zobraz(barva:byte); {obtahne oblast danou barvou (nestara se o kurzor), na funkci nema vliv}
          procedure kontrola;
          end;
{Neviditelna obdelnikovita klikaci oblast. Pamatuje si, cim a kam presne jsme
do ni naposledy klikli.}


UkNaTP = ^textovepole; {pomocny}
TextovePole = object(wzakladobj)
              x,y:integer;{levy horni roh}
              sirka,vyska:word;{rozmery - pro usnadneni kontroly kliku mysi}
              MaxZnaku:byte;{max. delka retezce, urcuje sirku pole}
              pozice:integer;{poloha kurzoru v textu, 0 = pred prvnim znakem, -1 = vybran cely text}
              Hodnota:string[100];{text (omezeni delky je jenom kvuli tomu, ze vic se stejne na obrazovku nevejde)}
              default:string[100];{puvodni text, obnovi se pri stisku Esc}
              ProSoubor:boolean;{true => zakazou se znaky, ktere nejdou pouzit v nazvu souboru}
              stav:byte;{aktualni stav pole}
              predchozi,dalsi:uknatp;
              procedure init(ix,iy:integer;imaxzn:byte;itext:string;iakt,iprosoubor:boolean);
                               {iakt - jestli ma byt pole po inicializaci hned aktivni}
              procedure zobraz;
              procedure kontrola;
              procedure nasmeruj(var shiftTab,Tab:textovepole);
              end;
{Pole pro zadavani textu (1 radek). Kurzor se presouva pomoci mysi nebo
klavesami home, end a sipkami. Funguji delete a backspace. Esc rusi zadany
text a nahradi ho puvodnim (itext pri inicializaci) a necha pole aktivni.
Enter, tabulator nebo klik mysi mimo pole pole deaktivuji a napsany text
nechaji. Pole se opet aktivuje kliknutim mysi. Neaktivni pole neovlivnuje
klavesnici ani mys, lze ho kontrolovat spolu s jinymi ovladacimi prvky. Jako
aktivni si zabira klavesnici pro sebe (pokud neni kontrolovano jako uplne
posledni). POZOR: na konci cyklu, ve kterem textove pole testujete, je nutne
pockat na uvolneni tlacitek na mysi (napr. procedura Resetuj z Mys3). To
proto, ze pole sama na nej necekaji, aby ta, co jsou kontrolovana drive,
"nekradla" mys a klavesnici tem, co jsou kontrolovana pozdeji.
Ukazatele Predchozi a Dalsi umoznuji prechazeni mezi jednotlivymi textovymi
poli na obrazovce klavesou Tab (dalsi) nebo kombinaci shift+Tab (predchozi).
Nastavit je muzete bud rucne nebo metodou Nasmeruj. Po inicializaci maji oba
hodnotu nil, cili tabulator textove pole pouze deaktivuje a nikam vas
neprepne.
Pro urceni, jakym zpusobem bylo ukonceno psani do pole, slouzi polozka Stav,
ktera muze nabyvat techto hodnot:}
const _Neaktivni=0; {chova se inertne, nic nedela, kurzor neni videt}
      _Aktivni=1;   {kurzor je videt, lze psat}
      _Enterem=2;   {byl stisknut enter, pole je ted neaktivni}
      _Escapem=3;   {byl stisknut escape, pole je aktivni, cely text je oznacen}
      _Odkliknutim=4; {bylo kliknuto mimo pole nebo byl stisknut tabulator, pole je neaktivni}


type UkNaPolozkuMenu = ^PolozkaMenu;
     PolozkaMenu = record
                   texty:UkNaRadek;{ukazatel na seznam radku textu}
                   lhy,pdy:integer;{souradnice okraju}
                   dalsi,predchozi:uknapolozkumenu;{pro obousmerny spojovy seznam}
                   end;
     Menu = object
            PrvniRadek:UkNaRadek;
            PrvniPolozka:UkNaPolozkuMenu;
            hodnota:byte;
            procedure init;
            procedure PridejText(textik:string);
            procedure PridejPolozku(textik:string);
            procedure kontrola;
            procedure zrus;
            end;
{Dialog s uvodnim textem a nekolika moznostmi na vyber. Ovlada se trochu jinak
nez je u ostatnich objektu zvykem:
 init - vynuluje interni ukazatele. Pred prvnim pouzitim je nutne ji pouzit,
        ale nikdy jindy!!! (hlavne ne, pokud uz jsou zadane nejake texty)
 PridejText - prida dalsi text do uvodu. Lze pouzit i vickrat po sobe, text se
              bude pripojovat (vzdy na novem radku). Radky se deli bud
              automaticky, pokud vychazeji delsi nez:} const SirkaMenu=40;     {
              nebo rucne znakem '|'. Uvodni text neni povinny.
 PridejPolozku - prida jednu polozku do menu. Prvnim zavolanim vlozite prvni,
                 druhym druhou atd.. Pro radkovani uvnitr jedne polozky plati
                 totez co o uvodnim textu. Neni nutne zadavat zadne polozky,
                 v takovem pripade se automaticky vytvori polozka " OK ".
 Kontrola - vykresli menu, necha uzivatele vybrat nejakou moznost a do Hodnoty
            ulozi jeji cislo (prvni = 1, druha 2 atd.). Ovlada se mysi (vyber
            levym tlacitkem, ostatni nic) nebo klavesnici (sipky a Tab vyber,
            Enter nebo mezernik potvrzeni).
            POZOR: protoze menu byva obvykle velke jak krava, neuklada se
            pozadi okna a po pouziti zustane menu nakreslene na obrazovce!!!
            Vycuhovani z obrazovky je narozdil od ostatnich objektu osetreno
            (tj. nedojde k pretekani z praveho okraje na levy a podobne,
            proste se to orizne).
            Metodu Kontrola je mozne volat vickrat po sobe, ulozene texty
            nijak neovlivnuje. Take se da zavolat Kontrola, pak pridat par
            radku textu nebo polozek a zavolat Kontrolu znova.
 Zrus - zrusi vsechny texty v menu (uvod, polozky atd.) a pripravi ho pro
        nove pouziti.
Vzhledem k tomu, ze Menu je vlastne dialog a ne objekt, ktery by lezel na
obrazovce a cekal na akci uzivatele, ztraci metoda Zmeneno smysl, a proto zde
neni implementovana.}

type
Okno = object
       x,y:integer;{levy horni roh}
       y2:integer;{yova souradnice efektivniho leveho horniho rohu (pokud je nejaky nadpis, tak je tohle hned pod nim)}
       sirka,vyska:word;
       PozadiOkna:pointer;
       BylKurzor:boolean;
       procedure init(ix,iy:integer;isirka,ivyska:word;nadpis:nadpstr);
       procedure zrus;
       end;
{Viz Windows :-). Init okno zobrazi (a necha vypnuty kurzor), Zrus ho zase
zrusi (vrati puvodni pozadi a vrati kurzor do stavu, v jakem byl pred Initem).
Vyska modre listy je urcena aktualnim nastavenim pisma (osetreny jsou i ruzne
prasarny s ridicimi kody v nadpise), sirka okna se automaticky zvetsi, pokud
by se do nej nadpis nevesel. Pokud zadate nadpis='', zadna modra lista se
nezobrazi.}

(************************** globalni promenne: ******************************)
var WokFont:_font; {vychozi systemovy font; jde nastavit i vlastni, ale mel by vzdy byt 8 pixelu vysoky}
    WokKurzor:tkurzor; {vychozi kurzor pro mys}
    WT:xtext; {objekt pro ulozeni textovych konstant}

    WokZruseno:boolean; {tuto promennou nastavuji na true funkce Vloztextokno,
     Vlozcislookno, Vyberbarvuokno, Vybersouborokno, Vlozmaskuokno a
     Vlozvzorekokno, pokud byly ukonceny zrusenim (z navratove hodnoty se to
     nepozna, protoze se v takovem pripade vraci defaultni hodnota).}

(*********************** verejne procedury a funkce: ************************)
{Pokud jde o nejake okno a souradnice se mu nezadavaji rucne, zobrazi se vzdy
nekde pobliz stredu obrazovky, ktery si samo spocita.}

procedure InfOkno(ix,iy:integer; hlaska:string; resB:boolean);
{Male zlute informacni okenko s textem, odklikne se cimkoli. Text Hlaska
se automaticky rozklada do vice radku (slova se nedeli). Rucne muzete radek
ukoncit znakem '|'. Ix a iy je stred okna, ne levy horni roh. Pokud je resb
true, pocka se na konci procedury na uvolneni vsech tlacitek na mysi a
klavesnici (procedura Resetuj), jinak ne. Rozmery okna jsou urceny rozmery
textu, ktery se ma zobrazit. Okno svou polohu automaticky prizpusobuje tak,
aby nikde nevycnivalo z obrazovky.}

const DialogyStandardnimFontem:boolean=false;
{true - v nasledujicich dialogovych oknech se bude pouzivat standardni font
        z promenne Wokfont (obvykle MSDFLT.FNT, 8x8 pixelu), po skonceni dane
        procedury nebo funkce se obnovi puvodne nastaveny font
 false - pouzije se ten font, ktery je zrovna nastaven.}

function AnoNeOkno(texty:string):boolean;
{Okno s otazkou a tlacitky Ano a Ne. Retezec Texty obsahuje text otazky.
O automatickem i rucnim rozkladani textu do radku plati totez jako u Infokna,
jenomze tady muze byt radek o neco delsi. V modre liste okna je standardne
zobrazen nadpis 'Dotaz:'. Pokud ho chcete zmenit, zacnete retezec Texty znaky
'%%n', za ne napiste novy text nadpisu a zakoncete ho sekvenci '%%N' (pozor na
velikost pismen). Priklad: '%%nTohle bude nadpis%%NTohle uz je text okna.'.
"Ano" se da zvolit i klavesami A a Enter, "ne" klavesami N a Esc.
Funkce vraci true, pokud bylo zvoleno Ano.
Sirka okna je urcena sirkou nejdelsiho radku nebo nadpisu, vyska poctem radku.}
procedure OKOkno(texty:string);
{Okno se zpravou a tlacitkem OK. Texty i pripadne rucni zadavani nadpisu
funguji stejne jako u AnoNeOkna (vychozi nedpis je 'Zprva:'). Odklepnout se
da mysi (tlacitko OK), Enterem, mezernikem nebo klavesou O.}
function VlozTextOkno(nadpis:nadpstr; MaxZn:byte; DefaultText:string; ProSoubor:boolean):string;
{Dialogove okno pro zadani nejakeho textu.
 nadpis - bude zobrazen v horni modre liste okna. Pokud zadate '', zobrazi
          se 'Zadej text:'
 maxzn - maximalni pocet znaku textu (neprehanet, davejte max. cca 75,
         protoze do okna se vejde jenom jeden radek, tak aby se veslo
         na obrazovku)
 defaulttext - text, ktery je v okne zobrazen na zacatku a funkce ho vraci v
               pripade,ze je zadavani zruseno (tlacitko Zrus nebo klavesa Esc)
 prosoubor - true => zakaze psani diakritiky, interpunkce a mezer, vhodne pro
             bezpecne psani jmen souboru}
function VlozCisloOkno(nadpis:nadpstr; _od,_do,default:longint):longint;
{Dialog pro zadavani cisel.
 nadpis - viz vyse, pokud je '', pak se vypise 'Zadej slo:'
 _od,_do - rozsah pozadovaneho cisla
 default - cislo, ktere je v okne zobrazeno na zacatku a funkce ho vraci v
           pripade, ze je zadavani zruseno
Pokud napisete nejakou kravinu (pismena misto cisel apod.), bude automaticky
smazana a cislo zustane v okne, abyste ho mohli pripadne jeste opravit.}
function Nabidka(ix,iy:integer; texty:string):byte;
{Male menu podobne mistni nabidce ve Windows. Ovlada se mysi (klik na polozku
znamena vyber, klik vedle = zrusit) nebo klavesnici (sipky, Enter, Esc).
 ix,iy - levy horni roh (vycuhovani z obrazovky je osetreno, nedojde k nemu)
 texty - seznam textu jednotlivych polozek. Radky oddelujte znakem '|', radek
         s textem '-' znamena delici caru.
Vraci cislo vybrane polozky, v pripade zruseni (Esc nebo klik vedle) nulu.
Prvni polozka ma cislo 1, ostatni se pocitaji tak, ze delici cary
se preskoci. Priklad pouziti:
 case nabidka(xmys,ymys,'prvni polozka|druha polozka|-|treti polozka') of ...}
function VyberBarvuOkno(nadpis:nadpstr; default:byte):byte;
{Okno s vyberem barvy pro 256 barev, vraci cislo te vybrane. Ovlada se mysi
nebo klavesnici (sipky, Home, End, Enter/O, Esc/Z).
 nadpis - je zobrazen v horni liste okna, mel by mit max. 19 znaku (jestli
          bude delsi, okno se automaticky rozsiri, ale uz nebude vypadat tak
          hezky). Pokud zadate '', zobrazi se 'Vyber barvu:'.
 default - vychozi barva, ktera je vybrana na zacatku a kterou funkce vraci
           v pripade zruseni}
procedure ZjistujSouradnice;
{Zobrazi zamerovaci kriz, se kterym pohybujete pomoci mysi, a male okenko s
jeho aktualnimi souradnicemi. Velmi uzitecna a casto pouzivana vecicka, kdyz
potrebujete zjistit, kam co na obrazovku umistit. Pro libovolne rozliseni.
Barvy: okenko se souradnicemi podle Infokna, zamerovaci kriz je bily
(barva 15) a xoruje se s pozadim.}

function VyberSouborOkno(nadpis:nadpstr; koncovky:string3; adresare:dirstr):string;
{Necha uzivatele vybrat soubor a vrati jeho jmeno.
 nadpis - zobrazi se v horni liste okna. Pokud je '', zobrazi se
          'Vyber soubor:'.
 koncovky - jake soubory se maji zobrazovat. Napr.: 'txt', 'PAS',
            '*' (jakakoli koncovka), '' (bez koncovky) apod. Na velikosti
            pismen nezalezi.
 adresare - '' => zobrazi se obsah aktualniho adresare, seznam podadresaru
                  (obvykle i '..'), stavovy radek s aktualni cestou
                  a tlacitko pro prepinani mezi disky. Jde prochazet
                  mezi adresari. Jmeno souboru se vraci i s kompletni
                  cestou, napr. 'C:\TP\SOUBOR.BLA'.
            jmeno nejakeho konkretniho adresare => zobrazi se jen tento jeden
                adresar, nejde mezi nimi prochazet ani prepinat mezi disky.
                Jmeno souboru bude vraceno obvykle s cestou relativni k
                aktualnimu adresari. Napr.: '.' znamena aktualni adresar,
                vrati se 'SOUBOR.BLA'. 'SAVE' znamena podadresar SAVE umisteny
                v aktualnim adresari, vrati se 'SAVE\SOUBOR.BLA'. 'C:' je
                korenovy adresar, vrati se 'C:\SOUBOR.BLA'. Funguji i adresare
                '\' (koren) a '..' (o uroven vys). Znak '\' na konci cesty byt
                nemusi, ale muze.
Pri pouziti tlacitka Novy se novy soubor fyzicky nevytvori, jen se vrati
zadane jmeno. Jmeno se zadava bez koncovky a dostane implicitne tu, ktera byla
uvedena v parametru Koncovky (To neni chyba, je to schvalne. Jenom ale tohle
tlacitko nepouzivejte, kdyz je ta koncovka treba '*' nebo tak neco - nejake
jmeno by se sice vratilo, ale asi by bylo nepouzitelne.).
Pri pouziti tlacitka Zrus (nebo klavesy Esc) funkce vraci ''(prazdny retezec).
Mezi adresari se prochazi jednim kliknutim, soubor je potreba oznacit a pak
bud kliknout podruhe, nebo na OK, nebo stisknout Enter.
Po skonceni funkce bude nastaven stejny adresar, jako byl pred jejim volanim.
Barvy: pole se seznamy souboru a adresaru a texty v nich jako Textovepole,
okraje stavoveho radku s cestou jako Okno, text cesty jako texty u Anoneokna}
function VlozMaskuOkno(nadpis:nadpstr; default:word):word;
{Pro zadavani wordu po bitech - bitove masky pro caru z jedotky VESA.
Levym mysitkem se kresli, pravym maze. Kdyz date nadpis '', zobrazi se
'Maska ry:'.}
procedure VlozVzorekOkno(nadpis:nadpstr; var vzorek:_pole8bytu);
{Pro zadavani 8bytoveho pole po bitech - bitove masky pro obdelnik z jednotky
VESA. Levym mysitkem se kresli, pravym maze. Kdyz date nadpis '', zobrazi se
'Vzorek vpln:'}
procedure NastavMysOkno(ukladat:boolean);
{Necha uzivatele nastavit vodorovnou a svislou citlivost mysi, prohodit leve a
prave tlacitko a pripadne zapnout korekci pohybu, pokud se mu kurzor
nepohybuje plynule (nektere mysi ovladace to delaji). Parametr rika, jestli se
ma dovolit ukladani nastaveni do hlavniho konfiguracniho souboru (Cfgsoubor z
jednotky CFG).
Horke klavesy: Enter udela totez, co tlacitko "Nastav, ale neukladej",
Esc jako "Zrus". Sipky nahoru/dolu nastavuji svislou citlivost,
sipky doleva/doprava vodorovnou.}
procedure NastavKlOkno(Popisky:uknaradek; var Rozlozeni);
{Dialog pro prirazeni klavesovych kombinaci k danym akcim. Vystupem procedury
je pole kodu klaves, ktere uz si ve svem programu musite nejak zpracovat sami.
0 znamena nedefinovano (xreadkey nulu nikdy nevraci, takze zamena nehrozi).
 Popisky - nazvy jednotlivych akci; co radek, to jeden nazev. Maximalni delka
           jednoho je 20 znaku; pokud je delsi, orizne se.
 Rozlozeni - pole kodu klaves: array[0..pocet_popisek-1] of word (kody od
             funkce xReadkey z jednotky Klavesy).
             Poradi kodu v poli musi samozrejme odpovidat poradi nazvu akci
             v Popiskach, jinak je procedura k nicemu!
             Kazda zmena, kterou uzivatel provede, se do tohoto pole okamzite
             uklada bez moznosti zruseni nebo vraceni.}

(***************************) implementation (*******************************)
uses klavesy,errmsg,cfg;

function wzakladobj.zmeneno:boolean;
Begin
zmeneno:=_zmeneno;
_zmeneno:=false;
End;{wzakladobj.zmeneno}

procedure tlacitko.init(ix,iy:integer;inapis,ipopis:string;ipridavek:byte);
Begin
x:=ix; y:=iy;
napis:=inapis;
popis:=ipopis;
pridavek:=ipridavek;
hodnota:=0;
self.zobraz;
End;{tlacitko.init}

procedure tlacitko.zobraz;
Begin
_zmertext(napis);
sirka:=_sirkatextu+6;
vyska:=_vyskatextu+6+pridavek shl 1;
kvyp;
_box(x,y,x+sirka,y+vyska,tlhbarva,tpdbarva,tvbarva);
_print(x+3-_deltax,y+(vyska shr 1)-(_vyskatextu shr 1)-_deltay,ttxtbarva,napis);
kzap;
End;{tlacitko.zobraz}

procedure tlacitko.kontrola;
Begin
if mysvpoli(x,y,x+sirka,y+vyska) then
  begin
  hodnota:=stavtlacitek;{ulozeni, cim bylo kliknuto}
  if stavtlacitek<>0 then begin{animace kliknuti tlacitka}
                          kvyp;
                          _box(x,y,x+sirka,y+vyska,tpdbarva,tlhbarva,0);{stisknute - obracene stinovany ramecek}
                          kzap;
                          resetuj;{pockani na uvolneni mysi}
                          kvyp;
                          _box(x,y,x+sirka,y+vyska,tlhbarva,tpdbarva,0);{pusteno - normalni ramecek}
                          kzap;
                          end;
  if (hodnota=2)and(popis<>'') then begin{kdyz je zadany nejaky popis, zobrazi se a nebere se to jako kliknuti}
                                    infokno(xmys,ymys,popis,true);
                                    hodnota:=0;
                                    end;
  end
 else hodnota:=0;{mys je mimo tlacitko}
End;{tlacitko.kontrola}

procedure tlacitko2.init(ix,iy:integer;ipopis:string);
Begin
x:=ix; y:=iy;
popis:=ipopis;
hodnota:=0;
self.zobraz;
End;{init}

procedure tlacitko2.zobraz;
Begin
kvyp;
with obrazky do _putimage(x,y,sirka,vyska,data^[1]);
kzap;
End;{tlacitko2.zobraz}

procedure tlacitko2.kontrola;
Begin
with obrazky do if mysvpoli(x,y,x+sirka-1,y+vyska-1)
  then begin
       hodnota:=stavtlacitek;
       if stavtlacitek<>0 then begin{animace kliknuti}
                               kvyp;
                               _putimage(x,y,sirka,vyska,data^[2]);{stisknuto - druhy obrazek}
                               kzap;
                               resetuj;{pockani na uvolneni mysi}
                               kvyp;
                               _putimage(x,y,sirka,vyska,data^[1]);{pusteno - prvni obrazek}
                               kzap;
                               end;
       if (hodnota=2)and(popis<>'') then begin
                                         infokno(xmys,ymys,popis,true);
                                         hodnota:=0;
                                         end;
       end
  else hodnota:=0;{mys je mimo tlacitko}
End;{tlacitko2.kontrola}

procedure tlacitko2.zrus;
Begin
zrusobrazky(obrazky);
End;{tlacitko2.zrus}

procedure zatrzitko.init(ix,iy:integer;itextik,ipopis:string;ihodnota:boolean);
Begin
x:=ix; y:=iy;
textik:=itextik; popis:=ipopis;
hodnota:=ihodnota;
_zmeneno:=false;
self.zobraz;
End;{zatrzitko.init}

procedure KresliZatrzitko(_x,_y:word;jo:boolean);
Begin
if jo then begin
           _putpixel(_x+2,_y+4,zzbarva);
           _bar(_x+2,_y+5,_x+3,_y+6,zzbarva);
           _line(_x+3,_y+7,_x+8,_y+2,zzbarva);
           _line(_x+4,_y+7,_x+8,_y+3,zzbarva);
           _line(_x+4,_y+8,_x+8,_y+4,zzbarva);
           end
      else _bar(_x+1,_y+1,_x+9,_y+9,zvbarva);
End;{kreslizatrzitko}

procedure zatrzitko.zobraz;
Begin
_zmertext(textik);
sirka:=_sirkatextu+14;
vyska:=_vyskatextu; if vyska<10 then vyska:=10;
kvyp;
_box(x,y,x+10,y+10,zlhbarva,zpdbarva,zvbarva);
kreslizatrzitko(x,y,hodnota);
_print(x+14-_deltax,y+1-_deltay,ztxtbarva,textik);
kzap;
End;{zatrzitko.zobraz}

procedure zatrzitko.kontrola;
Begin
if mysvpoli(x,y,x+sirka,y+vyska) then
 if lmys then begin
              hodnota:=not hodnota;
              _zmeneno:=true;
              kvyp;
              kreslizatrzitko(x,y,hodnota);
              kzap;
              resetuj;
              end
         else if pmys and(popis<>'') then infokno(xmys,ymys,popis,true);
End;{zatrzitko.kontrola}

procedure prepinac.init(ix,iy:integer; inadpis,itexty,ipopis:string; ihodnota:byte);
var pocr:word;
Begin
x:=ix; y:=iy;
nadpis:=inadpis;
popis:=ipopis;
{$ifdef blbuvzdornost}
while itexty[1]='|' do delete(itexty,1,1);{orezani kravin ze zacatku}
while itexty[byte(itexty[0])]='|' do dec(byte(itexty[0]));{orezani kravin z konce}
{pozn.: byte(retezec[0]) = length(retezec)}
 repeat{nalezeni a odstraneni prazdnych radku}
 i:=pos('||',itexty);
 if i<>0 then delete(itexty,i,1);
 until i=0;
{$endif}
{rozlozeni textu do radku:}
texty:=nil;
nactitexty(texty,itexty,250); {delka radku je tedy prakticky neomezena a je na vas, aby se pak vesly na obrazovku}
{pocatecni hodnota:}
pocr:=pocetradkuvseznamu(texty);
if ihodnota>pocr then hodnota:=pocr
                 else hodnota:=ihodnota;
_zmeneno:=false;
{zobrazeni:}
zobraz;
End;{prepinac.init}

procedure puntik(x,y:integer;barva:byte);{nakresli puntik do kolecka prepinace}
Begin
_hline(x+3,x+4,y+2,barva);
_bar(x+2,y+3,x+5,y+4,barva);
_hline(x+3,x+4,y+5,barva);
End;{puntik}

procedure prepinac.zobraz;
{}procedure kolecko(y:integer);{nakresli stinovane bile kolecko, x = x celeho prepinace}
{}Begin
{}_hline(x+2,x+5,y,plhbarva);{horni okraj}
{}_bar(x+1,y+1,x+6,y+6,pvbarva);{vnitrek}
{}_putpixel(x+1,y+1,plhbarva); _putpixel(x+6,y+1,plhbarva);{dva body u horniho kraje}
{}_vline(x,y+2,y+5,plhbarva); _vline(x+7,y+2,y+5,ppdbarva);{bocni strany}
{}_putpixel(x+1,y+6,ppdbarva); _putpixel(x+6,y+6,ppdbarva);{body u dolniho kraje}
{}_hline(x+2,x+5,y+7,ppdbarva);{spodni okraj}
{}End;{kolecko}
var i:word;
    vybrany:uknaradek;
Begin{prepinac.zobraz}
kvyp;
{vypocet rozmeru podle nastaveni textu:}
sirka:=_sirkatextu2(nejdelsiradek(texty))+12;{cili sirka nejdelsiho radku plus pridavek na klikaci kolecko}
vyskaradku:=_vyskatextu2+2;
{zobrazeni nadpisu:}
_zmertext(nadpis);
_print(x,y-_deltay-_vyskatextu-4,ptxtbarva,nadpis);
{vykresleni jednotlivych polozek:}
vybrany:=texty; i:=0;
while vybrany<>nil do begin
                      kolecko(y+i*vyskaradku);
                      _print(x+13,y+i*vyskaradku+1,ptxtbarva,vybrany^.obsah);
                      vybrany:=vybrany^.dalsi; inc(i);
                      end;
{oznaceni vybrane polozky:}
if hodnota<>0 then puntik(x,y+(hodnota-1)*vyskaradku,ppbarva);
kzap;
End;{prepinac.zobraz}

procedure prepinac.kontrola;
var NovaHodnota:integer;
    PocetRadku:word;
Begin
pocetradku:=pocetradkuvseznamu(texty);
if mysvpoli(x,y,x+sirka,y+pocetradku*vyskaradku) then{mys je nad aktivni oblasti prepinace}
 if lmys then begin{kliknuto levym tlacitkem - akce}
              {vypocet cisla radku z polohy mysi:}
              novahodnota:=((ymys-y)div vyskaradku)+1;
              {orezani teto hodnoty do rozumnych mezi (to kdyby treba nekdo kliknul a pak ujel mysi mimo oblast):}
              if novahodnota<1 then novahodnota:=1;
              if novahodnota>pocetradku then novahodnota:=pocetradku;
              if novahodnota<>hodnota
                then begin{kliklo se na jinou polozku nez byla oznacena minule}
                     _zmeneno:=true;
                     kvyp;
                     if hodnota<>0 then puntik(x,y+(hodnota-1)*vyskaradku,pvbarva);{vymazani puv. puntiku}
                     hodnota:=novahodnota;
                     puntik(x,y+(hodnota-1)*vyskaradku,ppbarva);{nakresleni noveho}
                     kzap;
                     end;
              resetuj;{cekani na pusteni mysi}
              end
         else if pmys then if popis<>'' then infokno(xmys,ymys,popis,true){kliknuto pravym tlacitkem - info}
                                        else resetuj;
End;{prepinac.kontrola}

procedure prepinac.zrus;
Begin
if texty<>nil{tahle kontrola sice neni stoprocentne ucinna, ale aspon zabrani ruseni prepinace vickrat po sobe}
  then zrustexty(texty);
End;{prepinac.zrus}


{Cela Roleta by teoreticky sla vyrobit z Textoveho pole, Tlacitka a Nabidky,
ale Nabidka je pro tyto ucely malo pruzna (bez posuvniku, zadavani jen z
jednoho retezce apod.), takze to musim rozepsat.}

procedure roleta.nuluj;
Begin
polozky:=nil;{procedura Nactitexty tohle nutne potrebuje, aby vedela, ze je seznam jeste prazdny}
vybrana:=nil;
hodnota:=0;
_zmeneno:=false;
End;{roleta.nuluj}

procedure roleta.PridejPolozky(zdroj:string);
Begin
nactitexty(polozky,zdroj,200);{odradkovavani polozek si (predpokladam) bude
 kazdy delat rucne, takze na sirce radku nezalezi}
End;{roleta.PridejPolozky}

procedure roleta.init(ix,iy:integer; ihodnota:word; ipopis:string);
var w:word;
Begin
x:=ix; y:=iy;
popis:=ipopis;
maxdelka:=nejdelsiradek(polozky);
if maxdelka<1 then maxdelka:=1;{pojistka proti nulove sirce}
maxhodnota:=pocetradkuvseznamu(polozky);
{nalezeni vybrane polozky:}
if (polozky=nil)or(ihodnota=0)
  then begin
       hodnota:=0;
       vybrana:=nil;
       end
  else begin
       hodnota:=1; vybrana:=polozky;
       while (vybrana<>nil)and(vybrana^.dalsi<>nil)and(hodnota<ihodnota) do
         begin
         vybrana:=vybrana^.dalsi;
         inc(hodnota);
         end;
       end;
zobraz;
End;{roleta.init}

procedure roleta.zobraz;
Begin
sirka1:=_sirkatextu2(maxdelka)+8;{sirka bileho pole s aktualnim vyberem}
sirka2:=sirka1+_sirkatextu2(1)+7;{sirka celeho objektu vcetne rozbalovaciho tlacitka}
vyska:=_vyskatextu2+6;
kvyp;
{textove pole:}
_box(x,y,x+sirka1,y+vyska,slhbarva,spdbarva,svbarva);
{obsah pole:}
if polozky=nil then begin{kdyz nejsou zadne moznosti, radsi okenko preskrtneme, aby to nevypadalo, ze nefunguje}
                    _line(x+1,y+1,x+sirka1-1,y+vyska-1,sskrtbarva);
                    _line(x+1,y+vyska-1,x+sirka1-1,y+1,sskrtbarva);
                    end
               else if vybrana<>nil then _print(x+4,y+3,stxtbarva,vybrana^.obsah);
{"tlacitko":}
_box(x+sirka1+1,y,x+sirka2,y+vyska,tlhbarva,tpdbarva,tvbarva);
_print(x+sirka1+4,y+3,ttxtbarva,'');
kzap;
End;{roleta.zobraz}

procedure roleta.kontrola;
var RadkuVRolete:byte;{max. pocet radku zobrazenych v rolete najednou}
    pozice:integer;{na jake strance zrovna jsme, cislovano od 0, odpovida po.hodnota}
    VyskaRolety:word;{[pix]}
    i,j,k:word;{pocitadla}
    po:posuvnik;{posuvnik v rolete}
    PozRol:sprite;{pozadi rolety}
    koncit,{ukoncovaci podminka cyklu}
    pozadiB:boolean;{jestli se povedlo nacist pozadi rolety}
    p:uknaradek;{pomocny}
    aktivni:uknaradek;{ukazuje na radek, ktery je prave aktivni (je nad nim mys)}
    ap:integer;{index aktualne aktivni polozky v rolete pocitany od 0 od zacatku aktualni pozice; zaporny=nic}
{}procedure KresliRadek(index:byte; _aktivni:boolean; zdroj:uknaradek);{vykresli radek v rolete}
{}var bt,bp:byte;
{}    s:word;
{}Begin
{}if _aktivni then begin bt:=sratxtbarva; bp:=sravbarva; end
{}            else begin bt:=srntxtbarva; bp:=srvbarva; end;
{}if po.pocetpoloh=1 then s:=sirka2
{}                   else s:=sirka1;
{}_bar(x+1,y+vyska+2+(_vyskatextu2+2)*index,x+s-1,y+vyska+1+(_vyskatextu2+2)*(index+1),bp);
{}if zdroj<>nil then _print(x+2,y+vyska+3+(_vyskatextu2+2)*index,bt,zdroj^.obsah);
{}End;{kresliradek}
Begin
if mysvpoli(x,y,x+sirka2,y+vyska) then
  if lmys and (polozky<>nil)
    then begin
         {vykresleni stisknuteho tlacitka:}
         kvyp;
         _box(x+sirka1+1,y,x+sirka2,y+vyska,tpdbarva,tlhbarva,0);
         {vypocet vysky rolety:}
         if maxhodnota<=10 then radkuvrolete:=maxhodnota
                           else begin
                                radkuvrolete:=10;
                                {vypocet optimalniho poctu radku v rolete:}
                                while maxhodnota div (radkuvrolete-1)=maxhodnota div radkuvrolete do dec(radkuvrolete);
         {tj. snizujeme pocet, dokud se nezvetsuje pocet stranek, cimz zmensime prazdne misto na konci posledni stranky rolety}
                                end;
         vyskarolety:=(_vyskatextu2+2)*radkuvrolete+2;
         pozadib:=pozrol.init(sirka2+1,vyskarolety);
         {zobrazeni rozbalene rolety:}
         pozrol.nacti(x,y+vyska+1);
         _box(x,y+vyska+1,x+sirka2,y+vyska+vyskarolety,srlhbarva,srpdbarva,srvbarva);
         pozice:=(integer(hodnota)-1) div radkuvrolete;
         if pozice<0 then pozice:=0;
         po.pocetpoloh:=maxhodnota div radkuvrolete+ord(maxhodnota mod radkuvrolete>0);
         if po.pocetpoloh>1 then po.init(x+sirka1+1,y+vyska+1,vyskarolety-2-2*vyska,po.pocetpoloh,pozice)
                            else po.hodnota:=pozice;
         p:=polozky;
         for i:=1 to pozice do
          for j:=1 to radkuvrolete do p:=p^.dalsi;
         kvyp;{posuvnik ho pri inicializaci zapnul}
         for i:=0 to radkuvrolete-1 do begin
                                       kresliradek(i,false,p);
                                       if p<>nil then p:=p^.dalsi;
                                       end;
         aktivni:=nil; ap:=-1;
         koncit:=false;
         kzap;
         resetuj;
         {interakce s uzivatelem:}
          repeat
          cekejnacokoli;
          if po.pocetpoloh>1 then po.kontrola;
          if stavtlacitek<>0 then begin
                                  if not mysvpoli(x,y+vyska+1,x+sirka2,y+vyska+vyskarolety-1) then aktivni:=nil;
                                  koncit:=true;
                                  end
           else if pozice<>po.hodnota then begin
                                           pozice:=po.hodnota;
                                           p:=polozky;
                                           for i:=1 to pozice do
                                            for j:=1 to radkuvrolete do p:=p^.dalsi;
                                           kvyp;
                                           for i:=0 to radkuvrolete-1 do
                                             begin
                                             kresliradek(i,false,p);
                                             if p<>nil then p:=p^.dalsi;
                                             end;
                                           kzap;
                                           aktivni:=nil;
                                           ap:=-1;
                                           end
            else if keypressed then case readkey of #13:koncit:=true;
                                                    #27:begin
                                                        aktivni:=nil;
                                                        koncit:=true;
                                                        end;
                                                    else kresetuj;
                                                    end
             else if mysvpoli(x,y+vyska+1,x+sirka2,y+vyska+vyskarolety-1)
                    then begin {mys se pohnula a je v oblasti rolety}
                         i:=(ymys-y-vyska-2)div(_vyskatextu2+2);
                         kvyp;
                         {deaktivace aktivniho:}
                         if (ap>=0)and(i<>ap) then begin
                                                   kresliradek(ap,false,aktivni);
                                                   ap:=-1;
                                                   end;
                         {aktivace noveho:}
                         if ap<0 then begin
                                      ap:=i;
                                      j:=i+pozice*radkuvrolete;{skutecne cislo polozky}
                                      aktivni:=polozky;
                                      for k:=1 to j do if aktivni<>nil then aktivni:=aktivni^.dalsi;
                                      kresliradek(ap,true,aktivni);
                                      end;
                         kzap;
                         end
                    else begin {mys se pohnula a je mimo roletu}
                         if (aktivni<>nil)and(ap>=0) then begin{deaktivace aktivniho}
                                                          kvyp;
                                                          kresliradek(ap,false,aktivni);
                                                          kzap;
                                                          ap:=-1;
                                                          aktivni:=nil;
                                                          end;
                         end;
          until koncit;
         {smazani rolety:}
         kvyp;
         if pozadib then begin
                         pozrol.zobraz(x,y+vyska+1);
                         pozrol.zrus;
                         end
                    else _bar(x,y+vyska+1,x+sirka2,y+vyska+vyskarolety,opozadinamazani);
         kzap;
         {vypocet hodnoty:}
         if aktivni<>nil then begin
                              vybrana:=polozky;
                              i:=1;
                              while vybrana<>aktivni do begin
                                                        inc(i);
                                                        vybrana:=vybrana^.dalsi;
                                                        end;
                              end
                         else i:=hodnota;
         self.zobraz;{potrebuje jenom ukazatel Vybrana, ne Hodnotu}
         resetuj;
         if i<>hodnota then begin
                            hodnota:=i;
                            _zmeneno:=true;
                            end;
         end
    else if pmys and (popis<>'') then infokno(xmys,ymys,popis,true);
End;{roleta.kontrola}

procedure roleta.zrus;
Begin
zrustexty(polozky);{ukazatel Polozky bude mit po tomhle hodnotu nil}
hodnota:=0;
vybrana:=nil;
End;{roleta.zrus}

procedure ciselnaosa.init(ix,iy:integer;isirka:word;imin,imax,ihodnota:longint;inadpis,itextmin,itextmax:string;
                          icisla:boolean;ipozadi:byte);
Begin
x:=ix; y:=iy;
sirka:=isirka;
min:=imin; max:=imax;
if max<=min then max:=min+1;{blbuvzdornost musi byt ;-)}
hodnota:=ihodnota;
_zmeneno:=false;
if hodnota<min then hodnota:=min    {-''-}
               else if hodnota>max then hodnota:=max;
nadpis:=inadpis; textmin:=itextmin; textmax:=itextmax;
psatcisla:=icisla;
pozadi:=ipozadi;
zobraz;
End;{ciselnaosa.init}

procedure ciselnaosa.zobraz;
var pomstr:string[11];
    pixhodnota:word;
Begin
{tlacitka:}
ubert.init(x,y,#17,'',0);
x2:=x+ubert.sirka+5;
pridejt.init(x2+sirka+4,y,#16,'',0);
kvyp;
{nadpisy:}
_zmertext(nadpis);                   {bez popisku pro min. a max. hodnotu:}
if (textmin='')and(textmax='') then _print(x2+(sirka-1) shr 1-_sirkatextu shr 1,y-_vyskatextu-2,cotxtbarva,nadpis)
                               else begin    {s popisky:}
                                    _print(x2+(sirka-1) shr 1-_sirkatextu shr 1,y-_vyskatextu2-_vyskatextu-4,
                                           cotxtbarva,nadpis);
                                    _print(x,y-_vyskatextu2-2,cotxtbarva,textmin);
                                    _print(pridejt.x+pridejt.sirka-_sirkatextu2(byte(textmax[0])),y-_vyskatextu2-2,
                                           cotxtbarva,textmax);
                                    end;
{min. a max. hodnota ciselne:}
if psatcisla then begin
                  str(min,pomstr);
                  _print(x,y+ubert.vyska+3,cotxtbarva,pomstr);
                  str(max,pomstr);
                  _print(pridejt.x+pridejt.sirka-_sirkatextu2(byte(pomstr[0])),y+ubert.vyska+3,cotxtbarva,pomstr);
                  end;
_bar(x2-3,y,x2+sirka+2,y+ubert.vyska,pozadi);{smazani posuvniku}
{"drazka" posuvniku:}
_box(x2,y+ubert.vyska shr 1-1,x2+sirka-1,y+ubert.vyska shr 1+1,colhbarva,copdbarva,covbarva);
_maskedline(x2,y+ubert.vyska shr 1+4,x2+sirka-1,y+ubert.vyska shr 1+4,cotxtbarva,$4040);
_maskedline(x2,y+ubert.vyska shr 1+5,x2+sirka-1,y+ubert.vyska shr 1+5,cotxtbarva,$4040);
{nakresleni toho posuvneho tentononcu:}
pixhodnota:=((hodnota-min)*sirka)div(max-min+1);{souradnice stredu ramecku od leveho kraje osy}
_box(x2+pixhodnota-3,y,x2+pixhodnota+3,y+ubert.vyska,tlhbarva,tpdbarva,tvbarva);
{vypsani aktualni hodnoty:}
if psatcisla then begin
                  str(hodnota,pomstr);
                  napis(x2+sirka shr 1-_sirkatextu2(byte(pomstr[0])) shr 1,y+ubert.vyska+3,cotxtbarva,pozadi,pomstr);
                  end;
kzap;
End;{ciselnaosa.zobraz}

procedure ciselnaosa.kontrola;
{}procedure prekresli;{smaze posuvny obdelnicek a znovu ho vykresli ve spravne poloze}
{}var pixhodnota:word;{xova souradnice stredu obdelnicku pocitana od x2}
{}    pomstr:string[11];
{}Begin
{}if hodnota>max then hodnota:=max                  {kontrola rozsahu hodnoty}
{}               else if hodnota<min then hodnota:=min;
{}pixhodnota:=((hodnota-min)*sirka)div(max-min+1);{vypocet polohy obdelnicku z hodnoty}
{}kvyp;
{}_bar(x2-3,y,x2+sirka+2,y+ubert.vyska,pozadi);{smazani vseho}
{}{nakresleni drazky:}
{}_box(x2,y+ubert.vyska shr 1-1,x2+sirka-1,y+ubert.vyska shr 1+1,colhbarva,copdbarva,covbarva);
{}_maskedline(x2,y+ubert.vyska shr 1+4,x2+sirka-1,y+ubert.vyska shr 1+4,cotxtbarva,$4040);
{}_maskedline(x2,y+ubert.vyska shr 1+5,x2+sirka-1,y+ubert.vyska shr 1+5,cotxtbarva,$4040);
{}{nakresleni posuvneho ramecku:}
{}_box(x2+pixhodnota-3,y,x2+pixhodnota+3,y+ubert.vyska,tlhbarva,tpdbarva,tvbarva);
{}if psatcisla then    {vypis aktualni hodnoty pod osu}
{} begin
{} str(hodnota,pomstr);
{} pomstr:='  '+pomstr+'  ';
{} napis(x2+sirka shr 1-_sirkatextu2(byte(pomstr[0])) shr 1,y+ubert.vyska+3,cotxtbarva,pozadi,pomstr);
{} end;
{}kzap;
{}End;{prekresli}
var PuvodniHodnota:longint;
Begin
puvodnihodnota:=hodnota;
ubert.kontrola; pridejt.kontrola;
if mysvpoli(x2,y,x2+sirka-1,y+ubert.vyska)
 then if lmys then begin
                   hodnota:=(((max-min+1)*(xmys-x2)) div sirka)+min;{vypocet hodnoty z polohy mysi}
                   prekresli;
                   end
              else if pmys then infokno(xmys,ymys,wt.dej(wtcohelp),true);
if ubert.hodnota<>0 then begin
                         if ubert.hodnota=1 then if shiftpressed then dec(hodnota,100)
                                                                 else dec(hodnota,1)
                                            else if ubert.hodnota=2 then if shiftpressed then dec(hodnota,1000)
                                                                                         else dec(hodnota,10);
                         prekresli;
                         ubert.hodnota:=0;
                         end;
if pridejt.hodnota<>0 then begin
                           if pridejt.hodnota=1 then if shiftpressed then inc(hodnota,100)
                                                                     else inc(hodnota,1)
                                                else if pridejt.hodnota=2 then if shiftpressed then inc(hodnota,1000)
                                                                                               else inc(hodnota,10);
                           prekresli;
                           pridejt.hodnota:=0;
                           end;
if hodnota<>puvodnihodnota then _zmeneno:=true;
End;{ciselnaosa.kontrola}

procedure posuvnik.init(ix,iy:integer;ivyska,ipocetpoloh,ihodnota:word);
Begin
x:=ix; y:=iy;
{dany pocet poloh - musi byt aspon jeden:}
if ipocetpoloh=0 then pocetpoloh:=1
                 else pocetpoloh:=ipocetpoloh;
{pocatecni poloha - pokud je mimo zadany rozsah, musi se mu prizpusobit:}
if ihodnota<=pocetpoloh-1 then hodnota:=ihodnota
                          else hodnota:=pocetpoloh-1;
{vyska jednoho bloku (a posuvneho ramecku) - musi byt aspon 1:}
vyskapolohy:=ivyska div pocetpoloh;
if vyskapolohy=0 then vyskapolohy:=1;
_zmeneno:=false;
self.zobraz;
End;{posuvnik.init}

procedure posuvnik.zobraz;
Begin
{horni tlacitko - urci rozmery posuvniku:}
nahorut.init(x,y,'','',0);
kvyp;
with nahorut do begin
                {stredni cast posuvniku:}
                _bar(x,y+vyska+1,x+sirka,y+vyska+pocetpoloh*vyskapolohy,posbarva);
                {posuvny ramecek:}
                _box(x,y+vyska+1+self.hodnota*vyskapolohy,x+sirka,y+vyska+self.hodnota*vyskapolohy+vyskapolohy,
                     tlhbarva,tpdbarva,tvbarva);
                end;
{dolni tlacitko:}
dolut.init(x,y+nahorut.vyska+vyskapolohy*pocetpoloh+1,'','',0);
End;{posuvnik.zobraz}

procedure posuvnik.kontrola;
{}procedure vykresli;
{}Begin
{}kvyp;
{}with nahorut do begin
{}                _bar(x,y+nahorut.vyska+1,x+nahorut.sirka,y+nahorut.vyska+pocetpoloh*vyskapolohy,posbarva);
{}                _box(x,y+nahorut.vyska+1+self.hodnota*vyskapolohy,x+nahorut.sirka,
{}                     y+nahorut.vyska+self.hodnota*vyskapolohy+vyskapolohy,tlhbarva,tpdbarva,tvbarva);
{}                end;
{}kzap;
{}End;{vykresli}
var PuvodniHodnota:word;
Begin{posuvnik.kontrola}
puvodnihodnota:=hodnota;
dolut.kontrola; nahorut.kontrola;
if mysvpoli(x,y+nahorut.vyska+1,x+nahorut.sirka,y+nahorut.vyska+pocetpoloh*vyskapolohy) and (stavtlacitek<>0)
 then begin
      hodnota:=(ymys-(y+nahorut.vyska+1)) div vyskapolohy;
      vykresli;
      resetuj;
      end
  else if (dolut.hodnota<>0)and(hodnota<pocetpoloh-1)then begin
                                                          inc(hodnota);
                                                          vykresli;
                                                          end
   else if (nahorut.hodnota<>0)and(hodnota>0) then begin
                                                   dec(hodnota);
                                                   vykresli;
                                                   end;
if hodnota<>puvodnihodnota then _zmeneno:=true;
End;{posuvnik.kontrola}

procedure hotspot.init(ia,ib,ic,id:integer; ires:boolean; ipopis:string);
Begin
a:=ia; b:=ib; c:=ic; d:=id; resetovat:=ires;
hodnota:=0; _zmeneno:=false; popis:=ipopis;
End;{hotspot.init}

procedure hotspot.kontrola;
var puvodnihodnota:byte;
    puvodnix,puvodniy:word;
Begin
hodnota:=puvodnihodnota; puvodnix:=xkliku; puvodniy:=ykliku;
if mysvpoli(a,b,c,d)
  then begin
       hodnota:=stavtlacitek;
       if (hodnota=2)and(popis<>'') then begin
                                         infokno(xmys,ymys,popis,true);
                                         hodnota:=0;
                                         end;
       if hodnota<>0 then begin
                          xkliku:=xmys-a;
                          ykliku:=ymys-b;
                          if resetovat then resetuj;
                          end;
       end
  else hodnota:=0;
if (hodnota<>puvodnihodnota)or(puvodnix<>xkliku)or(puvodniy<>ykliku) then _zmeneno:=true;
End;{hotspot.kontrola}

procedure hotspot.zobraz(barva:byte);
Begin
_rectangle(a,b,c,d,barva);
End;{hotspot.zobraz}

procedure textovepole.init(ix,iy:integer;imaxzn:byte;itext:string;iakt,iprosoubor:boolean);
Begin
x:=ix; y:=iy;
maxznaku:=imaxzn;
if byte(itext[0])>maxznaku then byte(itext[0]):=maxznaku;{oriznuti retezce, pokud je moc dlouhy}
hodnota:=itext;
default:=itext;
_zmeneno:=false;
if iakt then stav:=_aktivni
        else stav:=_neaktivni;
pozice:=-1;{pro zacatek oznacime cely text}
prosoubor:=iprosoubor;
predchozi:=nil; dalsi:=nil;
self.zobraz;
End;{textovepole.init}

procedure textovepole.zobraz;
var rk:byte;
Begin
rk:=_ridicikody; _ridicikody:=_vypisovat;
{vypocet rozmeru:}
sirka:=maxznaku*__roztec+5;
vyska:=_vyskatextu2+5;
kvyp;
{ramecek:}
_box(x,y,x+sirka,y+vyska,tplhbarva,tppdbarva,tpvbarva);
if (stav=_aktivni)and(pozice>=0) then begin
                                      _print(x+4,y+3,tptxtbarva,hodnota);{text}
                                      _vline(x+3+__roztec*pozice,y+2,y+3+_vyskatextu2,tpkbarva);{kurzor}
                                      end
 else if (stav=_escapem)or((stav=_aktivni)and(pozice=-1)) then napis(x+4,y+3,tpozntxtbarva,tpozntxtpodbarva,hodnota)
  else _print(x+4,y+3,tptxtbarva,hodnota);{text bez kurzoru}                                            {oznaceny text}
kzap;
_ridicikody:=rk;
End;{textovepole.zobraz}

procedure textovepole.kontrola;
const povoleneznaky=['A'..'Z','a'..'z','0'..'9','.','_','\','~','-',':'{,'*','?'}];{znaky povolene v nazvech souboru}
var o:char;{pro cteni klavesnice}
    pomstr:string[1];{kvuli Insertu (char mu nestaci)}
    puvodnihodnota:string;
    puvodnistav:byte;
Begin
puvodnistav:=stav; puvodnihodnota:=hodnota;
if (stav=_aktivni)or(stav=_escapem) then
  begin
  if stavtlacitek<>0 then if mysvpoli(x,y,x+sirka,y+vyska) then begin{nastaveni pozice pomoci mysi}
                                                                stav:=_aktivni;
                                                                pozice:=(xmys-x-3) div __roztec;
                                                                if pozice<0 then pozice:=0
                                                                 else if pozice>byte(hodnota[0]) then pozice:=byte(hodnota[0]);
                                                                zobraz;
                                                                end
                                                           else begin{deaktivace kliknutim mimo pole}
                                                                stav:=_odkliknutim;
                                                                zobraz;
                                                                end;
  if keypressed then
    begin
    o:=readkey;
    case o of #0:begin         {kurzorove klavesy}
                 case byte(readkey) of lsipka:if pozice>0 then dec(pozice)
                                                          else pozice:=0;
                                       psipka:if pozice<byte(hodnota[0]) then inc(pozice);
                                       {alt+sipky - psani sipek (funguje jen s jednotkou Enhkbd.tpu):}
                                       altpsipka:tokeybuf(#26);
                                       althsipka:tokeybuf(#24);
                                       altdsipka:tokeybuf(#25);
                                       altlsipka:{tady to musim rozepsat, protoze znak sipky doleva ma stejny kod jeko Esc}
                                                 if pozice=-1 then begin
                                                                   hodnota:=#27;
                                                                   pozice:=1;
                                                                   stav:=_aktivni;
                                                                   end
                                                              else if byte(hodnota[0])<maxznaku then begin
                                                                                                  insert(#27,hodnota,pozice+1);
                                                                                                     inc(pozice);
                                                                                                     end;
                                       del,ctrldel:if pozice=-1 then begin
                                                                     hodnota:='';
                                                                     pozice:=0;
                                                                     end
                                                                else delete(hodnota,pozice+1,1);
                                       home:pozice:=0;
                                       endk:pozice:=byte(hodnota[0]);
                                       15:begin
                                          stav:=_odkliknutim;
                                          if predchozi<>nil then with predchozi^ do begin
                                                                                    pozice:=-1;
                                                                                    stav:=_aktivni;
                                                                                    zobraz;
                                                                                    end;
                                          end;
                                       end;
                 end;
              #8:begin{backspace}
                 if pozice>0 then begin {smazat znak vlevo od kurzoru a kurzor o znak vlevo}
                                  delete(hodnota,pozice,1);
                                  dec(pozice);
                                  end
                             else if pozice=-1 then begin {text je oznaceny - smaze se cely}
                                                    hodnota:='';
                                                    pozice:=0;
                                                    end;
                                              {else kurzor je pred textem, takze neni co mazat}
                 stav:=_aktivni;
                 end;
              #9:begin{tabulator}
                 stav:=_odkliknutim;
                 if dalsi<>nil then with dalsi^ do begin
                                                   pozice:=-1;
                                                   stav:=_aktivni;
                                                   zobraz;
                                                   end;
                 end;
              #13:stav:=_enterem;{enter}
              #27:begin{escape}
                  stav:=_escapem;
                  pozice:=-1;
                  hodnota:=default;
                  end;
              else if (not prosoubor)or(o in povoleneznaky) then
                begin {ostatni znaky - psani}
                if pozice=-1 then begin {text je oznaceny, znak ho nahradi}
                                  hodnota:=o;
                                  pozice:=1;
                                  stav:=_aktivni;
                                  end
                             else if byte(hodnota[0])<maxznaku then begin {pripsani na pozici kurzoru}
                                                                    pomstr:=o;
                                                                    insert(pomstr,hodnota,pozice+1);
                                                                    inc(pozice);
                                                                    end;
                end;
              end;{case o}
    zobraz;
    end;{if keypressed}
  end
 else {stav = _neaktivni, _enterem nebo _odkliknutim}
  if (stavtlacitek<>0)and mysvpoli(x,y,x+sirka,y+vyska) then
    begin {aktivace v pripade kliknuti mysi do pole}
    pozice:=-1;
    stav:=_aktivni;
    zobraz;
    end;
if hodnota<>puvodnihodnota then _zmeneno:=true;
End;{textovepole.kontrola}

procedure textovepole.nasmeruj(var shiftTab,Tab:textovepole);
Begin
predchozi:=@shifttab;
dalsi:=@tab;
End;{textovepole.nasmeruj}

procedure okno.init(ix,iy:integer;isirka,ivyska:word;nadpis:nadpstr);
var velikost:longint;
Begin
x:=ix; y:=iy;
sirka:=isirka; vyska:=ivyska;
bylkurzor:=kurzorb;
_zmertext(nadpis);
if (nadpis<>'')and(sirka<_sirkatextu+10) then sirka:=_sirkatextu+10;{rozsireni na sirku nadpisu}
velikost:=(sirka+1)*(vyska+1);
kvyp;
if (velikost<=65528)and(velikost<=maxavail) then begin
                                                 getmem(pozadiokna,velikost);
                                                 _getimage(x,y,x+sirka,y+vyska,pozadiokna);
                                                 end
                                            else pozadiokna:=nil;
_box(x,y,x+sirka,y+vyska,olhbarva,opdbarva,ovbarva);{okno}
y2:=y;
if nadpis<>'' then begin
                   _bar(x+2,y+2,x+sirka-3,y+_vyskatextu+4,oobdbarva);{prouzek pro nadpis}
                   _print(x+6-_deltax,y+4-_deltay,obarvanadpisu,nadpis);{nadpis}
                   inc(y2,_vyskatextu+4-_deltay);{souradnice spodku nadpisu}
                   end;
End;{okno.init}

procedure okno.zrus;
Begin
kvyp;
if pozadiokna=nil then begin
                       _bar(x,y,x+sirka,y+vyska,opozadinamazani);
                       _pushtext;
                       _nastavfont(wokfont,true);
                       _print(x,y,oembarva,wt.dej(wtomalopameti));
                       _print(x,y+9,oembarva,wt.dej(wtomalopameti+1));
                       _poptext;
                       end
                  else begin
                       inc(sirka); inc(vyska);
                       _putimage(x,y,sirka,vyska,pozadiokna);
                       freemem(pozadiokna,sirka*vyska);
                       end;
if bylkurzor then kzap;
End;{okno.zrus}

procedure menu.init;
Begin
prvniradek:=nil; prvnipolozka:=nil; {bude to potreba}
hodnota:=0;
End;{menu.init}

procedure menu.pridejtext(textik:string);
Begin
nactitexty(prvniradek,textik,sirkamenu);{viz jednotku Retezce}
End;{menu.pridejtext}

procedure menu.pridejpolozku(textik:string);
var nova,posledni:uknapolozkumenu;
Begin
if textik='' then exit;{prazdne polozky neberem}
new(nova);{vytvoreni nove polozky}
{pripojeni do seznamu:}
nova^.dalsi:=prvnipolozka;{nova bude na konci a seznam je kruhovy, takze z konce se bude ukazovat zase na zacatek}
if prvnipolozka=nil then begin{bude to uplne prvni polozka seznamu}
                         prvnipolozka:=nova;
                         nova^.predchozi:=nova; nova^.dalsi:=nova;{ukazuje sama na sebe}
                         end
                    else begin{bude se zarazovat na konec jiz nacateho seznamu}
                         posledni:=prvnipolozka^.predchozi;{je to kruh, takze ted jsme na posledni polozce}
                         posledni^.dalsi:=nova;{drive posledni, ted predposledni a bude ukazovat na novou}
                         prvnipolozka^.predchozi:=nova;{zacatek je propojeny s koncem}
                         nova^.predchozi:=posledni;{a je hotovo}
                         end;
nova^.texty:=nil;{tohle je nutne, aby fungovala procedura Nactitexty}
nactitexty(nova^.texty,textik,sirkamenu);
End;{menu.pridejpolozku}

procedure menu.zrus;
var p:uknapolozkumenu;
Begin
{zruseni uvodniho textu:}
zrustexty(prvniradek);
if prvnipolozka=nil then exit;{jestli tam nejsou zadne polozky, tak neni co rusit}
{rozpojeni kruhoveho seznamu polozek:}
prvnipolozka^.predchozi^.dalsi:=nil;
{Ted je seznam linearni (konci Nilem). Ruseni jednotlivych polozek:}
 repeat{jestli jsme jeste tady, tak v seznamu urcite neco je, takze muzeme pouzit repeat}
 p:=prvnipolozka^.dalsi;
 zrustexty(prvnipolozka^.texty);
 dispose(prvnipolozka);
 prvnipolozka:=p;
 until prvnipolozka=nil;
End;{menu.zrus}

procedure menu.kontrola;
var x,y,y2:integer;{levy horni roh a pomocna yova souradnice}
    sirka,vyska,sirka2:word;{rozmery okna a pomocny rozmer}
    unp,vybrana:uknapolozkumenu;{pomocny ukazatel a ukazatel na aktivni polozku}
    unr:uknaradek;{pomocny ukazatel pro zobrazovani textu}
    nejdelsi:byte;{pro uchovani delky nejdelsiho radku}
    hotovo,bylkurzor:boolean;{ukoncovaci podminka hlavniho cyklu a promenna pro ulozeni stavu kurzoru}
{}procedure KresliTlacitko(ktere:uknapolozkumenu;vybrane,stisknute:boolean);
{}{Vykresli tlacitko. Pozor, nestara se o kurzor.}
{}var lh,pd,v,t:byte;{pomocne barvy}
{}    y3:integer;{pomocna souradnice}
{}Begin
{}if stisknute then begin                 {prirazeni spravnych barev}
{}                  lh:=mislhbarva;
{}                  pd:=mispdbarva;
{}                  v:=misvbarva;
{}                  t:=mistxtbarva;
{}                  end
{}             else if vybrane then begin
{}                                  lh:=mialhbarva;
{}                                  pd:=miapdbarva;
{}                                  v:=miavbarva;
{}                                  t:=miatxtbarva;
{}                                  end
{}                             else begin
{}                                  lh:=minlhbarva;
{}                                  pd:=minpdbarva;
{}                                  v:=minvbarva;
{}                                  t:=mintxtbarva;
{}                                  end;
{}with ktere^ do begin    {vykresleni}
{}               __box(x+3,lhy,x+sirka-3,pdy,lh,pd,v);
{}               y3:=lhy+2;
{}               unr:=texty;
{}                repeat{prazdne polozky nejsou, tak muzeme repeatovat}
{}                _print(x+7+ord(stisknute),y3+1+ord(stisknute),t,unr^.obsah);
{}                unr:=unr^.dalsi;
{}                inc(y3,_vyskatextu2+2);
{}                until unr=nil;
{}               end;
{}End;{kreslitlacitko}
Begin{menu.kontrola}
bylkurzor:=kurzorb;
if prvnipolozka=nil then pridejpolozku(wt.dej(wtmenuOK));{at tam je aspon neco}
{predbezne hodnoty:}
sirka:=_sirkatextu2(nejdelsiradek(prvniradek))+8;
vyska:=4+pocetradkuvseznamu(prvniradek)*(_vyskatextu2+2);
{konecny vypocet sirky a vysky:}
unp:=prvnipolozka;
 repeat
 inc(vyska,pocetradkuvseznamu(unp^.texty)*(_vyskatextu2+2)+8);{pricteme vysku kazde polozky}
 sirka2:=_sirkatextu2(nejdelsiradek(unp^.texty))+14;{zjistime jeji sirku...}
 if sirka2>sirka then sirka:=sirka2;{...a jestli je vetsi nez prve namerena, aktualizujeme ji}
 unp:=unp^.dalsi;
 until unp=prvnipolozka;{dokud nejsme zpatky na prvni polozce kruhoveho seznamu}
{Vysku a sirku mame. Ted poloha okna:}
x:=(_maxx shr 1)-(sirka shr 1);      {nekam na prostredek obrazovky}
y:=(_maxy shr 1)-(vyska shr 1);
{vykresleni okna:}
kvyp;
__box(x,y,x+sirka,y+vyska,mlhbarva,mpdbarva,mvbarva);
{uvodni text:}
unr:=prvniradek;
y2:=y+4;
while unr<>nil do begin
                  _print(x+4,y2,mtxtbarva,unr^.obsah);
                  inc(y2,_vyskatextu2+2);
                  unr:=unr^.dalsi;
                  end;
{polozky menu:}
unp:=prvnipolozka; vybrana:=prvnipolozka;
 repeat
 with unp^ do begin{inicializace souradnic}
              lhy:=y2;
              pdy:=y2+pocetradkuvseznamu(unp^.texty)*(_vyskatextu2+2)+4;
              y2:=pdy+4;
              end;
 kreslitlacitko(unp,unp=vybrana,false);{vykresleni tlacitka}
 unp:=unp^.dalsi;
 until unp=prvnipolozka;
{hlavni cyklus:}
kzap;
hotovo:=false;
resetuj;
 repeat
 kurzor; {cekani na klik nestaci, protoze se tlacitka aktivuji pri pouhem najeti kurzoru bez klikani}
 {mys:}
 unp:=prvnipolozka;
  repeat {musime projit cely seznam a kazde tlacitko zkontrolovat}
  with unp^ do if mysvpoli(x+3,lhy,x+sirka-3,pdy)and((vybrana<>unp)or(stavtlacitek=1)) then
     begin{pokud je mys nad nejakym tlacitkem a klikneme nebo pokud tam je a to tlacitko jeste neni aktivni}
     kvyp;
     kreslitlacitko(vybrana,false,false);{deaktivace puvodne aktivniho tlacitka}
     vybrana:=unp;{vyber noveho tlacitka}
     kreslitlacitko(vybrana,true,stavtlacitek=1);{kresleni tlacitka pod kurzorem}
     kzap;
     if stavtlacitek=1 then hotovo:=true;{pokud jsme na nej klikli, koncime}
     end;
  unp:=unp^.dalsi;
  until unp=prvnipolozka;
 {klavesnice:}
 if keypressed then begin
                    sirka2:=xreadkey;{promenna sirka2 uz neni potreba, tak se zneuzije pro klavesnici}
                    kvyp;
                    case sirka2 of xlsipka,xhsipka:begin{nahoru}
                                                   kreslitlacitko(vybrana,false,false);
                                                   vybrana:=vybrana^.predchozi;
                                                   kreslitlacitko(vybrana,true,false);
                                                   end;
                                   xpsipka,xdsipka,9:begin{dolu (9 je tabulator)}
                                                     kreslitlacitko(vybrana,false,false);
                                                     vybrana:=vybrana^.dalsi;
                                                     kreslitlacitko(vybrana,true,false);
                                                     end;
                                   13,32:begin{potvrzeni (13 = enter, 32 = mezernik)}
                                         kreslitlacitko(vybrana,true,true);
                                         hotovo:=true;
                                         end;
                                   end;
                    kzap;
                    end;
 until hotovo;
{spocitani, ktera polozka byla vybrana:}
unp:=prvnipolozka;
y2:=1;{y2 se vyuzije jako pomocny citac}
while unp<>vybrana do begin
                      inc(y2);
                      unp:=unp^.dalsi;
                      end;
hodnota:=y2;{ulozeni vyberu}
if not bylkurzor then kvyp;{kurzor byl do tedka zapnuty, takze ho takhle vratime do puvodniho stavu}
End;{menu.kontrola}

procedure infokno(ix,iy:integer;hlaska:string;resb:boolean);
var seznam,vybrany:uknaradek;{pro manipulaci s textem}
    sirka,vyska:word;{sirka a vyska okna, slouzi k vypoctum}
    i:byte;{pomocna}
    bylkurzor:boolean;{jestli byl pred volanim infokna zapnut kurzor mysi}
    r:pointer;{pro ulozeni pozadi okenka}
    velikost:longint;{mnozstvi pameti, ktere bude potreba na ulozeni pozadi okna}
Begin
if hlaska='' then exit;
while hlaska[byte(hlaska[0])]=' 'do dec(byte(hlaska[0]));{odrezani zbytecnych mezer z konce}
seznam:=nil;{nutne}
nactitexty(seznam,hlaska,20);{rozlozeni zadaneho textu do radku (viz jednotku Retezce)}
sirka:=_sirkatextu2(nejdelsiradek(seznam))+11;{spocitani sirky...}
vyska:=pocetradkuvseznamu(seznam)*(_vyskatextu2+3)+3;{...a vysky okna}
dec(ix,(sirka shr 1));{protoze v parametrech ix a iy zadavame stred okna, musi se upravit na levy horni roh}
dec(iy,(vyska shr 1)); {(shr 1 je totez jako div 2, ale je to o trochu rychlejsi)}
if ix<0 then ix:=0; if (ix+sirka)>_maxx then ix:=_maxx-sirka;{tohle zajisti, ze je okno cele na obrazovce}
if iy<0 then iy:=0; if (iy+vyska)>_maxy then iy:=_maxy-vyska;
velikost:=(sirka+1)*(vyska+1);{velikost potrebne pameti pro pozadi okna}
bylkurzor:=kurzorb;{jestli se ma kurzor po skonceni procedury zapnout}
kvyp;
if velikost>=maxavail then r:=nil {pamet nestaci}
                      else begin  {pamet staci}
                           getmem(r,velikost); {alokace}
                           _getimage(ix,iy,ix+sirka,iy+vyska,r); {nacteni pozadi}
                           end;
_box(ix,iy,ix+sirka,iy+vyska,iolhbarva,iopdbarva,iovbarva);{vykresleni ramecku}
vybrany:=seznam;{skocime na prvni radek}
i:=0;{podle ni se bude pocitat y-ova souradnice radku}
 repeat
 _print(ix+6,iy+3+i*(_vyskatextu2+3),iobarvatextu,vybrany^.obsah);{vypsani radku}
 vybrany:=vybrany^.dalsi;{o radek v seznamu dal}
 inc(i);
 until vybrany=nil;{byl napsan posledni radek}
if bylkurzor then kzap;
resetuj; cekej;{ceka se na odkliknuti}
kvyp;
if r=nil then _bar(ix,iy,ix+sirka,iy+vyska,opozadinamazani)
         else begin                                          {smazani okna}
              _putimage(ix,iy,sirka+1,vyska+1,r);
              freemem(r,velikost);
              end;
if bylkurzor then kzap;
zrustexty(seznam);{smazani seznamu radku z pameti}
if resb then resetuj;{kdyz resb=false, zustane po odkliknuti okna stale
kliknuto=true. Infokno tak jde pouzit treba na zadani souradnic kliknutim}
End;{infokno}

function anoneokno(texty:string):boolean;
var prvni,vybrany:uknaradek;
    i:word;{pomocna}
    anoT,neT:tlacitko;
    ok:okno;
    ch:char;{pro cteni klaves}
    stav:(sNeurcen,sAno,sNe);{pomocna, pro zaverecny cyklus}
    maxdelka:word; pocetradku:byte;
    nadpis:string[50];
Begin
if texty='' then exit;
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
{osetreni nadpisu:}
if copy(texty,1,3)='%%n' then begin
                              i:=4;
                              while (i+2<byte(texty[0]))and(copy(texty,i,3)<>'%%N')do inc(i);
                              if i+2<byte(texty[0]) then begin
                                                         nadpis:=copy(texty,4,i-4);
                                                         delete(texty,1,i+2);
                                                         end;
                              end
                         else nadpis:=wt.dej(wtdotaz);
prvni:=nil;
nactitexty(prvni,texty,50);{rozdeleni zadaneho textu do radku}
 repeat
 _zmertext(nadpis);
 if _sirkatextu<144 then nadpis:=nadpis+' ';{co kdyby byly vsechny texty tak uzke, ze by nebylo misto na tlacitka?}
 until _sirkatextu>=144;
maxdelka:=_sirkatextu2(nejdelsiradek(prvni));{nalezeni nejdelsiho, podle ktereho se urci sirka okna...}
if _sirkatextu>maxdelka then maxdelka:=_sirkatextu;{...a nebo podle nadpisu, pokud je jeste delsi}
pocetradku:=pocetradkuvseznamu(prvni);
ok.init((_maxx shr 1)-maxdelka shr 1,{x}
        (_maxy shr 1)-((_vyskatextu2+2)*pocetradku)shr 1-_vyskatextu-26,{y}
        maxdelka+10,{sirka}
        _vyskatextu+40+(_vyskatextu2+2)*pocetradku,{vyska}
        nadpis);
with ok do begin
           {zobrazeni textu:}
           vybrany:=prvni;
           i:=y2+4;
            repeat
            _print(x+5,i+1,txtbarva,vybrany^.obsah);
            vybrany:=vybrany^.dalsi;
            inc(i,_vyskatextu2+2);
            until vybrany=nil;
           {tlacitka:}
           _pushtext; _nastavfont(wokfont,true);{kvuli tlacitkum}
           anot.init(x+sirka div 5,y+vyska-23,wt.dej(wtano),wt.dej(wtanohelp),2);
           net.init(x+sirka-sirka div 5-45,y+vyska-23,wt.dej(wtne),wt.dej(wtnehelp),2);
           end;
kzap;
ch:=' '; stav:=sneurcen;
 repeat{cekani na reakci uzivatele}
 resetuj;cekej;
 anot.kontrola; net.kontrola;
 if anot.hodnota=1 then stav:=sano;
 if net.hodnota=1 then stav:=sne;
 if keypressed then begin
                    ch:=readkey;
                    if (ch=#13)or(upcase(ch)=wt.dej(wtanoznak)) then stav:=sano
                     else if (ch=#27)or(upcase(ch)=wt.dej(wtneznak)) then stav:=sne;
                    end;
 until stav<>sneurcen;
_poptext;
anoneokno:=stav=sAno;
ok.zrus;
zrustexty(prvni);
if dialogystandardnimfontem then _poptext;
resetuj;
End;{anoneokno}

procedure okokno(texty:string);
var prvni,vybrany:uknaradek;
    i:word; ch:char;
    maxdelka:word; pocetradku:byte;
    Oukej:tlacitko;
    ok:okno;
    nadpis:string[50];
Begin
if texty='' then exit;
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
if copy(texty,1,3)='%%n' then begin
                              i:=4;
                              while (i+2<byte(texty[0]))and(copy(texty,i,3)<>'%%N')do inc(i);
                              if i+2<byte(texty[0]) then begin
                                                         nadpis:=copy(texty,4,i-4);
                                                         delete(texty,1,i+2);
                                                         end;
                              end
                         else nadpis:=wt.dej(wtzprava);
prvni:=nil;
nactitexty(prvni,texty,50);
 repeat
 _zmertext(nadpis);
 if _sirkatextu<50 then nadpis:=nadpis+' ';{co kdyby byly vsechny texty tak uzke, ze by nebylo misto na tlacitka?}
 until _sirkatextu>=50;
maxdelka:=_sirkatextu2(nejdelsiradek(prvni));
if _sirkatextu>maxdelka then maxdelka:=_sirkatextu;
pocetradku:=pocetradkuvseznamu(prvni);
ok.init((_maxx shr 1)-(maxdelka shr 1),{x}
        (_maxy shr 1)-((_vyskatextu2+2)*pocetradku)shr 1-_vyskatextu-26,{y}
        maxdelka+10,{sirka}
        _vyskatextu+40+(_vyskatextu2+2)*pocetradku,{vyska}
        nadpis);
with ok do begin
           vybrany:=prvni;
           i:=y2+4;
            repeat
            _print(x+5,i+1,txtbarva,vybrany^.obsah);
            vybrany:=vybrany^.dalsi;
            inc(i,_vyskatextu2+2);
            until vybrany=nil;
           _pushtext; _nastavfont(wokfont,true);{kvuli tlacitku}
           oukej.init(x+(sirka shr 1)-27,y+vyska-23,wt.dej(wtok6),wt.dej(wtokhelp),2);
           end;
kzap;
 repeat
 resetuj; cekej;
 oukej.kontrola;
 if keypressed then ch:=readkey else ch:=#0;
 until (oukej.hodnota=1)or(ch=' ')or(ch=#13)or(upcase(ch)=wt.dej(wtokznak));
_poptext;
ok.zrus;
zrustexty(prvni);
if dialogystandardnimfontem then _poptext;
resetuj;
End;{okokno}

function vloztextokno(nadpis:nadpstr;maxzn:byte;defaulttext:string;prosoubor:boolean):string;
var v:okno;
    tp:textovepole;
    okt,cancelt:tlacitko;
    okb,cancelb:boolean;
    delka:byte;
    o:char;
    puvrk:byte;
Begin
wokzruseno:=false;
puvrk:=_ridicikody;
_ridicikody:=_vypisovat;
if nadpis='' then nadpis:=wt.dej(wtzadejtext);
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
while _sirkatextu2(byte(nadpis[0]))<77 do nadpis:=nadpis+' ';
delka:=byte(nadpis[0]); if maxzn>delka then delka:=maxzn;
v.init(_maxx shr 1-(_sirkatextu2(delka) shr 1+5),_maxy shr 1-30,delka*__roztec+10,_vyskatextu2 shl 1+35,nadpis);
with v do begin
          tp.init(x+2,y2+3,maxzn,defaulttext,true,prosoubor);
          _pushtext;
          _nastavfont(wokfont,true);
          okt.init(x+sirka-88,y+vyska-20,wt.dej(wtok4),wt.dej(wtvltoknookhelp),1);
          cancelt.init(x+sirka-42,y+vyska-20,wt.dej(wtzrus),wt.dej(wtvltoknozrushelp),1);
          _lodtext;
          end;
okb:=false; cancelb:=false; resetuj;
 repeat
 cekej;
 tp.kontrola;
 _nastavfont(wokfont,true);{pro spravne zobrazeni diakritiky v popisech tlacitek}
 okt.kontrola; cancelt.kontrola;
 _lodtext;
 if keypressed then o:=readkey else o:=#0;
 if (okt.hodnota=1)or(tp.stav=_enterem)or(o=#13) then okb:=true;
 if (cancelt.hodnota=1)or(tp.stav=_escapem)or(o=#27) then cancelb:=true;
 resetuj;
 until okb or cancelb;
_poptext;
if okb then vloztextokno:=tp.hodnota
       else begin
            vloztextokno:=defaulttext;
            wokzruseno:=true;
            end;
v.zrus;
if dialogystandardnimfontem then _poptext;
_ridicikody:=puvrk;
End;{vloztextokno}

function vlozcislookno(nadpis:nadpstr;_od,_do,default:longint):longint;
var ok:okno;
    okt,cancelt,pridejt,ubert:tlacitko;
    tp:textovepole;
    okb,cancelb:boolean;
    kod:integer;
    vysledek:longint;
    pomstr:string[11];
    o:char;
    puvrk:byte;
Begin
wokzruseno:=false;
puvrk:=_ridicikody;
_ridicikody:=_vypisovat;
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
if default<_od then default:=_od;
if default>_do then default:=_do;
resetuj;
okb:=false; cancelb:=false;
if nadpis='' then nadpis:=wt.dej(wtzadejcislo);
while _sirkatextu2(byte(nadpis[0]))<77 do nadpis:=nadpis+' ';
{vypocet delky cisla:}
str(_od,pomstr);
kod:=byte(pomstr[0]);
str(_do,pomstr);
if byte(pomstr[0])>kod then kod:=byte(pomstr[0]);{bereme nejdelsi moznost}
{okno a spol.:}
_zmertext(nadpis);
ok.init(_maxx shr 1-(_sirkatextu shr 1+5),_maxy shr 1-30,kod*__roztec+10,_vyskatextu2+_vyskatextu+35,nadpis);
str(default,pomstr);
with ok do begin
           tp.init(x+2,y2+3,kod,pomstr,true,true);
           _pushtext;
           _nastavfont(wokfont,true);
           okt.init(x+sirka-88,y+vyska-20,wt.dej(wtok4),wt.dej(wtvlcoknookhelp),1);
           cancelt.init(x+sirka-42,y+vyska-20,wt.dej(wtzrus),wt.dej(wtvlcoknozrushelp),1);
           _lodtext;
           end;
 repeat
 cekej;
 tp.kontrola;
 _nastavfont(wokfont,true);
 okt.kontrola; cancelt.kontrola;
 _lodtext;
 if keypressed then o:=readkey else o:=#0;
 if (okt.hodnota=1)or(tp.stav=_enterem)or(o=#13) then okb:=true;
 if (cancelt.hodnota=1)or(tp.stav=_escapem)or(o=#27) then cancelb:=true;
 if okb then with tp do
              repeat
              val(hodnota,vysledek,kod);
              if (kod<>0)and(byte(hodnota[0])<>0) then
                begin
                delete(hodnota,kod,1);{hezky natvrdo}
                okb:=false;
                stav:=_aktivni;
                pozice:=byte(hodnota[0]);
                zobraz;
                end;
              until (kod=0)or(byte(hodnota[0])=0);
 resetuj;
 until okb or cancelb;
_poptext;
if okb then begin
            if vysledek<_od then vysledek:=_od;
            if vysledek>_do then vysledek:=_do;
            vlozcislookno:=vysledek;
            end
       else begin
            vlozcislookno:=default;
            wokzruseno:=true;
            end;
ok.zrus;
if dialogystandardnimfontem then _poptext;
_ridicikody:=puvrk;
End;{vlozcislookno}

function nabidka(ix,iy:integer;texty:string):byte;
type uknaradek=^radek;
     radek=record
           textik:string[30];
           a,b,c,d:integer;{jako u Hotspotu}
           dalsi,predchozi:uknaradek;
           end;
var prvni,vybrany:uknaradek;
    ok:okno;
    max,{delka nejdelsiho radku}
    pocetradku,pocetcar:byte;
    z:char;
    s,v,{pro vypocet rozmeru ramecku}
    i:word;{pomocna}
    koncit,bylkurzor:boolean;
{}procedure ctiradek;{cte radky z retezce Texty a zapisuje je do linearniho seznamu}
{}var ret:string[30];
{}Begin
{}ret:='';
{} repeat
{} inc(i);{pozice v retezci texty}
{} ret:=ret+texty[i];{precteni jednoho znaku}
{} until (texty[i]='|')or(i>=byte(texty[0]));{konci se pri precteni | nebo dojeti na konec textu}
{}while ret[byte(ret[0])] in ['|',' '] do dec(byte(ret[0]));{odriznuti prebytecnych znaku}
{}while ret[1]=' 'do delete(ret,1,1);
{}vybrany^.textik:=ret;
{}if i<byte(texty[0]) then begin{vytvoreni noveho radku v pameti}
{}                         new(vybrany^.dalsi);
{}                         vybrany^.dalsi^.predchozi:=vybrany;
{}                         vybrany:=vybrany^.dalsi;
{}                         vybrany^.dalsi:=nil;
{}                         end;             {nedostatek pameti nepredpokladam}
{}End;{ctiradek}
{}procedure kresliradek(oznacit:boolean);{kresli radek, na ktery ukazuje Vybrany}
{}Begin
{}kvyp;
{}with vybrany^ do begin
{}                 if textik='-' then begin{cara}
{}                                    _hline(a,c,b+2,nhbarva);
{}                                    _hline(a,c,b+3,ndbarva);
{}                                    end
{}                               else if oznacit then begin     {text}
{}                                                    _bar(a,b,c,d,nozntxtpodbarva);
{}                                                    _print(a+3,b+3,nozntxtbarva,textik);
{}                                                    end
{}                                               else begin
{}                                                    _bar(a,b,c,d,ovbarva);
{}                                                    _print(a+3,b+3,ntxtbarva,textik);
{}                                                    end;
{}                 end;
{}kzap;
{}End;{kresliradek}
{}function spocitej:byte;{spocita, kolikata polozka je oznacena}
{}var pom:uknaradek;
{}    cislo:byte;
{}Begin
{}pom:=prvni;
{}cislo:=1;
{}while pom<>vybrany do begin
{}                      if pom^.textik<>'-'then inc(cislo);
{}                      pom:=pom^.dalsi;
{}                      end;
{}spocitej:=cislo;
{}End;{spocitej}
Begin{nabidka}
{orezani prebytecnych znaku ze zacatku a konce vstupniho retezce:}
while texty[byte(texty[0])] in [' ','|','-'] do dec(byte(texty[0]));
while texty[1] in [' ','-','|'] do delete(texty,1,1);
if texty='' then exit;{nema cenu zobrazovat prazdne okno}
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
resetuj;
{nacteni radku ze vstupniho retezce:}
new(prvni); prvni^.predchozi:=nil; prvni^.dalsi:=nil; prvni^.textik:='';
vybrany:=prvni; i:=0;
 repeat ctiradek until i>=byte(texty[0]);
vybrany:=prvni; i:=1; pocetradku:=0; pocetcar:=0; max:=0;
 repeat{spocitani radku a nalezeni nejdelsiho}
 with vybrany^ do begin
                  if textik='-' then begin{je to rozdelovaci cara}
                                     inc(pocetcar);
                                     inc(i,6);
                                     end
                                else begin{je to text}
                                     inc(pocetradku);
                                     inc(i,14);
                                     end;
                  if max<byte(textik[0])then max:=byte(textik[0]);
                  end;
 vybrany:=vybrany^.dalsi;
 until vybrany=nil;
s:=_sirkatextu2(max)+6;{sirka ramecku}
v:=(_vyskatextu2+6)*pocetradku+6*pocetcar+4;{vyska ramecku}
if ix<0 then ix:=0 else if ix>_maxx-s then ix:=_maxx-s;{pojistka proti vycuhovani z obrazovky}
if iy<0 then iy:=0 else if iy>_maxy-v then iy:=_maxy-v;
vybrany:=prvni; i:=1;
 repeat{inicializace klikacich oblasti}
 with vybrany^ do begin
                  if textik='-' then begin{je to rozdelovaci cara}
                                     d:=iy+i+5;
                                     b:=iy+i;
                                     inc(i,6);
                                     end
                                else begin{je to text}
                                     d:=iy+i+_vyskatextu2+5;
                                     b:=iy+i;
                                     inc(i,_vyskatextu2+6);
                                     end;
                  a:=ix+1; c:=ix+s-1;
                  end;
 vybrany:=vybrany^.dalsi;
 until vybrany=nil;
bylkurzor:=kurzorb;
ok.init(ix,iy,s,v,'');{zobrazeni okenka}
{vypsani radku:}
vybrany:=prvni; i:=1; {na zacatku je oznacena prvni polozka}
 repeat
 kresliradek(i=1);
 vybrany:=vybrany^.dalsi; inc(i);
 until vybrany=nil;
koncit:=false; vybrany:=prvni;
 repeat{hlavni cyklus}
 cekejnacokoli;
 if keypressed then
   begin
   case readkey of #0:case readkey of #72:if vybrany<>prvni then begin{nahoru}
                                                                 kresliradek(false);
                                                                  repeat
                                                                  vybrany:=vybrany^.predchozi;
                                                                  until vybrany^.textik<>'-';
                                                                 kresliradek(true);
                                                                 end;
                                      #80:if vybrany^.dalsi<>nil then begin{dolu}
                                                                      kresliradek(false);
                                                                       repeat
                                                                       vybrany:=vybrany^.dalsi;
                                                                       until vybrany^.textik<>'-';
                                                                      kresliradek(true);
                                                                      end;
                                      end;
                   #13:begin{Enter}
                       nabidka:=spocitej;
                       koncit:=true;
                       end;
                   #27:begin{Escape}
                       nabidka:=0;
                       koncit:=true;
                       end;
                   end;
   end{if keypressed}
               else begin
                    if lmys then begin{bylo kliknuto, pocita se jen leve mysitko}
                                 if mysvpoli(ix,iy,ix+s,iy+v)then begin{mys je nad oknem}
                                                                  if vybrany^.textik<>'-'then
                                                                    begin
                                                                    nabidka:=spocitej;
                                                                    koncit:=true;
                                                                    end;
                                                                  end
                                                             else begin{kliklo se vedle}
                                                                  nabidka:=0;
                                                                  koncit:=true;
                                                                  end;
                                 end
                            else begin{jen se pohnula mys}
                                 with vybrany^ do if not mysvpoli(a,b,c,d) then kresliradek(false);
                                 vybrany:=prvni;
                                 while (vybrany<>nil)and not mysvpoli(vybrany^.a,vybrany^.b,vybrany^.c,vybrany^.d)do
                                  vybrany:=vybrany^.dalsi;
                                 if vybrany=nil then vybrany:=prvni{neukazujeme na nic}
                                                else kresliradek(true);{ukazujeme na nejaky radek}
                                 end;
                    end;
 until koncit;{konec hlavniho cyklu}
ok.zrus;{zruseni ramecku}
while prvni<>nil do begin{vycisteni pameti}
                    vybrany:=prvni;
                    prvni:=prvni^.dalsi;
                    dispose(vybrany);
                    end;
if dialogystandardnimfontem then _poptext;
if bylkurzor then kzap;
End;{nabidka}

function vyberbarvuokno(nadpis:nadpstr;default:byte):byte;
var ok:okno;
    vybrana,vybrana2:integer;
    okt,cancelt:tlacitko;
    hotovo:boolean;
    o:word;{na cteni klaves}
    i,j:word;
    lhx,lhy:word;{souradnice}
Begin
wokzruseno:=false;
lhx:=(_maxx shr 1)-80; lhy:=(_maxy shr 1)-45;
resetuj;
if nadpis='' then nadpis:=wt.dej(wtvyberbarvu);
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
vybrana:=default; vybrana2:=vybrana; hotovo:=false;
_zmertext(nadpis);
ok.init(lhx-3,lhy-_vyskatextu-7,165,188+_vyskatextu,nadpis);{okno}
_pushtext; _nastavfont(wokfont,true);
with ok do begin
           for i:=0 to 15 do for j:=0 to 15 do {barvy}
             _bar(lhx+10*j,lhy+10*i,lhx+10*j+9,lhy+10*i+9,i shl 4+j);
           okt.init(lhx,lhy+162,wt.dej(wtok10),wt.dej(wtokbarvyhelp),1);  {tlacitka}
           cancelt.init(lhx+121,lhy+162,wt.dej(wtzrus),wt.dej(wtzrusbarvuhelp),1);
           end;
 repeat
 {znacka:}
 _rectangle((vybrana and 15)*10+lhx+1,(vybrana shr 4)*10+lhy+1,(vybrana and 15)*10+lhx+9,(vybrana shr 4)*10+lhy+9,0);
 _rectangle((vybrana and 15)*10+lhx,(vybrana shr 4)*10+lhy,(vybrana and 15)*10+lhx+8,(vybrana shr 4)*10+lhy+8,14);
 napis(lhx+93,lhy+166,txtbarva,ovbarva,nastr2(vybrana,3));{vypis cisla barvy}
 kzap;
 resetuj; cekej;
 if stavtlacitek<>0 then begin
                         if mysvpoli(lhx,lhy,lhx+159,lhy+159)
                          then vybrana2:=(xmys-lhx) div 10+((ymys-lhy) div 10)shl 4;{vyber barvy mysi}
                         okt.kontrola; cancelt.kontrola;
                         if okt.hodnota=1 then hotovo:=true;
                         if cancelt.hodnota=1 then begin
                                                   vybrana2:=default;
                                                   hotovo:=true;
                                                   end;
                         end
                    else begin {vyber klavesnici}
                         i:=xreadkey;
                         if (upcase(chr(i))=wt.dej(wtokznak))or(i=13) then hotovo:=true{Enter}
                          else if (upcase(chr(i))=wt.dej(wtzrusznak))or(i=27) then begin{Escape}
                                                                                   vybrana2:=default;
                                                                                   wokzruseno:=true;
                                                                                   hotovo:=true;
                                                                                   end
                           else case i of
                                 xlsipka:if vybrana2>0 then dec(vybrana2) else vybrana2:=255;
                                 xpsipka:if vybrana2<255 then inc(vybrana2) else vybrana2:=0;
                                 xhsipka:begin
                                         dec(vybrana2,16);
                                         if vybrana2<0 then inc(vybrana2,256);
                                         end;
                                 xdsipka:begin
                                         inc(vybrana2,16);
                                         if vybrana2>255 then dec(vybrana2,256);
                                         end;
                                 xhome:dec(vybrana2,vybrana2 and 15);{konec radku}
                                 xendk:inc(vybrana2,integer(15)-vybrana2 and 15);{zacatek radku}
                                 end;
                         end;
 kvyp;
 {zruseni znacky:}
 _bar((vybrana and 15)*10+lhx,(vybrana shr 4)*10+lhy,(vybrana and 15)*10+lhx+9,(vybrana shr 4)*10+lhy+9,vybrana);
 vybrana:=vybrana2;
 until hotovo;
vyberbarvuokno:=vybrana;
_poptext;
ok.zrus;
if dialogystandardnimfontem then _poptext;
End;{vyberbarvuokno}

procedure zjistujsouradnice;
const vlevo=10;
var bylkurzor:boolean;
    ok:okno;
    px:integer;{pomocna x-ova souradnice}
    mx,my:integer;
    vpravo:word;{pomocna souradnice}
{}procedure vypis;{vypise do okenka souradnice kurzoru}
{}Begin
{}with ok do _box(x,y,x+sirka,y+vyska,iolhbarva,iopdbarva,iovbarva);{smazani starych cisel}
{}_print(ok.x+3,ok.y+2,iobarvatextu,nastr2(xmys,3)+':'+nastr2(ymys,3));{vypsani novych cisel do okenka}
{}End;{vypis}
Begin{zjistujsouradnice}
_pushtext; _nastavfont(wokfont,true);
vpravo:=_maxx-89;
bylkurzor:=kurzorb;{ulozeni kurzoru}
kvyp;
resetuj;
if xmys>320 then px:=vlevo else px:=vpravo;{urceni polohy okenka}
ok.init(px,_maxy-29,60,12,'');
mx:=xmys; my:=ymys;
vypis;
 repeat
 {zamerovaci kriz:}
 _xorhline(0,_maxx,my,15);
 _xorvline(mx,0,_maxy,15);
  repeat
  kurzor;
  until (xmys<>mx)or(ymys<>my)or(stavtlacitek<>0)or keypressed;
 _xorhline(0,_maxx,my,15);
 _xorvline(mx,0,_maxy,15);
 mx:=xmys; my:=ymys;
 if (abs(xmys-(px+30))<40)and(ymys>_maxy-59) then begin{pokud se kurzorem blizime k okenku...}
                                                  ok.zrus;{...presune se na druhy konec obrazovky, aby neprekazelo:}
                                                  if px=vlevo then px:=vpravo
                                                              else px:=vlevo;
                                                  ok.init(px,_maxy-29,60,12,'');
                                                  end;
 vypis;
 until (stavtlacitek<>0)or keypressed;
ok.zrus;
if bylkurzor then kzap;
_poptext;
End;{zjistujsouradnice}

function VyberSouborOkno(nadpis:nadpstr;koncovky:string3;adresare:dirstr):string;
const vecinaokno=9; vyskaveci=11;
var ok:okno;
    okt,cancelt,novyt,infot,diskt:tlacitko;
    apos,spos:posuvnik;
    prvnisoubor,{zacatek seznamu}
    su,{pri pocitani souboru pomocny, pak ukazuje na prvni soubor v okne}
    su2,su2t:uknatsoubor;{pomocne}
    prvniadresar,au,au2:uknatadresar;
    pocetsouboru,pocetadresaru,{pocty}
    sp,ap,{pozice v seznamu (cislo bloku posuvniku)}
    spb,apb,i:word;{pomocne}
    puvodnicesta,cesta:dirstr;
    koncit,koncit2:boolean;{podminky pro ukonceni vnitrniho a vnejsiho cyklu}
    ret:string[8];{pomocny}
    o:char;{pro klavesnici}
    lhxa,lhxs,lhy:integer;{souradnice okna a jeho casti}
    adresareb:boolean;
{}procedure avypis;{vypise adresare}
{}var i:word;
{}    j:byte;
{}Begin
{}ap:=apos.hodnota;{od jake pozice se ma vypisovat}
{}au:=prvniadresar;       {nalezeni prvni polozky, ktera uz se vejde do okna:}
{}if ap>0 then for i:=1 to ap do for j:=1 to vecinaokno do au:=au^.dalsi;
{}kvyp;
{}_box(lhxa,lhy,lhxa+70,lhy+100,tplhbarva,tppdbarva,tpvbarva);{nakresleni ramecku / vymazani predchoziho obsahu}
{}j:=0; au2:=au;{aby v au zustal ukazatel na prvni zobrazenou polozku}
{}while (au2<>nil)and(j<vecinaokno) do {dokud je co psat a vejde se to do okna}
{}  begin
{}  _print(lhxa+3,lhy+3+vyskaveci*j,tptxtbarva,au2^.jmeno);{vypsani jmena}
{}  inc(j); au2:=au2^.dalsi;{o polozku dal}
{}  end;
{}kzap;
{}au2:=nil;{s timhle pozdeji pocitam - dulezite}
{}End;{avypis}
{}procedure svypis;{vypise soubory}
{}var i:word;
{}    j:byte;
{}Begin
{}sp:=spos.hodnota;
{}su:=prvnisoubor;
{}if sp>0 then for i:=1 to sp do for j:=1 to vecinaokno do su:=su^.dalsi;
{}kvyp;
{}_box(lhxs,lhy,lhxs+100,lhy+100,tplhbarva,tppdbarva,tpvbarva);
{}j:=0; su2:=su;
{}while (su2<>nil)and(j<vecinaokno) do
{}  begin
{}  _print(lhxs+3,lhy+3+vyskaveci*j,tptxtbarva,su2^.jmeno+'.'+su2^.koncovka);
{}  inc(j); su2:=su2^.dalsi;
{}  end;
{}kzap;
{}su2:=nil;
{}End;{svypis}
{}procedure cvypis;{vypise aktualni cestu}
{}Begin
{}getdir(0,cesta);
{}kvyp;
{}_box(lhxa,lhy+103,lhxa+248,lhy+113,opdbarva,olhbarva,ovbarva);
{}_print(lhxa+2,lhy+105,txtbarva,zkratcestu(cesta,30));
{}kzap;
{}End;{cvypis}
{}procedure info;{zobrazi okno s informacemi o oznacenem souboru}
{}var dt:datetime;
{}    atr:string[45];
{}Begin
{}with su2^ do
{}  begin
{}  atr:='';            {slozeni popisu atributu:}
{}  if atributy and ReadOnly<>0 then atr:=wt.dej(wtjenprocteni);
{}  if atributy and Hidden<>0 then atr:=atr+wt.dej(wtskryty);
{}  if atributy and SysFile<>0 then atr:=atr+wt.dej(wtsystemovy);
{}  if atributy and Archive<>0 then atr:=atr+wt.dej(wtarchivovat);
{}  if atr='' then atr:=wt.dej(wtzadne)
{}            else begin
{}                 dec(byte(atr[0]));{odmazani mezery na konci}
{}                 atr[byte(atr[0])]:='.';{misto carky na konec tecku}
{}                 end;
{}  unpacktime(su2^.zmeneno,dt);
{}  okokno('%%n'+wt.dej(wtvlastnostisouboru)+'%%N'+wt.dej(wtsoubor)+jmeno+'.'+koncovka+'|'+
{}         wt.dej(wtvelikost)+nastr(velikost)+' B ('+nastr(velikost shr 10)+' KB)|'+
{}         wt.dej(wtposlednizmena)+nastr(dt.day)+'.'+nastr(dt.month)+'.'+nastr(dt.year)
{}            +', '+nastr(dt.hour)+':'+nastr3(dt.min,2)+':'+nastr3(dt.sec,2)+'|'+
{}         wt.dej(wtatributy)+atr);
{}  end;
{}End;{info}
{}procedure oukej;{potvrdi vybrany soubor}
{}Begin
{}if su2=nil then infokno(xmys,ymys,wt.dej(wtmusitevybratsoubor),true)
{}           else begin    {vraceni jmena vybraneho souboru...}
{}                if adresareb then begin
{}                                  if cesta[byte(cesta[0])]<>'\' then cesta:=cesta+'\';
{}                                   {korenovy adresar konci znakem '\', ale ostatni ne, proto je to potreba otestovat}
{}                                  vybersouborokno:=cesta+su2^.jmeno+'.'+su2^.koncovka;
{}                                  end
{}                             else vybersouborokno:=adresare+su2^.jmeno+'.'+su2^.koncovka;
{}                koncit:=true; koncit2:=true; {...a ukonceni obou cyklu}
{}                end;
{}End;{oukej}
{}procedure kancl;{zrusi vyber}
{}Begin
{}vybersouborokno:='';
{}koncit:=true; koncit2:=true;
{}wokzruseno:=true;
{}End;{kancl}
Begin{vybersouborokno}
wokzruseno:=false;
su2t:=nil;
lhxs:=_maxx shr 1-50;
lhxa:=lhxs-90;
lhy:=_maxy shr 1-50;
getdir(0,puvodnicesta);{aby se pak bylo kam vratit}
if adresare='' then adresareb:=true
               else begin
                    adresareb:=false;
                    if adresare[byte(adresare[0])]<>'\' then adresare:=adresare+'\';{pokud na konci neni '\', pripiseme ho}
                    if adresare='.\' then adresare:=''{aktualni adresar}
                                     else chdir(adresare);{skok do urceneho adresare}
                    end;
if nadpis='' then nadpis:=wt.dej(wtvybersoubor);
_pushtext;
_zmertext(nadpis);
if adresareb then begin
                  ok.init(lhxa-2,lhy-(_vyskatextu+7),252,122+_vyskatextu,nadpis);
                  _nastavfont(wokfont,true);
                  diskt.init(lhxs+119,lhy+40,wt.dej(wtdisk),wt.dej(wtdiskhelp),1);
                  end
             else begin
                  ok.init(lhxs-2,lhy-(_vyskatextu+7),162,110+_vyskatextu,nadpis);
                  _nastavfont(wokfont,true);
                  end;
novyt.init(lhxs+119,lhy,wt.dej(wtnovy),wt.dej(wtnovyhelp),1);
infot.init(lhxs+119,lhy+20,wt.dej(wtinfo),wt.dej(wtinfohelp),1);
okt.init(lhxs+119,lhy+64,wt.dej(wtok4),wt.dej(wtvsoknookhelp),1);
cancelt.init(lhxs+119,lhy+84,wt.dej(wtzrus),wt.dej(wtvsoknozrushelp),1);
koncit2:=false;
resetuj;
 Repeat{vnejsi cyklus}
 sp:=0; ap:=0;{aby se po prepnuti adresare vzdy zobrazily soubory od zacatku}
 koncit:=false;
 {vytvareni seznamu + inicializace posuvniku:}
 seznamsouboru(prvnisoubor,koncovky);  {seznam}
 su:=prvnisoubor; pocetsouboru:=0;
 while su<>nil do begin                {pocet}
                  inc(pocetsouboru);
                  su:=su^.dalsi;
                  end;
 i:=pocetsouboru div vecinaokno;     {pocet pozic (pro posuvnik)}
 if pocetsouboru mod vecinaokno>0 then inc(i);
 spos.init(lhxs+102,lhy,70,i,0);     {inicializace posuvniku}
 if adresareb then {to same pro adresare}
   begin
   seznamadresaru(prvniadresar);
   au:=prvniadresar; pocetadresaru:=0;
   while au<>nil do begin
                    inc(pocetadresaru);
                    au:=au^.dalsi;
                    end;
   i:=pocetadresaru div vecinaokno;
   if pocetadresaru mod vecinaokno>0 then inc(i);
   apos.init(lhxa+72,lhy,70,i,0);
   end;
 svypis;  {vypis+nastaveni su na prvnisoubor a su2 na nil}
 if adresareb then begin avypis; cvypis; end;
  repeat{vnitrni cyklus}
  cekej;
  spos.kontrola;
  if spos.hodnota<>sp then svypis; {pohyb v seznamu souboru posuvnikem}
  if lmys and mysvpoli(lhxs+1,lhy+1,lhxs+99,lhy+99) then
    begin              {kliknuto do seznamu souboru}
    kvyp;
    if su2<>nil then begin{zruseni oznaceni puvodniho souboru (jestli byl nejaky oznacen)}
                     su2t:=su2;{ulozeni toho, co bylo oznaceno predtim}
                     _bar(lhxs+1,lhy+1+vyskaveci*spb,lhxs+99,lhy+1+vyskaveci*spb+vyskaveci-1,tpvbarva);
                     _print(lhxs+3,lhy+3+vyskaveci*spb,tptxtbarva,su2^.jmeno+'.'+su2^.koncovka);
                     end;
    spb:=((ymys-lhy-1)div vyskaveci);{pozice souboru v okne}
    su2:=su;{prvni zobrazeny soubor}
    i:=1;
    if spb>0 then while (i<=spb)and(su2<>nil) do begin{nalezeni souboru, na ktery se kliklo}
                                                 su2:=su2^.dalsi;
                                                 inc(i);
                                                 end;
    if su2<>nil then begin{kdyz nebylo kliknuto do prazdne plochy}                     {oznaceni vybraneho souboru}
                     _bar(lhxs+1,lhy+1+vyskaveci*spb,lhxs+99,lhy+1+vyskaveci*spb+vyskaveci-1,tpozntxtpodbarva);
                     _print(lhxs+3,lhy+3+vyskaveci*spb,tpozntxtbarva,su2^.jmeno+'.'+su2^.koncovka);
                     if su2=su2t then oukej;{druhy klik na jeden soubor = OK}
                     end;
    kzap;
    resetuj;
    end;        {jestli byl vybran nejaky soubor, ukazuje na nej ted su2, jinak su2=nil}
  if adresareb then
    begin
    apos.kontrola;
    if lmys and mysvpoli(lhxa+1,lhy+1,lhxa+69,lhy+99) then
      begin
      apb:=((ymys-lhy-1)div vyskaveci);{apb=pozice adresare v okne}
      au2:=au;
      i:=1;
      if apb>0 then while (i<=apb)and(au2<>nil) do begin{nalezeni vybraneho adresare}
                                                   au2:=au2^.dalsi;
                                                   inc(i);
                                                   end;
      if au2<>nil then begin{kdyz byl nalezen...}
                       koncit:=true;{...ukonci se vnitrni cyklus...}
                       chdir(au2^.jmeno);{...a skoci se do vybraneho adresare}
                       end;            {po skonceni vnitrniho cyklu probehne znovu vnejsi, ktery vypise obsah noveho adresare}
      resetuj;
      end;
    if (apos.hodnota<>ap) then avypis;{posuvnik u adresaru}
    diskt.kontrola;
    if diskt.hodnota=1 then begin{tlacitko pro prepnuti disku}
                            ret:=vloztextokno(wt.dej(wtdisk2),1,cesta[1],true);
                            nastavdisk(ret[1]);
                            koncit:=true;
                            end;
    end;{if adresareb}
  okt.kontrola; cancelt.kontrola; novyt.kontrola; infot.kontrola;
  if okt.hodnota=1 then oukej;
  if cancelt.hodnota=1 then kancl;
  if novyt.hodnota=1 then begin
                          ret:=vloztextokno(wt.dej(wtjmenobezkoncovky),8,'',true);
                          if ret<>'' then begin{kdyz nebylo nic zadano, bere se to jako storno}
                                          if adresareb then vybersouborokno:=cesta+'\'+ret+'.'+koncovky
                                                       else vybersouborokno:=ret+'.'+koncovky;
                                          koncit:=true; koncit2:=true;
                                          end;
                          end;
  if infot.hodnota=1 then if su2=nil then infokno(xmys,ymys,wt.dej(wtmusitevybratsoubor),true)
                                     else info;
  if keypressed then begin
                     o:=readkey;
                     if o=#13 then oukej
                              else if o=#27 then kancl;
                     end;
  until koncit;{konec vnitrniho cyklu}
 zrusseznamsouboru(prvnisoubor);
 if adresareb then zrusseznamadresaru(prvniadresar);
 Until koncit2;{konec vnejsiho cyklu}
ok.zrus;{vymazani okna}
chdir(puvodnicesta);{skok do puvodniho adresare}
_poptext;
End;{vybersouborokno}

function vlozmaskuokno(nadpis:nadpstr;default:word):word;
var ok:okno;
    hs:hotspot;
    okt,cancelt,infot:tlacitko;
    i:byte;
    pole:array[0..15]of boolean;
    m,o,x,y:word;
{}procedure policko(ktere:byte);
{}Begin
{}kvyp;
{}if pole[ktere] then _box(x+11*ktere,y,x+11*ktere+9,y+10,10,8,2)
{}               else _bar(x+11*ktere,y,x+11*ktere+9,y+10,0);
{}kzap;
{}End;{policko}
Begin{vlozmaskuokno}
wokzruseno:=false;
x:=(_maxx shr 1)-100; y:=(_maxy shr 1)-30;
if nadpis='' then nadpis:=wt.dej(wtmaskacary);
m:=default; o:=0;
for i:=15 downto 0 do pole[i]:=((m shr (15-i)) and 1)=1;
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
_zmertext(nadpis);
ok.init(x-3,y-7-_vyskatextu,180,39+_vyskatextu,nadpis);
_pushtext; _nastavfont(wokfont,true);{kvuli tlacitkum}
okt.init(x,y+13,wt.dej(wtok14),wt.dej(wtvmoknookhelp),1);
cancelt.init(x+120,y+13,wt.dej(wtzrus),wt.dej(wtvmoknozrushelp),1);
infot.init(x+160,y+13,'?',wt.dej(wtvysvetlivky),1);
hs.init(x,y,x+11*16-1,y+10,false,'');
for i:=0 to 15 do policko(i);
resetuj;
 repeat
 cekej;
 hs.kontrola;
 okt.kontrola;
 cancelt.kontrola;
 infot.kontrola;
 case hs.hodnota of 1:begin
                      i:=hs.xkliku div 11;
                      pole[i]:=true;
                      policko(i);
                      end;
                    2:begin
                      i:=hs.xkliku div 11;
                      pole[i]:=false;
                      policko(i);
                      end;
                    end;
 if keypressed then o:=xreadkey;
 if (cancelt.hodnota=1)or(o=27) then begin
                                     vlozmaskuokno:=default;
                                     wokzruseno:=true;
                                     o:=1;
                                     end;
 if (okt.hodnota=1)or(o=13) then begin
                                 m:=0;
                                 for i:=15 downto 0 do begin
                                                       m:=m shl 1;
                                                       m:=m or ord(pole[i]);
                                                       end;
                                 vlozmaskuokno:=m;
                                 o:=1;
                                 end;
 if (infot.hodnota=1)or(o=xF1) then begin
                                    infokno(xmys,ymys,wt.dej(wtvmoknohelp),true);
                                    o:=0;
                                    end;
 until o=1;
ok.zrus; resetuj;
_poptext;
if dialogystandardnimfontem then _poptext;
End;{vlozmaskuokno}

procedure VlozVzorekOkno(nadpis:nadpstr; var vzorek:_pole8bytu);
var ok:okno;
    hs:hotspot;
    okt,cancelt,infot:tlacitko;
    i,j:byte;
    pole:array[0..7,0..7]of boolean;
    x,y,o:word;
{}procedure policko(px,py:byte);
{}Begin
{}kvyp;
{}if pole[py,px] then _box(x+11*px,y+11*py,x+11*px+9,y+11*py+9,10,8,2)
{}               else _bar(x+11*px,y+11*py,x+11*px+9,y+11*py+9,0);
{}kzap;
{}End;{policko}
Begin{vlozvzorekokno}
wokzruseno:=false;
if dialogystandardnimfontem then begin
                                 _pushtext;
                                 _nastavfont(wokfont,true);
                                 end;
if nadpis='' then nadpis:=wt.dej(wtvzorekvyplne);
o:=0;
for i:=0 to 7 do for j:=0 to 7 do pole[i,j]:=(vzorek[i] shl j)and $80=$80;
x:=(_maxx shr 1)-70; y:=(_maxy shr 1)-40;
_zmertext(nadpis);
ok.init(x-3,y-7-_vyskatextu,135,97+_vyskatextu,nadpis);
_pushtext; _nastavfont(wokfont,true);
okt.init(x+90,y+1,wt.dej(wtok4),wt.dej(wtvvoknookhelp),15);
cancelt.init(x+90,y+49,wt.dej(wtzrus),wt.dej(wtvvoknozrushelp),3);
infot.init(x+94,y+72,' ? ',wt.dej(wtvysvetlivky),0);
hs.init(x,y,x+11*8-1,y+11*8-1,false,'');
for i:=0 to 7 do for j:=0 to 7 do policko(j,i);
resetuj;
 repeat
 cekej;
 hs.kontrola;
 okt.kontrola;
 cancelt.kontrola;
 infot.kontrola;
 case hs.hodnota of 1:begin
                      j:=hs.xkliku div 11;
                      i:=hs.ykliku div 11;
                      pole[i,j]:=true;
                      policko(j,i);
                      end;
                    2:begin
                      j:=hs.xkliku div 11;
                      i:=hs.ykliku div 11;
                      pole[i,j]:=false;
                      policko(j,i);
                      end;
                    end;
 if keypressed then o:=xreadkey;
 if (okt.hodnota=1)or(o=13) then begin
                                 fillchar(vzorek,8,0);
                                 for i:=0 to 7 do
                                  for j:=0 to 7 do vzorek[i]:=vzorek[i] or ord(pole[i,j]) shl (7-j);
                                 o:=27;
                                 end;
 if cancelt.hodnota=1 then begin o:=27; wokzruseno:=true; end;
 if (infot.hodnota=1)or(o=xF1) then begin
                                    infokno(xmys,ymys,wt.dej(wtvvoknohelp),true);
                                    o:=0;
                                    end;
 until o=27;
ok.zrus; resetuj;
_poptext;
if dialogystandardnimfontem then _poptext;
End;{vlozvzorekokno}

procedure NastavMysOkno(ukladat:boolean);
var puvodnicx,puvodnicy,pomcx,pomcy:word;
    puvodnikb,puvodniotb,pomkb,pomotb:boolean;
    ok:okno; {okno, ve kterem to vsechno je}
    xo,yo:ciselnaosa; {posuvniky pro nastaveni vodorovne a svisle citlivosti}
    kz,oz:zatrzitko; {prepinace s korekci/bez korekce a normalne/pro levaky}
    testt,okulozt,okneulozt,zrust:tlacitko;
    koncit:boolean; {podminka ukonceni hlavniho cyklu}
    o:char; {pro klavesnici}
Begin
puvodnikb:=korekceb; puvodniotb:=obracenatlacitka;
zjisticitlivostmysi(puvodnicx,puvodnicy);
koncit:=false;
_pushtext; _nastavfont(wokfont,true); {nejsou tu zadne uzivatelske hlasky, tak pujdeme natvrdo standardnim fontem}
ok.init(_maxx shr 1-90,_maxy shr 1-90,180,175+ord(ukladat) shl 4,wt.dej(wtnastavenimysi));
with ok do
  begin
  xo.init(x+5,y2+24,133,1,100,puvodnicx,wt.dej(wtvodorovnacitlivost),wt.dej(wtvelka),wt.dej(wtmala),true,ovbarva);
   {Hodnota 100 nema zadny zvlastni vyznam, stejne dobre by tam mohlo byt neco
   vetsiho nebo i mensiho. Prakticky se stejne nikdy nebude nastavovat vetsi
   cislo nez cca 20 (to uz je docela hooodne pomaly kurzor).}
  yo.init(x+5,y2+75,133,1,100,puvodnicy,wt.dej(wtsvislacitlivost),wt.dej(wtvelka),wt.dej(wtmala),true,ovbarva);
  kz.init(x+5,y2+104,wt.dej(wtkorigovatskakani),wt.dej(wtkorskhelp),puvodnikb);
  oz.init(x+5,y2+117,wt.dej(wtvymenittlacitka),wt.dej(wtymentlachelp),puvodniotb);
  testt.init(x+3,y2+130,wt.dej(wtopatrnazkouska),wt.dej(wtopzkouskahelp),0);
  zrust.init(x+139,y2+130,wt.dej(wtzrus),wt.dej(wtnmoknozrushelp),0);
  okneulozt.init(x+3,y2+146,wt.dej(wtnastavaleneukladej),wt.dej(wtnmoknonnuhelp),0);
  if ukladat then okulozt.init(x+3,y2+162,wt.dej(wtnastavauloz),wt.dej(wtnmoknonuhelp)+cfgsoubor+wt.dej(wtnmoknonuhelp+1),0);
  end;
o:=#255; {neco bezpecneho, co se s nicim nepopere}
 repeat {hlavni uzivatelsky cyklus}
 cekej;
 xo.kontrola; yo.kontrola;
 kz.kontrola; oz.kontrola;
 testt.kontrola; zrust.kontrola;
 okneulozt.kontrola;
 if ukladat then okulozt.kontrola;
 if keypressed then o:=readkey;
 if (o=#0)and keypressed
   then case ord(readkey) of hsipka:begin
                                    if yo.hodnota<yo.max then inc(yo.hodnota);
                                    yo.zobraz;
                                    end;
                             dsipka:begin
                                    if yo.hodnota>yo.min then dec(yo.hodnota);
                                    yo.zobraz;
                                    end;
                             lsipka:begin
                                    if xo.hodnota>xo.min then dec(xo.hodnota);
                                    xo.zobraz;
                                    end;
                             psipka:begin
                                    if xo.hodnota<xo.max then inc(xo.hodnota);
                                    xo.zobraz;
                                    end;
                             end;
 if (zrust.hodnota=1)or(o=#27) then begin
                                    korekcemysi(puvodnikb);
                                    obracenatlacitka:=puvodniotb;
                                    citlivostmysi(puvodnicx,puvodnicy);
                                    koncit:=true;
                                    end;
 if (okneulozt.hodnota=1)or(o=#13) then begin
                                        korekcemysi(kz.hodnota);
                                        obracenatlacitka:=oz.hodnota;
                                        citlivostmysi(xo.hodnota,yo.hodnota);
                                        koncit:=true;
                                        end;
 if ukladat and (okulozt.hodnota=1)
   then begin
        korekcemysi(kz.hodnota);
        obracenatlacitka:=oz.hodnota;
        citlivostmysi(xo.hodnota,yo.hodnota);
        if uloznastavenimysi then koncit:=true
                             else koncit:=not anoneokno(wt.dej(nmoknochybaukl));
        end;
 if testt.hodnota=1 then begin
                         {ulozeni aktualnich hodnot:}
                         zjisticitlivostmysi(pomcx,pomcy);
                         pomkb:=korekceb; pomotb:=obracenatlacitka;
                         {nastaveni hodnot z posuvniku a kz:}
                         korekcemysi(kz.hodnota); obracenatlacitka:=oz.hodnota;
                         citlivostmysi(xo.hodnota,yo.hodnota);
                         {vyzkouseni kurzoru (behem funkce Anoneokno):}
                         if not anoneokno(wt.dej(wtnmoknootazka)) then
                           begin {nelibilo se => zpet k minulym hodnotam}
                           korekcemysi(pomkb);
                           obracenatlacitka:=pomotb;
                           citlivostmysi(pomcx,pomcy);
                           end;
                         end;
 until koncit;
resetuj;
ok.zrus;
_poptext;
End;{nastavmysokno}

procedure NastavKlOkno(popisky:uknaradek; var rozlozeni);
type PoleWordu=array[0..0] of word;
var zavrit,infot:tlacitko;
    rkz:zatrzitko;
    o,w,x,vt,pozice,maxpozice:integer;
    ok:okno;
    x2:integer;
{}function JmenoKlavesy(xcislo:word):string;{vraci jmeno klavesy podle kodu od Xreadkey (max. 9 znaku)}
{}Begin
{}case xcislo of 0:jmenoklavesy:='';{nedefinovana klavesa (nulu xreadkey nevraci)}
{}               8:jmenoklavesy:=wt.dej(wtbksp);
{}               9:jmenoklavesy:=wt.dej(wttab);
{}               13:jmenoklavesy:=wt.dej(wtenter);
{}               27:jmenoklavesy:=wt.dej(wtesc);
{}               32:jmenoklavesy:=wt.dej(wtmez);
{}               33..255:jmenoklavesy:=chr(xcislo);{bezne zobrazitelne znaky z ASCII tabulky}
{}               xlsipka:jmenoklavesy:=#27;{\                                 }
{}               xpsipka:jmenoklavesy:=#26;{ \znaky sipek z pocatku ASCII tabulky}
{}               xhsipka:jmenoklavesy:=#24;{ / (#27 ted nema nic spolecneho s kodem escapu)}
{}               xdsipka:jmenoklavesy:=#25;{/                                 }
{}               xins:jmenoklavesy:=wt.dej(wtins);
{}               xdel:jmenoklavesy:=wt.dej(wtdel);
{}               xhome:jmenoklavesy:=wt.dej(wthome);
{}               xendk:jmenoklavesy:=wt.dej(wtend);
{}               xpgup:jmenoklavesy:=wt.dej(wtpgup);
{}               xpgdn:jmenoklavesy:=wt.dej(wtpgdn);
{}               xf1..xf10:jmenoklavesy:='F'+nastr(xcislo-314);{F1..F10 jdou hezky za sebou, tak si trochu zkratim zapis}
{}               xf11:jmenoklavesy:='F11';
{}               xf12:jmenoklavesy:='F12';
{}               else jmenoklavesy:='#'+nastr(xcislo);{prece nemuzu vypsat uplne vsechno, ze jo...}
{}               end;
{}End;{jmenoklavesy}
{}procedure kresli(enterB:boolean);{prekresli okenko se seznamem akci}
{}var p:uknaradek;
{}    w,i:integer;
{ }procedure _radek(stav:byte);{vykresli jeden radek (stav: 0=neaktivni, 1=vybrana, 2=upravovana)}
{ }var textik:string[20];
{ }    bt,bp:byte;
{ }Begin
{ }case stav of 0:begin bt:=tptxtbarva; bp:=tpvbarva; end;
{ }             1:begin bt:=tpozntxtbarva; bp:=tpozntxtpodbarva; end;
{ }             2:begin bt:=nkatxtbarva; bp:=nkatxtpodbarva; end;
{ }             end;
{ }with ok do begin
{ }           _bar(x+3,y2+4+vt*i,x+sirka-3,y2+3+vt*(i+1),bp);
{ }           textik:=p^.obsah;
{ }           if byte(textik[0])>20 then textik[0]:=#20;
{ }           if not dialogystandardnimfontem then _lodtext;
{ }           _print(x+5,y2+5+vt*i,bt,textik);
{ }           if stav=2 then textik:='    ?'
{ }                     else textik:=jmenoklavesy(polewordu(rozlozeni)[w]);
{ }           if not dialogystandardnimfontem then _nastavfont(wokfont,true);
{ }           _print(x2,y2+5+vt*i,bt,'- '+textik);
{ }           end;
{ }End;{_radek}
{}Begin{kresli}
{}p:=popisky; w:=0;
{}if pozice>2 then while (w<maxpozice-4)and(w<pozice-2) do begin
{}                                                         inc(w);
{}                                                         p:=p^.dalsi;
{}                                                         end;
{}{ted p^ = prvni radek zobrazeny v okne, w = jeho cislo}
{}with ok do
{} begin
{} kvyp;
{} for i:=0 to 4 do
{}   begin
{}   if w<=maxpozice then if w=pozice then if enterb then _radek(2)
{}                                                   else _radek(1)
{}                                    else _radek(0)
{}                   else break;
{}   inc(w);
{}   p:=p^.dalsi;
{}   end;
{} kzap;
{} end;
{}End;{kresli}
Begin{nastavklokno}
if popisky=nil then exit;{kdyz neexistuje seznam akci, neni co resit}
maxpozice:=pocetradkuvseznamu(popisky)-1;
pozice:=0;
_pushtext;{uzivatelsky font je ulozen}
if dialogystandardnimfontem then _nastavfont(wokfont,true);
vt:=_vyskatextu2;{vyska uzivatelskeho fontu}
x2:=_sirkatextu2(20);{sirka popisku akci}
if x2<150 then x2:=150;{pojistka proti prilisnemu smrsknuti okna pri moc uzkem fontu}
_nastavfont(wokfont,true);
if _vyskatextu2>vt then vt:=_vyskatextu2;{vyska standardniho fontu}
inc(vt,2);{vyska radku = vyska vetsiho z obou fontu + pridavek 2 pixely}
ok.init((_maxx shr 1)-(x2 shr 1)-50,_maxy shr 1-45,x2+_sirkatextu2(12)+5,5*vt+38,wt.dej(wtkonfiguraceklaves));
with ok do
  begin
  inc(x2,x+8);{prepocet na absolutni souradnici (s pridavkem)}
  _box(x+2,y2+3,x+sirka-2,y2+4+5*vt,tplhbarva,tppdbarva,tpvbarva);
  infot.init(x+3,y+vyska-19,wt.dej(wtinfo),'',1);
  rkz.init(x+55,y+vyska-16,wt.dej(wtresitkonflikty),wt.dej(wtreskonflhelp),true);
  zavrit.init(x+sirka-57,y+vyska-19,wt.dej(wtzavrit),wt.dej(wtnkoknozavrithelp),1);
  end;
{je nastaven wokfont}
 repeat {vnejsi cyklus - pro vraceni na zacatek pri konfliktu klaves}
 o:=0;
 kresli(false);
 resetuj;
  repeat {vnitrni cyklus - hlavni}
  cekej;
  zavrit.kontrola; infot.kontrola; rkz.kontrola;
  if keypressed then
    begin
    o:=xreadkey;
    case o of xhsipka:if pozice>0 then dec(pozice);
              xdsipka:if pozice<maxpozice then inc(pozice);
              xhome:pozice:=0;
              xendk:pozice:=maxpozice;
              xpgup:if pozice>=5 then dec(pozice,5) else pozice:=0;
              xpgdn:if pozice<=maxpozice-5 then inc(pozice,5) else pozice:=maxpozice;
              13:begin
                 kresli(true);
                 o:=xreadkey;
                 if rkz.hodnota then {vymazani konfliktnich klaves:}
                  for w:=0 to maxpozice do if polewordu(rozlozeni)[w]=o then polewordu(rozlozeni)[w]:=0;
                 polewordu(rozlozeni)[pozice]:=o;{ulozeni nove klavesy}
                 o:=0;
                 end;
              end;
    kresli(false);
    end
   else o:=0;{not keypressed}
  if infot.hodnota<>0 then infokno(xmys,ymys,wt.dej(wtnkoknohelp),true)
  until (zavrit.hodnota=1)or(o=27);
 o:=27;
 {kontrola konfliktu klaves:}
 if maxpozice>0 then {pokud jsou aspon dve akce (jedna by nemela s cim kolidovat)}
   begin
   for w:=0 to maxpozice-1 do
    if polewordu(rozlozeni)[w]<>0 then
      for x:=w+1 to maxpozice do if polewordu(rozlozeni)[w]=polewordu(rozlozeni)[x] then begin
                                                                                         o:=10;{cokoli, jen ne 27}
                                                                                         break;
                                                                                         end;
   if (o<>27)and anoneokno('%%n'+wt.dej(wtpozor)+'%%N'+wt.dej(wtnkoknopozor)) then o:=27;
   end;
 until o=27;
ok.zrus;
_poptext;
End;{nastavklokno}

procedure InitWoknows(_font,_kurzor,_texty:boolean);
Begin
{standardni font:}
if _font then begin
              _nactifont2(wokfont,'msdflt.fnt');
              _nastavfont(wokfont,true);
              end;
{standardni kurzor mysi:}
if _kurzor then begin
                nactikurzor2(wokkurzor,'stdsipka.kur');
                nastavkurzor(wokkurzor);
                end;
{texty:}
if _texty then wt.init(woktexty,'','');
End;{initwoknows}

procedure ZrusWoknows;
Begin
if wt.pocetradku<>0 then wt.zrus;
_zrusfont(wokfont);
zrusmys;
zruskurzor(wokkurzor);
End;{zruswoknows}

BEGIN
wokfont.fdata:=nil;
wokkurzor.kdata:=nil;
wokzruseno:=false;
wt.pocetradku:=0;
END.